#ifndef PLOT_WINDOW_H
#define PLOT_WINDOW_H

#include <QMainWindow>
#include <QAction>
#include <QMenuBar>
#include <QSlider>
#include <QFileDialog>
#include <QDialog>
#include <QLayout>
#include <QLabel>
#include <QLineEdit>
#include <QComboBox>
#include <QPrintDialog>
#include <QPrinter>

#include "IntervalDialog.h"
#include "QtPLWidget.h"
#include "qprofile.h"

class PlotID;
class PlugPlotWidgets;

class PlotWindow : public QMainWindow
{
    Q_OBJECT
            
	public:
		PlotWindow(int plot_id, int width=500, int height=500, double saveWidth=600, double saveHeight=600, int resolution=72, QPrinter::PageSize pageSize=QPrinter::A4, QPrinter::Orientation orientation=QPrinter::Portrait, bool withGUI=false, QWidget* parent=NULL);
		
			QtPLWidget * plot;
                        
			void print_iden();
    
            void addWidget(QWidget *w, int r, int c, Qt::Alignment align, int numRows = 1);
            QSlider * addSlider(Qt::Orientation, int from, int to, int step);
    
			void setSmooth(bool b);
                        
			void setId(int plot_id);
                        
			int getId();
                        
			void setPLFocus();
                        
			void setSize(double width, double height, bool forceResize=false); 
			
			void setTitle(const QString& title);
			
			void fixAspectRatio(double &saveWidth, double &saveHeight);
			
			QPrinter::PageSize pageSize;
			QPrinter::Orientation orientation;
			int resolution; //dpi
			double weight; // used for print and vector graphics plots to weight all lines
			IntervalDialog * intervalDialog;
            QMenu * printWeightMenu;
            QMenu * compressionQuality;
			QString saveDirectory;
            QGridLayout * layout;
            QGridLayout *mainLayout;
            PlotID *ownerGUI;
            int percent;
            int qual;
			
	signals:
		
		void clicked(double x,double y, int pos_x, int pos_y); // interface between widget and window
		void doubleClicked(double x,double y, int pos_x, int pos_y); // interface between widget and window
		
		void rescaleScalar(double start, double end); // interface between intervalDialog and window
		void rescaleTime(DvEvent interval);
				
    public slots:
        
	void savePlotPNG(QString name=QString("")); // Functions for Mac
    void savePlotJPG(QString name=QString(""));
//	void savePlotPS();
	void savePlotPDF(QString name=QString(""));
	void savePlotSVG(QString name=QString(""));
    void setWeightP();
    void setWeightL();
	void setWeightM();
    void setWeightB();
    void setWeightH();
    void setWeightVH();
	void setPrintRes(int);
    void setElev(int value);
    void setAz(int value);
    void setQL(){qual = 5; compressionQuality->setTitle("JPG Quality (Low)");}
    void setQML(){qual = 30; compressionQuality->setTitle("JPG Quality (Medium Low)");}
    void setQM(){qual = 60; compressionQuality->setTitle("JPG Quality (Medium)");}
    void setQMH(){qual = 85; compressionQuality->setTitle("JPG Quality (Medium High)");}
    void setQH(){qual = 100; compressionQuality->setTitle("JPG Quality (High)");}
    void savePlotDirect(const char * saveName);
    void printPlotDirect();
    void toggleToolbar();
    
//         void savePlot();

	void printPlot();
	
	void changeDisplayedRatio(double r);
        
	protected:
            
        int plot_id;
            
        PLINT strm;
            
        double saveWidth, saveHeight;
        QPrinter::Orientation saveOrientation;
	    QString title;

        int extraLength;
        int extraBottom;
        int extraSide;
        int az;
        int elev;

        QToolBar *_file_tool_bar;
        QAction *pngAct;
//        QAction *psAct; 
        QAction *pdfAct;
        QAction *jpgAct;
        QAction *svgAct;
        QAction *printAct;
        QAction *printDirectAct;
        void createToolBars(); 
        void createActions();  
     	QAction* toolBar;


};

// ---------------------------------

class PlotID : public QComboBox
{
    Q_OBJECT
    
public:
    PlotID(const char* title, QList<PlotWindow*> *_plotWindows=NULL);
    ~PlotID(){
        if(delListOnClose) {
            for (int i=0; i<plotWindows->size(); i++) delete windowAt(i); 
            plotWindows->clear();
        }
    }
    
    bool IDused(int pID){
        for (int i=0; i<plotWindows->size(); i++) if((*plotWindows)[i]->getId() == pID) return true;
        return false;
    }
    
    DvString getName();
    
    void addWindow(PlotWindow*);
    int size(){return plotWindows->size();}
    PlotWindow * windowAt(int i){return (*plotWindows)[i];}
    void setPlotWindow (int index, PlotWindow *w){(*plotWindows)[index] = w;}
    int indexOf(int plid){
        for (int i=0; i<this->size(); i++) if((*plotWindows)[i]->getId() == plid) return i;
        return -1;
    }
    
    PlotWindow* newPlotWindow(double width, double height, int resolution=72, QPrinter::PageSize pageSize=QPrinter::A4, QPrinter::Orientation orientation=QPrinter::Portrait, bool withGUI=false, bool hidden=false);
    PlotWindow*   focusWindow(double width, double height, int resolution=72, QPrinter::PageSize pageSize=QPrinter::A4, QPrinter::Orientation orientation=QPrinter::Portrait, bool withGUI=false, bool hidden=false);

    PlotWindow *currentWindow;
    
public slots:
    void setWindow(int index);
    
protected:
    QList<PlotWindow*> *plotWindows;
    DvString title;
    bool delListOnClose;
};




#endif

