//  qops.h
//
//  Qsas - mid level operators
//
//  Tony Allen
//  Nov 2012
//
#ifndef QSAS_OPS_H_
#define QSAS_OPS_H_

#include <string.h>
#include <vector>

#include <QLineEdit>
#include <qtoolbutton.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qslider.h>
#include <qslot.h>
#include <qmenubar.h>
#include <qpushbutton.h>

#include "qt_version.h"

#include "qsas_version.h"
#include <QGroupBox>
#include <QMenu>
#include <QTreeView>
#include <QListWidget>
#include <QSpinBox>
#include "qsw_helpwindow.h"
#include "qdutil.h"
#include "floatspinbox.h"


class AngleDepWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  AngleDepWindow(int max);
  /// \fn AngleDepWindow()
  /// \brief Destructor of the AngleDepWindow class
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  virtual ~AngleDepWindow(){};
  void setObject(const char * WLctxt);

  QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)

  QMenuBar* menu_bar(){ return _menu_bar;};

  /// \fn QStatusBar* status_bar()
  /// \brief Function that returns the status bar pointer of the current main window object
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  QStatusBar* status_bar(){return msgBar;};
  QguWLinput *objField;
  QSpinBox *depField;
  QguWLinput *name_field;
  FloatSpinBox *newCentreFld;
  bool userSet;
  int top;
  bool newCentre;

public slots:
  void acceptOperation();
  void setUserInput();
  void setNewName();
  void setAccept();
  void dismiss();

protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};


class ArrayUnaryWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  ArrayUnaryWindow( int op);
  virtual ~ArrayUnaryWindow(){};
  void setObject(const char * WLctxt);

  QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)

  QMenuBar* menu_bar(){ return _menu_bar;};

  /// \fn QStatusBar* status_bar()
  /// \brief Function that returns the status bar pointer of the current main window object
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  QStatusBar* status_bar(){return msgBar;};
  QguWLinput *objField;
  QguWLinput *name_field;
  bool userSet;
  int opFlag;

public slots:
  void acceptOp();
  void setUserInput();
  void setNewName();

protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};

class ArrayBinaryWindow: public QMainWindow
{
    Q_OBJECT
    
protected:
    QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
    
public:
    ArrayBinaryWindow(int op);
    
    virtual ~ArrayBinaryWindow(){};
    void setObject(const char * WLctxt);
    
    QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)
    
    QMenuBar* menu_bar(){ return _menu_bar;};
    
    QStatusBar* status_bar(){return msgBar;};
    QguWLinput *objA;
    QguWLinput *objB;
    QguWLinput *name_field;
    int opFlag;
    
    public slots:
    void acceptOp();
    
protected:
    void keyPressEvent(QKeyEvent * e );
    QPushButton *saveBtn;
    QPushButton *dismissBtn;
};


class ArrayTriWindow: public QMainWindow
{
    Q_OBJECT
    
protected:
    QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
    
public:
    ArrayTriWindow(int op);
    
    virtual ~ArrayTriWindow(){};
    void setObject(const char * WLctxt);
    
    QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)
    
    QMenuBar* menu_bar(){ return _menu_bar;};
    
    QStatusBar* status_bar(){return msgBar;};
    QguWLinput *objA;
    QguWLinput *objB;
    QguWLinput *objC;
    QguWLinput *name_field;
    int opFlag;
    QLabel *lab_i;
    QLabel *lab_j;
    
    
    public slots:
    void acceptOp();
    
protected:
    void keyPressEvent(QKeyEvent * e );
    QPushButton *saveBtn;
    QPushButton *dismissBtn;
};




class RegressionWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  RegressionWindow( );

  virtual ~RegressionWindow(){};
  void setObject(const char * WLctxt);

  QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)

  QMenuBar* menu_bar(){ return _menu_bar;};

  /// \fn QStatusBar* status_bar()
  /// \brief Function that returns the status bar pointer of the current main window object
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  QStatusBar* status_bar(){return msgBar;};
  QguWLinput *objField;
  QguWLinput *name_field;
  bool userSet;

public slots:
  void acceptOp();
  void setUserInput();
  void setNewName();

protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};


class MergeTSWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  MergeTSWindow( QWidget *parent=0);
  virtual ~MergeTSWindow(){};
  void setObject(const char * WLctxt);

  QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)

  QMenuBar* menu_bar(){ return _menu_bar;};
  QStatusBar* status_bar(){return msgBar;};
  QguWLinput *objField1;
  QguWLinput *objField2;
  QguWLinput *name_field;

public slots:
  void merge();

protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *doBtn;
  QPushButton *dismissBtn;
};


/// \class MakeVectorWindow qdutil.h
///
/// This class specialises the QMainWindow class which provies a main application window.
///
/// Main windows are most often used to provide menus, toolbars and a status bar around a large central widget.
///
/// It inherits QMainWindow which is a built-in QT3 class
///
/// \remarks {
/// This class includes Q_OBJECT which is a QT3 macro that is used in all sub-classes of QObject in order to
/// benefit from the QT's Meta object system which are signals and slots mechanisms for inter-object communication,
/// runtime type information and the dynamic property system }

class MakeVectorWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  MakeVectorWindow( QWidget *parent=0);
  /// \fn ~MakeVectorWindow()
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  virtual ~MakeVectorWindow(){};
  void setObject(const char * WLctxt);

  QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)

  QMenuBar* menu_bar(){ return _menu_bar;};

  /// \fn QStatusBar* status_bar()
  /// \brief Function that returns the status bar pointer of the current main window object
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  QStatusBar* status_bar(){return msgBar;};
  QguWLinput *objField;
  QguWLinput *frame_field;
  QComboBox *rep_field;
  bool userSet;

public slots:
  void makeVector();
  void testArray();
  
protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *doBtn;
  QPushButton *dismissBtn;
};



class DerivativeWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  DerivativeWindow( );
  /// \fn ~DerivativeWindow()
  /// \brief Destructor of the DerivativeWindow class
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  virtual ~DerivativeWindow();
  void setObject(const char * WLctxt);

  QguWLinput *objField;
  QguWLinput *name_field;
  QComboBox *derivMethod;
  QguWLinput *gapSizeField;
  QLabel *methodLabl;
  bool userSet;

public slots:
  void doDerivative();
  void setUserInput();
  void setNewName();
  void setMethod(const QString &);
  void launchHelp();


protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};



class CreateRegularWindow: public QMainWindow
{
    Q_OBJECT
    
protected:
    QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
    
public:
    CreateRegularWindow();
    ~CreateRegularWindow(){;}
    
    QguWLinput *name_field;
    QComboBox *objType;
    QguWLinput *numberField;
    QguWLinput *spacingField;
    QguWLinput *startField;
    bool userSet;
    QLabel *unitLabl;
    
    public slots:
    void createObj();
    void setUserInput();
    void setType(const QString &);
    
    
protected:
    void keyPressEvent(QKeyEvent * e );
    QPushButton *saveBtn;
    QPushButton *dismissBtn;
};


// -----------------------------

class TTagToScalarWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  TTagToScalarWindow( QWidget *parent=0);
  virtual ~TTagToScalarWindow(){};
  void setObject(const char * WLctxt);

  QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)

  QMenuBar* menu_bar(){ return _menu_bar;};

  /// \fn QStatusBar* status_bar()
  /// \brief Function that returns the status bar pointer of the current main window object
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  QStatusBar* status_bar(){return msgBar;};
  QguWLinput *objField;
  QguWLinput *offsetField;
  QguWLinput *name_field;
  bool userSet;

public slots:
  void acceptOperation();
  void setUserName();
  void setUserOffset();
  void setNewOffset();
  void dismiss();

protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};

// -----------------------------

class ScalarToTTagWindow: public QMainWindow
{
    Q_OBJECT
    
protected:
    QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
    
public:
    ScalarToTTagWindow( QWidget *parent=0);
    virtual ~ScalarToTTagWindow(){};
    void setObject(const char * WLctxt);
    
    QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)
    
    QMenuBar* menu_bar(){ return _menu_bar;};
    
    QStatusBar* status_bar(){return msgBar;};
    QguWLinput *objField;
    QguWLinput *offsetField;
    QguWLinput *name_field;
    
    public slots:
    void acceptOperation();
    void dismiss();
    
protected:
    void keyPressEvent(QKeyEvent * e );
    QPushButton *saveBtn;
    QPushButton *dismissBtn;
};


// -----------------------------

class T1T2ToSecWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  T1T2ToSecWindow( QWidget *parent=0);
  virtual ~T1T2ToSecWindow(){};
  void setObjectStart(const char * WLctxt);
  void setObjectEnd(const char * WLctxt);

  QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)

  QMenuBar* menu_bar(){ return _menu_bar;};

  /// \fn QStatusBar* status_bar()
  /// \brief Function that returns the status bar pointer of the current main window object
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  QStatusBar* status_bar(){return msgBar;};
  QguWLinput *startField;
  QguWLinput *endField;
  QguWLinput *name_field;
  bool userSet;

public slots:
  void acceptOperation();
  void dismiss();

protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};

// -----------------------------

class TT1TT2ToSecWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  TT1TT2ToSecWindow( QWidget *parent=0);
  virtual ~TT1TT2ToSecWindow(){};
  void setObjectStart(const char * WLctxt);
  void setObjectEnd(const char * WLctxt);

  QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)

  QMenuBar* menu_bar(){ return _menu_bar;};

  /// \fn QStatusBar* status_bar()
  /// \brief Function that returns the status bar pointer of the current main window object
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  QStatusBar* status_bar(){return msgBar;};
  QguWLinput *startField;
  QguWLinput *endField;
  QguWLinput *name_field;
  bool userSet;

public slots:
  void acceptOperation();
  void dismiss();

protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};

// -----------------------------

class TIToSecWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  TIToSecWindow( QWidget *parent=0);
  virtual ~TIToSecWindow(){};
  void setObject(const char * WLctxt);

  QStatusBar* msgBar;       ///<  Message bar of the current main window object (also qsas window)

  QMenuBar* menu_bar(){ return _menu_bar;};

  /// \fn QStatusBar* status_bar()
  /// \brief Function that returns the status bar pointer of the current main window object
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  QStatusBar* status_bar(){return msgBar;};
  QguWLinput *objField;
  QguWLinput *name_field;
  bool userSet;

public slots:
  void acceptOperation();
  void dismiss();

protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};

// Integration here temporarily

class IntegralWindow: public QMainWindow
{
  Q_OBJECT

protected:
  QMenuBar *_menu_bar;      ///<  Menu bar of the current main window object
  
public:
  IntegralWindow( );
  /// \fn IntegralWindow()
  /// \brief Destructor of the IntegralWindow class
  /// \param     None
  /// \exception None
  /// \return    None
  ///
  virtual ~IntegralWindow();
  void setObject(const char * WLctxt);

  QguWLinput *objField;
  QguWLinput *name_field;
  QComboBox *integralMethod;
  QComboBox *gapMethod;
  QLabel *methodLabl;
  QLabel *gapMethodLabl;
  bool userSet;

public slots:
  void doIntegral();
  void setUserInput();
  void setNewName();
  void setMethod(const QString &);
  void setGapMethod(const QString &);
  void launchHelp();


protected:
  void keyPressEvent(QKeyEvent * e );
  QPushButton *saveBtn;
  QPushButton *dismissBtn;
};


# endif // QSAS_OPS_H_
