//  qsurface.h
//
//  Tony Allen
//  Oct 2010

#ifndef QSAS_SURFACE_UI_H_
#define QSAS_SURFACE_UI_H_

#include <qsplitter.h>
#include <qstatusbar.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qtabbar.h>
#include <qlayout.h>

#include <QApplication>
#include <QMimeData>
#include <QStyle>
#include <QHeaderView>
#include <QPainter>
#include <QScrollArea>

#include "qt_version.h"
#include "qslot.h"
#include "QbTimeEditors.h"
#include "IntervalDialog.h"

#include "floatspinbox.h"
#include "IntervalWidgets.h"
#include "ColourTable.h"

#include "qpui.h"
#include "PlotWindow.h"

#define QSURFACE_VERSION "1.0"


class polygon;


// class  QsurfaceWindow
// -----------------------

class QsurfaceWindow : public QMainWindow
{
	Q_OBJECT
 
 	// GUI widgets
	QTabWidget *tab_widget;
	QStatusBar* _status_bar;
	QTextEdit *textArea;
	
	// Data tab widgets
 	QguWLinput *dataSlot;
	RecordSlider *atSlider;
	QPushButton *animateBtn;
	QCheckBox * swapR;
	QCheckBox *autoPlot;
	QSlider *elevation;
	QSlider *azimuth;
	QSpinBox *delay;
	
	// plot tab widgets
	QLineEdit *titleSlot;
	QCheckBox * autoTitle;
	QLineEdit *RXLabSlot;
	QLineEdit *ThYLabSlot;
	QComboBox *plotType;
	QComboBox *axisType;
	FloatSpinBox *alphaSpin;
	
	QCheckBox *showCont;
	QCheckBox *showContLabel;
	QLineEdit * contSpace;
	FloatSpinBox *zOffset;

	QCheckBox *RXLog;
	QCheckBox *RXAuto;
	QLineEdit *RXMin;
	QLineEdit *RXMax;
	QCheckBox *YLog;
	QCheckBox *ThYAuto;
	QLineEdit *ThYMin;
	QLineEdit *ThYMax;
	QLabel * RXMinLab;
	QLabel * RXMaxLab;
	QLabel * ThYMinLab;
	QLabel * ThYMaxLab;
	QLabel * RaxisLab;
	QLabel * ThAxisLab;
	
	QCheckBox *colourLog;
	QCheckBox *colourAuto;
	QLineEdit *colMin;
	QLineEdit *colMax;
	ColourTableButton *colourTable;
	QLineEdit *colLabSlot;
	FloatSpinBox *title_size_fld;
	FloatSpinBox *text_size_fld;
	
	// page tab widgets
	QComboBox *page_size;
	QComboBox *pageUnits;
	int oldUnits;
	QComboBox *dpiPage;
	QLineEdit *pageWidth;
	QLineEdit *pageHeight;
	QComboBox *pageOrientation;
    PlotID *plotWin_id;
    QCheckBox *newWindow;
    FloatSpinBox *line_width_fld;
    FloatSpinBox *axis_width_fld;
	
	// flags and pointers
	bool isPolar;
	bool userTitle;
	bool userRLab;
	bool userThLab;
	bool userColLab;
	bool orderIsRTh;
	bool full; //indicates whether bins span over 360 degree
	bool dThetaSet;
	bool dRSet;
	bool zLogEmpty;
	bool isArcfill;
	bool escPushed;
	DvObject_var dobj;
	DvObject_var rDep;
	DvObject_var thetaDep;
	PLFLT **z;
	PLFLT **zLog;
	PLFLT *theta_plus;
	PLFLT *theta_minus;
	PLFLT *r_plus;
	PLFLT *r_minus;
	PLFLT *r;
	PLFLT *theta;
	PLFLT width;
	PLFLT height;
	PLFLT boxzmin;

    PlotWindow* win;
	int DPI;
	int oldDPI;
	double conv;
	bool oldLandscape;
	string timeText;

	// from ppp
	int nr; //length of r array
	int ntheta; //length of theta array
	int n; //total no. of data points = nr*ntheta
	int nr_start; // starting radial/x index (> userRmin)
	int nr_end; // end radial/x index (< userRmax)
	int nth_start; // starting y index (> userThmin)
	int nth_end; // end y index (< userThmax)
	double contStep;
	double ang2rad;
	double rad2deg;
	double rfillVal;
	double thetafillVal;
	double zfillVal;
	PLFLT zmin;
	PLFLT zmax;
	PLFLT rmax;
	PLFLT rmin;
	PLFLT rminNonZero;
	PLFLT thetaminNonZero;
	PLFLT userRmax;
	PLFLT userRmin;
	PLFLT userThmax;
	PLFLT userThmin;
	PLFLT rOffset;
	PLFLT thetamin;
	PLFLT thetamax;
	PLFLT room; //how much larger (factor) than the maximum radius of the plot should the 3d box be?
    PLFLT zroom;	//how much deeper, compared to zmax-zmin, should he box extend? (such that the contours are visible)
    PLFLT alpha;	//Alpha value for the background when mesh plot is chosen.
	PLFLT az;
    PLFLT el;
	PLcGrid2 cgrid;
	// --------
	
	public:
		QsurfaceWindow( QWidget *parent=0);
		~QsurfaceWindow();
        void saveAsXML(QString path);
        void restoreXML(QString path, bool replace=true);
		void setwindow(bool forceSameWindow=false);
		void setData();
		bool initData();
		void sort_r_theta();
		void quick_sort_2d(PLFLT xarray[], PLFLT z[], int start, int end, int length, PLFLT x_plus[], PLFLT x_minus[]);
		inline PLFLT min(PLFLT x, PLFLT y) { return (x < y ? x : y); }
		inline PLFLT max(PLFLT x, PLFLT y) { return (x > y ? x : y); }
		inline PLFLT SaferLog10(PLFLT x) { return (x < 1.0e-35) ? -35.0 : log10(x); }
		PLFLT near_nice_number(PLFLT x);
		PLFLT nice_min(PLFLT zmin, PLFLT stepsize);
//		void shade_cont() ;
//		void arcfill();
//		int colscale();
		void drawcolscale();
		int cont_lvl(PLFLT clevel[], int nmax=100);
		void select_vpor(int n);
		bool getDimensions(int &nx, int &ny);
		void QsurfaceAppendTextDisplay(const char * txt);
		void clearTextArea();
		void setColLab();
		void setAxisLabels();
		void nonZeroMinMax();
		void getMinMax();
		bool setDeltaTheta(PLFLT *result, const char *delta);
		bool setDeltaRad(PLFLT *result, const char *delta);
		QDomElement toXMLElement(QDomDocument&) const;
		void fromXMLElement(QDomElement&);
		void setGUItoPolar(bool isPolar);
		void draw_angle(PLFLT offset=1.);
		void draw_polar_axis();
		void draw_cont_polar();
		void doPlotPolar();
		void doPlotCartesian();
		void bin_histogram_polar();
		void plot_hist_bar3(vector <polygon*> &pol, bool withEdges, int nEntries);
		void surf_polar();
		void draw_cont_cartesian();
		void surf_cartesian();
		void bin_histogram_cartesian();
        QString _filename;        ///< Filename used to save the current session or working list content
        QString SAVE_EXTENSION;   ///< File extension used to save the current session or working list content
		
	public slots:
		void status(const QString& str) {_status_bar->showMessage(str,2000);};
		void doPlot(bool sameWindow=false);
		void doRun();
		void doAnimate();
        void saveAsXML();
        void restoreXML();
		void setDataObj();
		void setRecord(bool andPlot=true);
		void launchManual();
		void launchSurfacePlotHelp();
		void launchVersion();
		void autosetCol();
		void autosetRX();
		void autosetThY();
		void setUserTitle();
		void setUserRXLab();
		void setUserThYLab();
		void setColUser();
		void setSwapRTheta();
		void setLogRX();
		void setLogY();
		void setLogCol();
		void setAutoRX();
		void setAutoThY();
		void changeRX();
		void changeThY();
		void setUserColLab();
		void setPage();
		void changeView();
		void setAxisOpt();
		void setContours(bool);
		void setPlotType(int);
		void stepAnimate();
		void stopAnimate();
		void setTitle();
        void saveSession(QString path){saveAsXML(path+"/saveSet.qx_surface_plot");}
        void restoreSession(QString path){restoreXML(path+"/saveSet.qx_surface_plot");}
		
	signals:
		void reValidateData();

	protected:
		void keyPressEvent(QKeyEvent * e );
		QPushButton *plotBtn;
		QPushButton *runBtn;
		QPushButton *dismissBtn;
        int startElevation;
        int startAzimuth;
        int startSlider;
};



void QSurfaceMakeWindow(QWidget*,const char*);
void QsurfaceShow();

//A class of x, y, z vector coordinates with operators overloaded as multiply, dot product, cross product etc.
class threevector
{
private:
	PLFLT xcoord, ycoord, zcoord; // Private data members
	PLFLT fxyz;

public:
// Default constructor
	threevector()
	{
		xcoord = 0.0;
		ycoord = 0.0;
		zcoord = 0.0;
 	}

// Cartesian constructor
	threevector(PLFLT x, PLFLT y, PLFLT z)
	{
		xcoord = x;
		ycoord = y;
		zcoord = z;		
		fxyz = 0;
	}

	threevector(PLFLT x, PLFLT y, PLFLT z, PLFLT f)
	{
		xcoord = x;
		ycoord = y;
		zcoord = z;
		fxyz = f;
	}

//Spherical polar constructor
	threevector(PLFLT r, PLFLT theta, PLFLT phi, char l)
	{
		if (l == 'd')
		{
			theta *= M_PI/180;
			phi *= M_PI/180;		//converting degree to radian
		}

		xcoord = r*sin(theta)*cos(phi);
		ycoord = r*sin(theta)*sin(phi);
		zcoord = r*cos(theta);
		fxyz = 0;
	}

	threevector(PLFLT r, PLFLT theta, PLFLT phi, PLFLT f, char l)
	{
		if (l == 'd')
		{
			theta *= M_PI/180;
			phi *= M_PI/180;		//converting degree to radian
		}

		xcoord = r*sin(theta)*cos(phi);
		ycoord = r*sin(theta)*sin(phi);
		zcoord = r*cos(theta);
		fxyz = f;
	}

//Access methods for x, y, and z coord
	PLFLT getx()
	{
		return xcoord;
	}

	PLFLT gety()
	{
		return ycoord;
	}

	PLFLT getz()
	{
		return zcoord;
	}

	PLFLT getf()
	{
		return fxyz;
	}

//Access methods for r, theta and phi
	PLFLT getr()
	{
		return mag3();
	}

	PLFLT gettheta()
	{
		PLFLT i = acos(zcoord/mag3());
		return i;
	}

	PLFLT getphi()
	{
		PLFLT i = atan2(ycoord, xcoord);
		return i;
	}

//Accsess methods square and mag3
	PLFLT square()					//Returns three vector square magnitude
	{
		PLFLT sq = xcoord*xcoord + ycoord*ycoord + zcoord*zcoord;
		return sq;
	}

	PLFLT mag3()					//Returns three vector magnitude
	{
		PLFLT mag = sqrt(square());
		return mag;
	}

//Modifier Methods for x y z
	void setx(PLFLT value)
	{
		xcoord = value;
	}

	void sety(PLFLT value)
	{
		ycoord = value;
	}

	void setz(PLFLT value)
	{
		zcoord = value;
	}

//Overloading operators
	threevector operator+(threevector v_other)	//adding two vectors
	{
		threevector v_sum(v_other.getx() + xcoord, v_other.gety() + ycoord, v_other.getz() + zcoord);
		return v_sum;
	}

	void operator+=(threevector v_other)	//+= for two vectors by modifying x, y, and z directly instead of returning a value.
	{
		xcoord += v_other.getx();
		ycoord += v_other.gety();
		zcoord += v_other.getz();
	}

	threevector operator-(threevector v_other) //subtracting two vectors
	{
		threevector v_dif(xcoord - v_other.getx(), ycoord - v_other.gety(), zcoord - v_other.getz());
		return v_dif;
	}

	threevector operator*(PLFLT value)	//multiplying a vector by a scalar, returning a vector.
	{
		threevector v_product(xcoord*value, ycoord*value, zcoord*value);
		return v_product;
	}

	PLFLT operator*(threevector v_other)	//dot product: multiplying two vectors.
	{
		PLFLT v_dot = xcoord*v_other.getx() + ycoord*v_other.gety() + zcoord*v_other.getz();
		return v_dot;
	}

	threevector operator/(PLFLT value)	//dividing a vector by a scalar.
	{
		threevector v_div(xcoord/value, ycoord/value, zcoord/value);
		return v_div;
	}

	threevector operator^(threevector v_other)	//Overloading ^ as the Cross Product
	{
		threevector v_cross(
			ycoord*v_other.getz() - zcoord*v_other.gety(),
			zcoord*v_other.getx() - xcoord*v_other.getz(),
			xcoord*v_other.gety() - ycoord*v_other.getx()
			);
		return v_cross;
	}
	void operator=(threevector v_other)	//Overloading ^ as the Cross Product
	{
		xcoord = v_other.getx();
		ycoord = v_other.gety();
		zcoord = v_other.getz();
	}
	
	PLFLT getdepth(PLFLT el, PLFLT az){
		az = atan2(sin(az), cos(az));				//This forces az to be between -pi to pi radians.

		return xcoord*sin(az)*cos(el) + ycoord*cos(az)*cos(el) - zcoord*sin(el);
	}


};


//A triangle object contains three points (threevector class) in space, as well as the depth of each point from the observer.
//Triangles are special because all three points lie on a plane hence has well defined normal vector. Should keep this class different from the polygon class.

class triangle {
private:
	threevector a, b, c;
	PLFLT da, db, dc;
	int i, edge_index;						//edge is an indicator of whether the triangle is situated at the end of all bins.

public:
	triangle(){};
	triangle(threevector a1, threevector b1, threevector c1, PLFLT el, PLFLT az) {
		edge_index = 0;
		
		// convert angle deg to rad
		a = a1;
		b = b1;
		c = c1;

		// This defines the plane parallel to the window. 
		// Not quite. Because z(x,y) is 1 to 1 function we only need to sort the elements by depth projected on the x-y plane.
		threevector nhat(cos(el) * sin(az), cos(el) * cos(az), 0 /*-sin(el)*/);

		da = (a /*- corner*/) * nhat;
		db = (b /*- corner*/) * nhat;
		dc = (c /*- corner*/) * nhat;
	}

	threevector get0(){
		return a;
	}
	threevector get1(){
		return b;
	}
	threevector get2(){
		return c;
	}

	PLFLT getxydepth() {
		return max(da,max(db, dc));
	}
	PLFLT getxydepth(int i)
	{
		if (i == 0)
				return da;
		if (i == 1)
				return db;
		if (i == 2)
			return dc;
		else
			return getxydepth();
	}

	PLFLT val() {
		PLFLT average = (a.getz() + b.getz() + c.getz())/3.;
		return average;
	}
	void setedge(int n){
		edge_index = n;
	}
	int edge(){
		return edge_index;
	}
	void operator=(triangle tri_other)
	{
		a = tri_other.get0();
		b = tri_other.get1();
		c = tri_other.get2();
		da = tri_other.getxydepth(0);
		db = tri_other.getxydepth(1);
		dc = tri_other.getxydepth(2);
		edge_index = tri_other.edge();
	}
};

// -------------------

//Just like triangle class, but with more points.
class polygon {
private:
	threevector *vert;
	PLFLT *xydepth, *zdepth;
	int i, pts, edge_index;						//edge is an indicator of whether the polygon is situated at the end of all bins.

public:
	polygon(){
		// default empty constructor
		xydepth = 0;
		zdepth = 0;
		vert = 0;
	};
	polygon(threevector verticies[], int pt, PLFLT el, PLFLT az) {
		edge_index = 0;
		pts = pt;
		
		xydepth = new PLFLT[pts];
		zdepth = new PLFLT[pts];
		vert = new threevector[pts];

		threevector n(sin(az), cos(az), 0);
		threevector z(0, 0, -1);

		for (i = 0; i < pts; i++) {

			vert[i] = verticies[i];

			xydepth[i] = n*vert[i];
			zdepth[i] = n*vert[i];
		}
	}
	polygon(PLFLT xpts[], PLFLT ypts[], PLFLT zpts[], int pt, PLFLT el, PLFLT az){
		edge_index = 0;
		pts = pt;
		
		xydepth = new PLFLT[pts];
		zdepth = new PLFLT[pts];
		vert = new threevector[pts];

		threevector n(sin(az), cos(az), 0);
		threevector z(0, 0, -1);

		for(i=0; i<pts; i++){
			threevector temp_constructor(xpts[i], ypts[i], zpts[i]);
			vert[i] = temp_constructor;
			xydepth[i] = n*vert[i];
			zdepth[i] = n*vert[i];
		}

	}
	~polygon(){
		if(vert != 0) delete[] vert;
		if(zdepth != 0) delete[] zdepth;
		if(xydepth != 0) delete[] xydepth;
	};

	PLFLT getzdepth() {
		return arrmax(zdepth, pts);
	}
	PLFLT getxydepth() {
		return arrmax(xydepth, pts);
	}
	threevector getvert(int i) {
		return vert[i];
	}
	PLFLT getzdepth(int i){
		return zdepth[i];
	}
	PLFLT getxydepth(int i){
		return xydepth[i];
	}
	int getpts() {
		return pts;
	}
	PLFLT val() {
		PLFLT average = 0;

		for (i = 0; i < pts; i++) {
			average += vert[i].getz();
		}
		average /= pts;
		return average;
	}
	void setedge(int n){
		edge_index = n;
	}
	int edge(){
		return edge_index;
	}
	void operator=(polygon p_other)
	{
		pts = p_other.getpts();
		vert = new threevector[pts];
		zdepth = new PLFLT[pts];
		xydepth = new PLFLT[pts];
		edge_index = p_other.edge();

		for(i=0; i<pts; i++)
		{
			vert[i] = p_other.getvert(i);
			zdepth[i] = p_other.getzdepth(i);
			xydepth[i] = p_other.getxydepth(i);
		}
	}
};

template <class T>
void quicksortpoly(vector <T*> &pol, int start, int end);





# endif // QSAS_SURFACE_UI_H_
