#ifndef QCALC_CALC_ITEM_H
#define QCALC_CALC_ITEM_H

#include "qcalcCommon.h"
#include "qcalcGraphicElements.h"

// Generic virtual calculator item: input, output or operator
class VCalcItem
{
	public:
		// Basic initialisation. Registers itself to VCalcItem::currentItems
		VCalcItem();
		
		// deletes its slots. Unregisters itself from VCalcItem::currentItems
		virtual ~VCalcItem();
		
		// Returns the list of connected items via its parents (items connected TO it) and or via its children (connected FROM it)
		QList<VCalcItem*> connectedItems(bool parents=false, bool children=true) const;
		
		// Checks if is a parent of another item. Used to prevent loops.
		bool isParentOf(VCalcItem* i) const;
		
		// Connects to the "target" item's inNumber-th VInSlot, via its outNumber-th VOutSlot // TODO still used?
// 		bool setConnection(int outNumber, VCalcItem* target, int inNumber);
		
		// Re-evaluates this object's output slots
		virtual void reevaluateOutSlots(){}

        inline DvObject_var getNotJoinedVal(){return not_joined_value;}
		
		// Mark the output slots need reevaluation. Spreads recursively to the children items.
		void flagOutSlotsForReevaluation();
		
		// The name is what's displayed on its GUI counterpart
		inline QString getName(){return name;}
		inline void setName(const QString& name)
		{
			this->name=name;
		}

        virtual QDomElement toXMLElement(QDomDocument&) const
        {
            return QDomElement();
        }

        virtual void fromXMLElement(QDomElement& el, QGraphicsScene* scene);
		
		// In case it belongs to a black box, its parent black box, NULL else
		VOperatorItem* container;

        bool isBlackBox; // true if and only if it is a black box
		
		static int calc_counter; // Used to create proper indexes
		int calc_id; // id for the calc item
		int temp_id; // used only when saving/restoring a session
		
		inline virtual bool hasGui(){return false;}
		inline virtual bool isSelected()const {return selected;};
		
		inline virtual void setSelected(bool sel=true)
		{
			selected=sel;
		}
		
// 		virtual void createMissingConnectors(QGraphicsScene* scene){}
		
		bool selected;
		QPointF positionInBlackBox; // To be able to do something nice when unwrapping a black box
		
		int page; // Page the item belongs to
		
		QVector<VInSlot*> inputs; // Set of input slots. Possibly none, for an InputItem for example
		QVector<VOutSlot*> outputs; // Set of output slots
		
	protected:
		QString name;
        DvObject_var not_joined_value;

};


// Generic GUI item for an input, an output, an operator...
class CalcItem: public QObject, public QGraphicsRectItem
{
    Q_OBJECT
            
	public:
		
		CalcItem(QGraphicsItem* parent);
		
		virtual ~CalcItem(){;}
		
		virtual void paint(QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget=NULL);
		
		// The help string that is displayed in the item when hovering a slot for example
		virtual void setDisplayedString(const QString&){}
		
		inline void setTitle(const QString& title)
		{
			this->title=title;
			update();
		}
		
		void resetTooltips();
		
		// updates the inbound and outbound connectors' graphics
		virtual void updateConnectorsPositions();
		
		// Displays an error message, if err is true (red icon appearing)
		void setErrMessage(bool err=false, const QString& message="");

        virtual QDomElement toXMLElement(QDomDocument& doc) const;

        virtual void fromXMLElement(QDomElement& el);
		
		virtual int page(){return -1;}
		
		// Change those in the children classes to specify their colours
		QColor outlineColor;
		QColor color1;
		QColor color2;
        
	protected:
		double width, height;
		QString title;
		RotoZoomButton* rzButton;
		CloseButton* closeButton;
		ErrorButton* errButton;
		
		virtual QVariant itemChange(GraphicsItemChange change, const QVariant& value);
		
		QList<InSlot*> inSlots;
		QList<OutSlot*> outSlots;
};

#endif
