#include "qcalcWindow.h"
#include <QVBoxLayout>
#include <QPushButton>
#include "qcalcSlots.h"
// #include "qcalcTimeSlots.h"
#include "qcalcCalcItem.h"
#include "qcalcInputItem.h"
#include "qcalcOperatorItem.h"
#include "qcalcOutputItem.h"
#include "qcalcExtractionItem.h"
#include "qcalcJoinItem.h"
// #include "qcalcTimeLineItem.h"

#include <QMdiArea> 
#include <QMdiSubWindow>

int VInputItem::counter=0;
int VExtractionItem::counter=0;
int VOutputItem::counter=0;
int VOperatorItem::counter=0;
// int VTimeLineItem::counter=0;
int VCalcItem::calc_counter=0;
int VJoinItem::counter=0;

extern qwuiWindow* qsasWindow;

CalcWindow *QCALC_Proto_widget;

 QcalcFeedback::QcalcFeedback()
 	: QTextEdit()
{
}
	
void QcalcFeedback::closeEvent( QCloseEvent* ce)
{     
	ce->ignore();
}

CalcWindow::CalcWindow(QWidget* parent, const char* Title):
	QMainWindow()
{
	
    QSplitter * vSplitter=new QSplitter(Qt::Vertical);
	
	QMdiArea* area=new QMdiArea();
	area->setOption(QMdiArea::DontMaximizeSubWindowOnActivation);

	myScene=new CalcScene(area);

	//DO NOT define a rect for the view. Else, it will be of fixed size. Not defining one allows it to stretch
	// Initialise it to a proper size by adding an object in it
	QGraphicsLineItem* line=new QGraphicsLineItem(-1000,-1000,1000,1000);
	myScene->addItem(line);

	myView=new CalcView(myScene);
	QMdiSubWindow * viewArea=area->addSubWindow(myView);
	viewArea->setWindowFlags(Qt::FramelessWindowHint);
	viewArea->setWindowState(Qt::WindowMaximized);
    feedbackArea=new QcalcFeedback();
	QMdiSubWindow * feedArea=area->addSubWindow(feedbackArea, Qt::CustomizeWindowHint|Qt::WindowMinimizeButtonHint|Qt::WindowTitleHint|Qt::WindowStaysOnTopHint);
	feedArea->setWindowTitle("Feedback area");
	feedArea->move(0, 280);
	feedArea->resize(620,200);
	 vSplitter->addWidget(area);
	 QWidget* hWidget=new QWidget;
	 QHBoxLayout* hLayout=new QHBoxLayout;
	 hLayout->setContentsMargins(1,1,1,1);
	QPushButton* button=new QPushButton("Evaluate all");
	hLayout->addWidget(button);
	connect(button, SIGNAL(clicked()), this, SLOT(evaluate()));
	
	button2=new QPushButton("Evaluate page");
	hLayout->addWidget(button2);
	connect(button2, SIGNAL(clicked()), this, SLOT(evaluatePage()));
	button2->setDefault(true);
	
	QGroupBox * pageBox=new QGroupBox("Page");
	pageBox->setMaximumWidth(180);
	QHBoxLayout * pageLayout=new QHBoxLayout;
	pageLayout->setContentsMargins(1,1,1,1);
	QPushButton* ppage=new QPushButton("<<");
	pageLayout->addWidget(ppage);
	connect(ppage, SIGNAL(clicked()), this, SLOT(previousPage()));
	
	pageLine=new QLineEdit;
	pageLine->setValidator(new QIntValidator(1,20,pageLine));
	pageLine->setText(QString("%1").arg(CalcScene::currentPage));
	pageLine->adjustSize();
	pageLayout->addWidget(pageLine);
	connect(pageLine, SIGNAL(textChanged(const QString&)), this, SLOT(changePage(const QString&)));
	
	QPushButton* npage=new QPushButton(">>");
	pageLayout->addWidget(npage);
	connect(npage, SIGNAL(clicked()), this, SLOT(nextPage()));
	pageBox->setLayout(pageLayout);
	
	hLayout->addWidget(pageBox);
	hWidget->setLayout(hLayout);
	vSplitter->addWidget(hWidget);
	
	
    contextWidget=new ContextWidget(this, myView);
	QMdiSubWindow * contextArea=area->addSubWindow(contextWidget, Qt::CustomizeWindowHint|Qt::WindowMinimizeButtonHint|Qt::WindowTitleHint|Qt::WindowStaysOnTopHint);
	contextArea->setWindowTitle("Components and Timeline");
	contextArea->move(0, 0);

	
	setCentralWidget(vSplitter);

	
	setWindowTitle(Title);

    connect(myScene, SIGNAL(selectionChanged()), this, SLOT(updateContextWidget()));
    connect(myScene, SIGNAL(hadDoubleClick()), this, SLOT(raiseContextWidget()));
	connect(myScene, SIGNAL(refreshContextWidget()), this, SLOT(clearContextWidget()));
    connect(myScene, SIGNAL(trackingStarted()), this, SLOT(trackingMessage()));
    connect(myScene, SIGNAL(selectTimeline(const QString&)), this, SLOT(selectTimeline(const QString&)));

    QMenu* fileMenu=menuBar()->addMenu("File");
    fileMenu->addAction("Save", this, SLOT(save()), Qt::CTRL+Qt::Key_S);
	fileMenu->addAction("Save As...", this, SLOT(saveAs()));
	fileMenu->addAction("Export Selection...", this, SLOT(exportSelection()), Qt::CTRL+Qt::Key_E);
	fileMenu->addSeparator();
    fileMenu->addAction("Include...", this, SLOT(include()), Qt::CTRL+Qt::Key_I);
	fileMenu->addAction("Restore...", this, SLOT(restore()), Qt::CTRL+Qt::Key_R);
	QMenu* editMenu=menuBar()->addMenu("Edit");
	editMenu->addAction("Cut", this, SLOT(cut()), Qt::CTRL+Qt::Key_X);
	editMenu->addAction("Copy", this, SLOT(copy()), Qt::CTRL+Qt::Key_C);
	editMenu->addAction("Paste", this, SLOT(paste()), Qt::CTRL+Qt::Key_V);
	
	QAction *delAct = editMenu->addAction("Delete", myScene, SLOT(deleteSelection()));
	QList<QKeySequence> keyList;
    keyList.append(Qt::Key_Delete);
    keyList.append(Qt::Key_Backspace);
    delAct->setShortcuts(keyList);
	
	editMenu->addSeparator();
	editMenu->addAction("Select all", myScene, SLOT(selectAll()), Qt::CTRL+Qt::Key_A);
	editMenu->addAction("Clear page", myScene, SLOT(clearPage()));
	editMenu->addAction("Clear all", myScene, SLOT(clearAll()));
	QMenu* addMenu=menuBar()->addMenu("Add");
	connect(addMenu, SIGNAL(aboutToShow()), myView, SLOT(autoSelectPosition()));
	addMenu->addAction("Input", myView, SLOT(addInputItem()));
	addMenu->addAction("Extractor", myView, SLOT(addExtractionItem()));
	addMenu->addAction("Join Setup", myView, SLOT(addJoinItem()));
	QMenu* addOp=addMenu->addMenu("Operator");
	QMenu* binOp=addOp->addMenu("Arith f(A,B)");
	binOp->addAction("A+B", myView, SLOT(addOpAdd()));
	binOp->addAction("A-B", myView, SLOT(addOpSub()));
	binOp->addAction("A*B", myView, SLOT(addOpMul()));
	binOp->addAction("A/B", myView, SLOT(addOpDiv()));
	binOp->addAction("A^B", myView, SLOT(addOpPow()));
	binOp->addAction("A\%B", myView, SLOT(addOpRemainder()));
	QMenu* unOp=addOp->addMenu("Arith f(A)");
	unOp->addAction("abs(A)", myView, SLOT(addOpAbs()));
	unOp->addAction("ln(A)", myView, SLOT(addOpLog()));
	unOp->addAction("log(A)", myView, SLOT(addOpLog10()));
	unOp->addAction("1/A", myView, SLOT(addOpInverse()));
	unOp->addAction("-A", myView, SLOT(addOpOpp()));
	unOp->addAction("sqrt(A)", myView, SLOT(addOpSqrt()));
	unOp->addAction("exp(A)", myView, SLOT(addOpExp()));
	QMenu* trigOp=addOp->addMenu("Trig");
	trigOp->addAction("cos(A)", myView, SLOT(addOpCos()));
	trigOp->addAction("sin(A)", myView, SLOT(addOpSin()));
	trigOp->addAction("tan(A)", myView, SLOT(addOpTan()));
	trigOp->addAction("cot(A)", myView, SLOT(addOpCot()));
	trigOp->addAction("acos(A)", myView, SLOT(addOpAcos()));
	trigOp->addAction("asin(A)", myView, SLOT(addOpAsin()));
	trigOp->addAction("atan(A)", myView, SLOT(addOpAtan()));
	trigOp->addAction("atan2(A,B)", myView, SLOT(addOpAtan2()));
	trigOp->addAction("acot(A)", myView, SLOT(addOpAcot()));
	trigOp->addAction("cosh(A)", myView, SLOT(addOpCosh()));
	trigOp->addAction("sinh(A)", myView, SLOT(addOpSinh()));
	trigOp->addAction("tanh(A)", myView, SLOT(addOpTanh()));
	trigOp->addAction("coth(A)", myView, SLOT(addOpCoth()));
	trigOp->addAction("acosh(A)", myView, SLOT(addOpAcosh()));
	trigOp->addAction("asinh(A)", myView, SLOT(addOpAsinh()));
	trigOp->addAction("atanh(A)", myView, SLOT(addOpAtanh()));
	trigOp->addAction("acoth(A)", myView, SLOT(addOpAcoth()));
	trigOp->addAction("(-180,180) to (0,360)", myView, SLOT(addOpAngle360()));
	trigOp->addAction("(0,360) to (-180,180)", myView, SLOT(addOpAngle180()));
	trigOp->addAction("To Radians", myView, SLOT(addOpToRad()));
	trigOp->addAction("To Degrees", myView, SLOT(addOpToDeg()));
    QMenu* vectOp=addOp->addMenu("Vector Ops");
    vectOp->addAction("A.B", myView, SLOT(addOpDotProd()));
    vectOp->addAction("AxB", myView, SLOT(addOpVectProd()));
    vectOp->addAction("Normalize", myView, SLOT(addOpNormalize()));
    vectOp->addAction("outer product", myView, SLOT(addOpOuter()));
    myView->addPluginMenu(vectOp, "Vectors", true);
    QMenu* MatOp=addOp->addMenu("Array Ops");
    MatOp->addAction("Matrix Inverse", myView, SLOT(addOpMatInv()));
    MatOp->addAction("trace(A)", myView, SLOT(addOpTrace()));
    MatOp->addAction("inner product", myView, SLOT(addOpInner()));
    MatOp->addAction("outer product", myView, SLOT(addOpOuter()));
    MatOp->addAction("transpose(A)", myView, SLOT(addOpTranspose()));
    MatOp->addAction("determinant(A)", myView, SLOT(addOpDeterminant()));
    MatOp->addAction("adjoint(A)", myView, SLOT(addOpAdjoint()));
    MatOp->addAction("element(i)", myView, SLOT(addOpElement_i()));
    MatOp->addAction("element(i,j,k...)", myView, SLOT(addOpElement_ijk()));
    MatOp->addAction("remove(i)", myView, SLOT(addOpRemove_i()));
    MatOp->addAction("A(i,j) * B(i,j) -> C(i,j)", myView, SLOT(addOpPiecewise()));
    MatOp->addAction("A(i,j) / B(i,j) -> C(i,j)", myView, SLOT(addOpPwDiv()));
    QMenu* derivOp=addOp->addMenu("Derivative");
	derivOp->addAction("3pt derivative", myView, SLOT(addOpDiff3()));
	derivOp->addAction("5pt derivative", myView, SLOT(addOpDiff5()));
	QMenu* integOp=addOp->addMenu("Integral");
	integOp->addAction("definite integral", myView, SLOT(addOpIntDef()));
	integOp->addAction("indefinite integral", myView, SLOT(addOpIntIndef()));
	QMenu* modOp=addOp->addMenu("Filtering");
    myView->addPluginMenu(modOp, "Filtering", true);
	QMenu* unitOp=addOp->addMenu("Change Units");
    myView->addPluginMenu(unitOp, "UnitOperations", true);
	QMenu* timeOp=addOp->addMenu("Time Operations");
    myView->addPluginMenu(timeOp, "TimeOperations", true);
    timeOp->addAction("Timetags to seconds from offset", myView, SLOT(addOpTTtoS()));
    timeOp->addAction("Seconds from offset to Timetags", myView, SLOT(addOpStoTT()));
	timeOp->addAction("Time Interval to seconds", myView, SLOT(addOpTIVLtoS()));
	timeOp->addAction("Time2 - Time1 in sec", myView, SLOT(addOpT2_T1()));
	timeOp->addAction("Timetags2 - Timetags1 in seconds", myView, SLOT(addOpTT2_TT1()));
    QMenu* createRegOp=addOp->addMenu("Create Regular");
    createRegOp->addAction("Time", myView, SLOT(addOpCreateRegTime()));
    createRegOp->addAction("Double", myView, SLOT(addOpCreateRegDbl()));
    createRegOp->addAction("Int", myView, SLOT(addOpCreateRegInt()));
	QMenu* statsOp=addOp->addMenu("Statistics");
	statsOp->addAction("Linear Regression", myView, SLOT(addOpLinReg()));
    myView->addPluginMenu(statsOp, "Statistics", true);
	QMenu* eventsOp=addOp->addMenu("Event Tables");
    myView->addPluginMenu(eventsOp, "EventTables", true);
    QMenu* plugOp=addOp->addMenu("Plug-ins");
    QMenu* geoOp=plugOp->addMenu("Geophysics");
    myView->addPluginMenu(geoOp, "Geophysics", true);
    QMenu* anaOp=plugOp->addMenu("Analysis");
    myView->addPluginMenu(anaOp, "Analysis", true);
	plugOp->addSeparator();
	plugOp->addAction("Browse...", myView, SLOT(addPlugInBrowse()));
	
	addMenu->addAction("Output", myView, SLOT(addOutputItem()));
	bbOp=addOp->addMenu("Black Box from library");
	myView->addBlackBoxMenu(bbOp);
    connect(bbOp, SIGNAL(triggered(QAction*)), myView, SLOT(addBB(QAction*)));
	QMenu* templateOp=addMenu->addMenu("Template");
	templateOp->addAction("A+B", myView, SLOT(addTemplateAdd()));
	templateOp->addAction("A-B", myView, SLOT(addTemplateSub()));
	templateOp->addAction("A*B", myView, SLOT(addTemplateMul()));
	templateOp->addAction("A/B", myView, SLOT(addTemplateDiv()));
	templateOp->addAction("-A", myView, SLOT(addTemplateOpp()));
	
	fileName="";
	
	connect(myView, SIGNAL(updateBlackBoxMenu()), this, SLOT(updateBlackBoxMenu()));
	
	feedbackArea->setHtml(
	"<b>Introduction to the calculator</b> "
	 "<ul><li>Calculations comprise <b>Inputs</b> connected to <b>Operators</b> connected to <b>Outputs</b> </li>"
	 "<li>Create these <b>items</b> using the <b>Components<\b> panel or the right-click menu</li>"
	 "<li><b>Inputs</b> are also created by dropping data objects from the Working list in the Calculator graph area."
	 "<li>Connect <b>Input</b> items to <b>Operator</b> items by dragging a connector from an <b>Input</b> item output slot (arrow on the bottom) to an <b>Operator</b> items input slot (arrow on the top)</li>"
	 "<li>Create <b>Output</b> items, give them a name in the text box, and connect an <b>Operator</b> item output slot to the <b>Output</b> item input slot. Press evaluate (or return) to get the result on the Working list</li>"
	 "<li> <b>Time line</b> items can be added to control time tags and intervals used for all calculations on the page (double click any input or join) to raise the timeline tab.</li>"
	 "<li>To extract a component of an intermediate object, insert an <b>Extraction</b> item between an input and its target</li>"
	 "</ul>"
	);
	
	// delete scene spacer
	myScene->removeItem(line);
	delete line;

	connect(myScene, SIGNAL(newInput(const QString&)), contextWidget, SLOT(newInput(const QString&)));
	connect(qsasWindow, SIGNAL(saveSessionRequest(QString)),  this, SLOT(saveSession(QString)) );
    connect(qsasWindow, SIGNAL(restoreSessionRequest(QString)), this, SLOT(restoreSession(QString)) );
    connect(qsasWindow, SIGNAL(restoreCalcRequest(QString)), this, SLOT(restoreSession(QString)) );
    connect(qsasWindow, SIGNAL(calcPageRequest(int)), this, SLOT(evaluatePage(int)) );


	connect(myScene, SIGNAL(currentPageChanged()), this, SLOT(updatePageNumber()));
	
	button2->setFocus();
}

CalcWindow::~CalcWindow()
{
	delete myScene;
	delete myView;
}

void CalcWindow::evaluate()
{
	feedbackArea->setText("");
    QplugSetTextDisplay((QTextEdit*) feedbackArea); // Allows QplugAppendTextDisplay() to be used
	for(int i=0; i<myScene->pages.size(); i++){
		QString txt = myScene->evaluatePage(i+1);
		feedbackArea->append(txt);
	}
}

void CalcWindow::evaluatePage(int i)
{
	feedbackArea->setText("");
    QplugSetTextDisplay((QTextEdit*) feedbackArea); // Allows QplugAppendTextDisplay() to be used

	QString txt = myScene->evaluatePage(i); // default is current
	feedbackArea->append(txt);

}

void CalcWindow::updateBlackBoxMenu()
{
	bbOp->clear();
	myView->addBlackBoxMenu(bbOp);
}

void CalcWindow::save()
{
	if(fileName=="")
	{
		QString path=QFileDialog::getSaveFileName(this, "Save as...", QString(), "QSAS calculator save (*.qx_calc)");
		if(!path.isEmpty())
		{
			if(myScene->saveAs(path))
			{
				fileName=path;
			}
		}
    }
	else
	{
		myScene->saveAs(fileName);
	}
}

void CalcWindow::saveSession(QString path){
	QString XMLpath = path;
	XMLpath += "/saveSet.qx_calc";
	myScene->saveAs(XMLpath);
}

void CalcWindow::restoreSession(QString path){
	QString XMLpath = path;
    if( !XMLpath.endsWith(".qx_calc", Qt::CaseInsensitive) ) XMLpath += "/saveSet.qx_calc";
        myScene->open(XMLpath);
}

void CalcWindow::saveAs()
{
    QString path=QFileDialog::getSaveFileName(this, "Save as...", QString(), "QSAS calculator save (*.qx_calc)");
    if(!path.isEmpty())
    {
		if(!path.contains(".qx_calc")){
			path += ".qx_calc";
		}
        myScene->saveAs(path);
		fileName=path;
    }
}

void CalcWindow::exportSelection()
{
	QString path=QFileDialog::getSaveFileName(this, "Export as...", QString(), "QSAS calculator save (*.qx_calc)");
    if(!path.isEmpty())
    {
        myScene->exportAs(path);
    }
}

void CalcWindow::include()
{
    QString path=QFileDialog::getOpenFileName(this, "Open...", QString(), "QSAS calculator save (*.qx_calc)");
    if(!path.isEmpty())
    {
        myScene->open(path, false);
		fileName=path;
    }
}

void CalcWindow::restore()
{
    QString path=QFileDialog::getOpenFileName(this, "Open...", QString(), "QSAS calculator save (*.qx_calc);; All files (*)");
    if(!path.isEmpty())
    {
        myScene->open(path);
		fileName=path;
    }
}

void CalcWindow::updateContextWidget()
{
    
    // don't let tabs update while dragging (propagates info between extractors)
    // done by mouse up.
    if(myScene->draggingSelectionRect) {
        // just deselect things, items is empty here
        contextWidget->setView(false);
        return;
    }
    
    contextWidget->setView(true);
    
}
void CalcWindow::selectTimeline(const QString& name)
{
    QString _name(name);
    contextWidget->setTimeLine(_name);
}

void CalcWindow::raiseContextWidget()
{
    QList<QGraphicsItem *> items=myScene->selectedItems();

	contextWidget->showNormal();
	contextWidget->showItemTab(items);
	

}

void CalcWindow::clearContextWidget()
{

	contextWidget->setView(false);

}

void CalcWindow::cut()
{
	QString xmlString=myScene->toXML(true).toString();
	QClipboard *clipboard = QApplication::clipboard();
    QMimeData* data=new QMimeData;
	data->setData("application/calcitems", xmlString.toLatin1().data());
	clipboard->setMimeData(data);

	myScene->deleteSelection();
}

void CalcWindow::copy()
{
	QString xmlString=myScene->toXML(true).toString();
	QClipboard *clipboard = QApplication::clipboard();
    QMimeData* data=new QMimeData;
	data->setData("application/calcitems", xmlString.toLatin1().data());
	clipboard->setMimeData(data);
}

void CalcWindow::paste()
{
	const QClipboard *clipboard = QApplication::clipboard();
	const QMimeData* data=clipboard->mimeData();
	if(data->hasFormat("application/calcitems"))
	{
		QString xmlString=QString(data->data("application/calcitems"));
		QDomDocument doc;
		doc.setContent(xmlString);
		myScene->fromXML(doc, false);
	}

}

void CalcWindow::trackingMessage()
{
	feedbackArea->append("Click the location for your item. Right click or press Esc to cancel");
	feedbackArea->moveCursor(QTextCursor::End);
	// ensure tab does not respond to drag
	contextWidget->tabWidget->setCurrentIndex(0);
}

void CalcWindow::keyPressEvent(QKeyEvent* keyEvent)
{
	if(keyEvent->key() == Qt::Key_Escape) myScene->escPressed();
	else if(keyEvent->key() == Qt::Key_Return) myScene->evaluatePage();
	else QMainWindow::keyPressEvent(keyEvent);
}

void CalcWindow::mousePressEvent(QMouseEvent* event)
{
	if(event->button()==Qt::RightButton)
	{
		myScene->escPressed();
	}
	
	QMainWindow::mousePressEvent(event);
	button2->setFocus();

}

void CalcWindow::nextPage()
{
	int n = CalcScene::currentPage;
	n++;
	if(n>20) n=20;
	
	if(myScene->pages.size() < n) {
		myScene->addPage();
	}
	
	pageLine->setText(QString("%1").arg(n));
	CalcScene::currentPage = n;
	
	myScene->updateVisibleItems();

	contextWidget->setTimeLine(myScene->pages[n-1]->timeLineName);
}

void CalcWindow::previousPage()
{
	int n = CalcScene::currentPage;
	n--;
	if(n<1) n=1;
	
	CalcScene::currentPage = n;
	pageLine->setText(QString("%1").arg(n));
	myScene->updateVisibleItems();

	contextWidget->setTimeLine(myScene->pages[n-1]->timeLineName);
}

void CalcWindow::changePage(const QString& s)
{
	int n = s.toInt();
	if(n>20) n=20;
	
	while(myScene->pages.size() < n) {
		myScene->addPage();
	}
	CalcScene::currentPage=n;
	myScene->updateVisibleItems();

	contextWidget->setTimeLine(myScene->pages[CalcScene::currentPage-1]->timeLineName);
}

void CalcWindow::updatePageNumber()
{
	pageLine->setText(QString("%1").arg(CalcScene::currentPage));
	contextWidget->setTimeLine(myScene->pages[CalcScene::currentPage-1]->timeLineName);
	
}

void QcalcShowCalcProto()
{
	QCALC_Proto_widget->show();
}

CalcWindow* QCalcMakeWindowProto(QWidget *parent, const char * Title)
{
	if ( QCALC_Proto_widget == NULL )
	{
		QCALC_Proto_widget=new CalcWindow(parent, Title);
	}
	
	return QCALC_Proto_widget;
}


