//  qjui.cc
//
// Join Widget
//
// Mar 2002: Tony Allen
// Jan 2014: Tony Allen - migrate to DVOS
//

#include "qjui.h"
#include "qexui.h"
#include "qdutil.h"
#include "qwui.h"
#include "qdutil.h"

#include <qpalette.h>
#include <qgroupbox.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qtooltip.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <stdio.h>


#include <stdlib.h>

class QjuiWindow;

using namespace QSAS;

extern qwuiWindow* qsasWindow;
extern int qsas_restore_release;

QjuiWindow *QJUI_widget=static_cast<QjuiWindow*>(NULL);
QjuiWindow *QJEX_widget=static_cast<QjuiWindow*>(NULL);

// JOIN Window
//---------------

QjuiWindow::QjuiWindow(QWidget* parent, const char * name, bool qexp)
	:QMainWindow(0)
{
	this->setPalette(qApp->palette());
	setAcceptDrops(true);
	setFocusPolicy(Qt::WheelFocus);
	
	verbose=false;

	//// Body of the window
	
	_top_widget=new QWidget(this);
	
	QBoxLayout * main_layout=new QBoxLayout(QBoxLayout::TopToBottom, _top_widget);
	main_layout->setSpacing(4);
	main_layout->setMargin(4);
	
	//// SPLITTER
	
	_top_splitter=new QSplitter(_top_widget);
	
	//// LIST VIEW

	_join_widget=new QjuiWidget(_top_splitter, this);
    _join_widget->setMinimumHeight(400);
    
	_top_splitter->addWidget(_join_widget);
    

	
	//// Options panel
	
	QWidget * opts_widget=new QWidget(_top_splitter);

	QBoxLayout * opts_layout=new QBoxLayout(QBoxLayout::TopToBottom, opts_widget);


    noJoin = new QCheckBox("Do Not Join");
    noJoin->setChecked(false);
    connect( noJoin, SIGNAL(toggled(bool)), this, SLOT(setNoJoin(bool)) );
    opts_layout->addWidget( noJoin);
    noJoin->setVisible(false);
 
    pad = new QCheckBox("Pad variables to same length");
    pad->setChecked(false);
    opts_layout->addWidget( pad);
    pad->setVisible(false);

    if(qexp){
         noJoin->setVisible(true);
    }
    

	//// Join option
	
	join_widget=new QGroupBox("Join Settings for Selected Items", opts_widget);
	
	QGridLayout * join_layout=new QGridLayout(join_widget);
	join_layout->setSpacing(3);
	
    
	QLabel* algorithm_label = new QLabel("Join Algorithm ",join_widget);
	join_layout->addWidget( algorithm_label, 0, 0, Qt::AlignRight );
		
	_algorithm = new QComboBox(join_widget);
	_algorithm->addItem(DV_LINEAR );
    _algorithm->addItem(DV_BOXCAR );
    _algorithm->addItem(DV_NN );
	join_layout->addWidget( _algorithm, 0, 1, Qt::AlignLeft );
	connect(_algorithm, SIGNAL(activated(int)), _join_widget, SLOT(setAlgorithm(int)));
			
	QLabel* width_label = new QLabel("Boxcar width ",join_widget);
	join_layout->addWidget( width_label, 1, 0, Qt::AlignRight );
		
	_width_field = new QguWLinput( join_widget, "width_field" );  
	_width_field->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	connect(_width_field, SIGNAL(edited_OK()), _join_widget, SLOT(setWidthField()));
	_width_field->setToolTip("width of the boxcar (for time series this is in seconds)");
	join_layout->addWidget( _width_field, 1, 1 );
	_width_field->setEnabled(false);
		
	QLabel* blank_label = new QLabel(" ",join_widget);
	join_layout->addWidget( blank_label, 2, 0, Qt::AlignRight );
		
	QLabel* blank2_label = new QLabel(" ",join_widget);
	join_layout->addWidget( blank2_label, 2, 1, Qt::AlignRight );
		
	QLabel* gap_label = new QLabel("Gap Handling ",join_widget);
	join_layout->addWidget( gap_label, 3, 0, Qt::AlignRight );
		
	_gap = new QComboBox(join_widget);
	_gap->addItem(DV_LINEAR );
	_gap->addItem(DV_ZERO_FILL );
	_gap->addItem(DV_FILL );
	_gap->addItem(DV_NN );
	_gap->addItem(DV_REMOVE );
	join_layout->addWidget( _gap, 3, 1, Qt::AlignLeft );
	connect(_gap, SIGNAL(activated(int)), _join_widget, SLOT(setGap()));
		
    QLabel* gapSize_label = new QLabel("Gap threshold ",join_widget);
    join_layout->addWidget( gapSize_label, 4, 0, Qt::AlignRight );
    
    _gap_field = new QguWLinput( join_widget, "gap_field" );
    _gap_field->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
    connect(_gap_field, SIGNAL(edited_OK()), _join_widget, SLOT(setGapField()));
    _gap_field->setToolTip("minimum empty interval to be treated as a gap (for time series this is in seconds)");
    join_layout->addWidget( _gap_field, 4, 1 );

    QLabel* end_label = new QLabel("End Handling ",join_widget);
    join_layout->addWidget( end_label, 5, 0, Qt::AlignRight );
    
    _end = new QComboBox(join_widget);
    _end->addItem(DV_LINEAR );
    _end->addItem(DV_ZERO_FILL );
    _end->addItem(DV_FILL );
    _end->addItem(DV_NN );
    _end->addItem(DV_REMOVE );
    connect(_end, SIGNAL(activated(int)), _join_widget, SLOT(setEnd()));
    _end->setToolTip("what to do data does not span target series");
    join_layout->addWidget( _end, 5, 1 );

    QLabel* min_box_label = new QLabel("Min Boxcar Num ",this);
    join_layout->addWidget( min_box_label, 6, 0, Qt::AlignRight );
    
    _box_min_field = new QguWLinput( this, "min_box_field", 8 );
    _box_min_field->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
    _box_min_field->setText("3");
	_box_min_field->setEnabled(false);
    
    _box_min_field->setToolTip("minimum number of points in the boxcar");
    join_layout->addWidget( _box_min_field, 6, 1);
    connect(_box_min_field, SIGNAL(edited_OK()), _join_widget, SLOT(setMinBoxField()));

	opts_layout->addWidget(join_widget);
	
	//// Time option
	
	time_widget=new QGroupBox("Join Target", opts_widget);
	
	QGridLayout * time_layout=new QGridLayout(time_widget);
	time_layout->setSpacing(4);
	
	 QLabel* TimeInterval_label = new QLabel("Interval ",time_widget);
	time_layout->addWidget( TimeInterval_label, 0, 0, Qt::AlignRight );
		
	_TimeIvlSelect = new QComboBox(time_widget);
	_TimeIvlSelect->insertItem(0, "from Data" );
	_TimeIvlSelect->insertItem(1, "from Object" );
	time_layout->addWidget( _TimeIvlSelect, 0, 1, Qt::AlignLeft );
	connect(_TimeIvlSelect, SIGNAL(activated(int)), this, SLOT(setTTivl(int)));
		
	IvlSource_label = new QLabel("  ",time_widget);
	time_layout->addWidget( IvlSource_label, 1, 0, Qt::AlignRight );
		
	_IvlSource = new QguWLinput( time_widget, "IvlSource" );  
	_IvlSource->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
    _IvlSource->active = true;
		
	_IvlSource->setToolTip("not active");
	time_layout->addWidget( _IvlSource, 1, 1, Qt::AlignLeft );
	
	connect(_IvlSource, SIGNAL(edited_OK()), this, SLOT(setIntervalOption()));

	time_layout->addWidget(new QLabel("", time_widget), 2, 0); //space
	
	QLabel* TimeTags_label = new QLabel("Tags ",time_widget);
	time_layout->addWidget( TimeTags_label, 3, 0, Qt::AlignRight );
		
	_TimeTagSelect = new QComboBox(time_widget);
	_TimeTagSelect->insertItem(0, "Create Regular");
	_TimeTagSelect->insertItem(1, "from Object");
	_TimeTagSelect->insertItem(2, "from 1st Data Obj");
	time_layout->addWidget( _TimeTagSelect, 3, 1, Qt::AlignLeft );
	connect(_TimeTagSelect, SIGNAL(activated(int)), this, SLOT(setTTsource(int)));
		
	TagSource_label = new QLabel("Drop object ",time_widget);
	time_layout->addWidget( TagSource_label, 4, 0, Qt::AlignRight );
		
	_TagSource = new QguWLinput( time_widget, "TagSource" );  
	_TagSource->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
	connect(_TagSource, SIGNAL(hadDropEvent()), this, SLOT(tagsHadDrop()));
	
	_TagSource->setToolTip("Uses DEPEND_0 of this object as target time or scalar sequence to join onto");
	time_layout->addWidget( _TagSource, 4, 1, Qt::AlignLeft );

	opts_layout->addWidget(time_widget);

	//// Link option
	
	opts_widget->adjustSize();
	
	_top_splitter->addWidget(opts_widget);
	_top_splitter->adjustSize();
	
	main_layout->addWidget(_top_splitter);
	
	//// FEEDBACK
	
	feedback_widget=new QGroupBox("Sequence Dependency (Time or Scalar Sequence) ", _top_widget);
	QGridLayout * feedback_layout=new QGridLayout(feedback_widget);
	feedback_layout->setSpacing(4);
	feedback_layout->setColumnStretch(1, 10);
	
	QLabel* Start_label = new QLabel("Start: ",feedback_widget);
	feedback_layout->addWidget( Start_label, 0, 0, Qt::AlignRight );

	Start_field = new QLabel("<Default>",feedback_widget);
	feedback_layout->addWidget( Start_field, 0, 1, Qt::AlignLeft );

	QLabel* End_label = new QLabel("End: ",feedback_widget);
	feedback_layout->addWidget( End_label, 1, 0, Qt::AlignRight );

	End_field = new QLabel("<Default>",feedback_widget);
	feedback_layout->addWidget( End_field, 1, 1, Qt::AlignLeft );

	QLabel* Ntags_label = new QLabel("     Number of Tags in Interval: ",feedback_widget);
	feedback_layout->addWidget( Ntags_label, 0, 2, Qt::AlignRight );

	Ntags_field = new QLabel("<All>",feedback_widget);
	feedback_layout->addWidget( Ntags_field, 0, 3, Qt::AlignLeft );

	QLabel* spacing_label = new QLabel("Tag Spacing: ",feedback_widget);
	feedback_layout->addWidget( spacing_label, 1, 2, Qt::AlignRight );

	spacing_field = new QLabel("<Default>",feedback_widget);
	feedback_layout->addWidget( spacing_field, 1, 3, Qt::AlignLeft );

	main_layout->addWidget(feedback_widget);
	//// BUTTONS
	
	_buttons_widget=new QWidget(_top_widget);
	buttons_layout=new QBoxLayout(QBoxLayout::LeftToRight, _buttons_widget);

	buttons_layout->setSpacing(4);
	
	//  buttons added in JOIN or EXPORT create windows  
	
	main_layout->addWidget(_buttons_widget);
	
	setCentralWidget(_top_widget);
	
		//// Menus
	
	menuBar()->clear();
	
	QMenu* file_menu = menuBar()->addMenu("File");
	QMenu* edit_menu = menuBar()->addMenu("Edit");
	options_menu = menuBar()->addMenu("Options");
	menuBar()->addSeparator();
	QMenu * help = menuBar()->addMenu(" Help");

		
	file_menu->clear();
	file_menu->addAction("&Save", this, SLOT(saveXML()), Qt::CTRL+Qt::Key_S);
	file_menu->addAction("Save As...", this, SLOT(saveAsXML()));
	file_menu->addSeparator();
	file_menu->addAction("Include...", this, SLOT(include()));
	file_menu->addAction("&Restore", this, SLOT(restore()), Qt::CTRL+Qt::Key_R);
	file_menu->addSeparator();
	file_menu->addAction("Use Default Settings", this, SLOT(defaultAll()));
		
	edit_menu->clear();
	edit_menu->addAction("Re-Name Item", _join_widget, SLOT( renameSelectedItem()));
	edit_menu->addSeparator();
	edit_menu->addAction("&Copy", _join_widget, SLOT( copy() ), Qt::CTRL+Qt::Key_C );
	edit_menu->addAction("Paste", _join_widget, SLOT(paste()), Qt::CTRL+Qt::Key_V);
	edit_menu->addAction("select &All", _join_widget, SLOT( selectAll() ), Qt::CTRL+Qt::Key_A );
	edit_menu->addSeparator();
	edit_menu->addAction("Copy Settings <-",this,SLOT(keepChoices()), Qt::CTRL+Qt::Key_Left);
	edit_menu->addAction("Paste Settings ->",this,SLOT(applyChoices()), Qt::CTRL+Qt::Key_Right);
	edit_menu->addSeparator();
	
	QAction *delAct = edit_menu->addAction("Delete Item", _join_widget, SLOT(deleteListItems()), Qt::Key_Delete);
	QList<QKeySequence> keyList;
    keyList.append(Qt::Key_Delete);
    keyList.append(Qt::Key_Backspace);
    keyList.append(Qt::CTRL+Qt::Key_X);
    delAct->setShortcuts(keyList);
	
	edit_menu->addAction("Clear List", this, SLOT(deleteAllItems()));

	options_menu->clear();
	verboseAction = options_menu->addAction("Verbose", this, SLOT(verboseToggle()));
	verboseAction->setCheckable(true);


	help->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->addSeparator();
	help->addAction("...Data Export", this, SLOT( launchManExport() ));
	help->addAction("...Join", this, SLOT( launchManJoin() ));
	help->addSeparator();
	help->addAction("Module Version", this, SLOT( launchVersion() ));
	
	
	adjustSize();
	
	status(" ");
}

void QjuiWindow::setNoJoin(bool noJoin ){
    join_widget->setVisible( !noJoin );
    time_widget->setVisible( !noJoin );
    feedback_widget->setVisible( !noJoin );
    
    pad->setVisible(noJoin);
    
}

void QjuiWindow::launchManual(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::launchManual \n");
	fflush(stdout);
 }
   qsasWindow->help_window->manEntry("index.html");
}

void QjuiWindow::launchManExport(){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::launchManExport \n");
	fflush(stdout);
 }
   qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}

void QjuiWindow::launchManJoin(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::launchManJoin \n");
	fflush(stdout);
 }
   qsasWindow->help_window->manEntry("QSAS_analysis.html");
}

void QjuiWindow::launchVersion(){

 if(QuSlotDebug())
 {
	printf("DirectImportWindow::launchVersion \n");
	fflush(stdout);
 }
 
  QString text;
  text  = QString("Version Numbers...\n"); 
  text += QString(QSAS_VERSION_ID);
  text += "\n Join/Export  ";
  text += QString(QJUI_VERSION);
  QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );
}


void QJUImakeWindow( QWidget *parent, const char * Title )
{
	if(QJUI_widget==NULL)
  	{
     	QJUI_widget = QJUICreatePopup(parent, Title);
     	QJUI_widget->setWindowTitle(Title);

     	QJUI_widget->JoinBtn = new QPushButton( "Join", QJUI_widget->_buttons_widget);
     	QJUI_widget->buttons_layout->addWidget(QJUI_widget->JoinBtn);
     	QJUI_widget->connect( QJUI_widget->JoinBtn, SIGNAL(clicked()), QJUI_widget, SLOT(DoJoin()) );
     	QJUI_widget->dismissBtn = new QPushButton( "Dismiss", QJUI_widget->_buttons_widget);
     	QJUI_widget->buttons_layout->addWidget(QJUI_widget->dismissBtn);
     	QJUI_widget->connect( QJUI_widget->dismissBtn, SIGNAL(clicked()), QJUI_widget, SLOT(close()) );
  
     	QJUI_widget->_filename += QString("/saveSet");
     	QJUI_widget->SAVE_EXTENSION = QString(".qx_join");
     	QJUI_widget->_filename += QJUI_widget->SAVE_EXTENSION;
     	QJUI_widget->_TimeTagSelect->setCurrentIndex(2);
     	QJUI_widget->_TimeTagSelect_Def = 2;
     	QJUI_widget->_TimeTagSelect_cpy = 2;
		QJUI_widget->_saveset_filter=QString("QSAS join save (*.qx_join);; XML files (*.xml)");
		QJUI_widget->adjustSize();
		QJUI_widget->isJoin = true;

	}
	return;
 
}


void QEXPmakeWindow( QWidget *parent, const char * Title )
{
  if(QJEX_widget==NULL)
  {
     QJEX_widget = QJUICreatePopup(parent, Title, true);
     QJEX_widget->setWindowTitle(Title);
     QJEX_widget->WriteBtn = new QPushButton( "Export...", QJEX_widget->_buttons_widget);
     QJEX_widget->WriteBtn->setMinimumSize(QJEX_widget->WriteBtn->sizeHint());
     QJEX_widget->connect( QJEX_widget->WriteBtn, SIGNAL(clicked()),  QJEX_widget, SLOT(writeFile()) );
     QJEX_widget->buttons_layout->addWidget(QJEX_widget->WriteBtn);
     QJEX_widget->dismissBtn = new QPushButton( "Dismiss", QJEX_widget->_buttons_widget);
     QJEX_widget->dismissBtn->setMinimumSize(QJEX_widget->dismissBtn->sizeHint());
     QJEX_widget->connect( QJEX_widget->dismissBtn, SIGNAL(clicked()), QJEX_widget, SLOT(close()) );
     QJEX_widget->buttons_layout->addWidget(QJEX_widget->dismissBtn);

     QJEX_widget->_filename += QString("/saveSet");
     QJEX_widget->SAVE_EXTENSION = QString(".qx_exp");
     QJEX_widget->_filename += QJEX_widget->SAVE_EXTENSION;
     QJEX_widget->_TimeTagSelect->setCurrentIndex(2);
     QJEX_widget->_TimeTagSelect_Def = 2;     
     QJEX_widget->_TimeTagSelect_cpy = 2;
 	 QJEX_widget->_saveset_filter=QString("QSAS join save (*.qx_exp);; XML files (*.xml)");
	 QJEX_widget->adjustSize();
	 QJEX_widget->isJoin = false;
  }

  return;
}


void QjuiShowExport(){
  QSize qsasGUISize = qsasWindow->frameSize();
  QJEX_widget->move(qsasGUISize.width(),45);
  QJEX_widget->show();
}

void QjuiShowJoin(){  
  QSize qsasGUISize = qsasWindow->frameSize();
  QJUI_widget->move(qsasGUISize.width(),60);
  QJUI_widget->show();
}

QjuiWindow * QJUICreatePopup(QWidget *parent, const char * Title, bool qexp)
{
 QjuiWindow * Window = new QjuiWindow(parent,Title,qexp);
 Window->move(50,50);
 Window->setDefaults();
 Window->defaultAll();
 
 Window->connect(parent, SIGNAL(saveSessionRequest(QString)),  Window, SLOT(saveSession(QString)) );
 Window->connect(parent, SIGNAL(restoreSessionRequest(QString)),   Window, SLOT(restoreSession(QString)) );
 Window->connect(parent, SIGNAL(clearListRequest()),   Window, SLOT(deleteAllItems()) );
 return Window;
}

QjuiWindow::~QjuiWindow(){;}

 void QjuiWindow::dragEnterEvent(QDragEnterEvent *event)
{ 
 if(QuSlotDebug())
 {
	printf("QjuiWindow::dragEnterEvent \n");
	fflush(stdout);
 }
 
      this->activateWindow();
      this->raise();
}


void QjuiWindow::saveXML()
{
 	ofstream file;
	file.open(_filename.toLatin1().data(),ios::out);

	if(file.is_open())
	{
		QDomDocument doc("QSAS_join_save");

		QDomElement root=doc.createElement("QSAS_Join");
		doc.appendChild(root);
		string guiID;
		guiID=QSAS_VERSION_ID;

		QDomElement version=createTextXMLNode(doc, "QSAS_Version", guiID);
		root.appendChild(version);

		QDomElement join_options=doc.createElement("Join_Options");
		join_options.appendChild(createTextXMLNode(doc, "Algorithm", _algorithm->currentIndex()));
		join_options.appendChild(createTextXMLNode(doc, "Width", _width_field->text()));
		join_options.appendChild(createTextXMLNode(doc, "Gap_Handling", _gap->currentIndex()));
        join_options.appendChild(createTextXMLNode(doc, "Gap_Threshold", _gap_field->text()));
        join_options.appendChild(createTextXMLNode(doc, "Min_Box_Num", _box_min_field->text()));
        join_options.appendChild(createTextXMLNode(doc, "End_Handling", _end->currentIndex()));
		root.appendChild(join_options);

		QDomElement time_source=doc.createElement("Time_Source");
		time_source.appendChild(createTextXMLNode(doc, "Interval", _TimeIvlSelect->currentIndex()));
		time_source.appendChild(createTextXMLNode(doc, "Object", _IvlSource->text()));
		time_source.appendChild(createTextXMLNode(doc, "Time_Tags", _TimeTagSelect->currentIndex()));
		time_source.appendChild(createTextXMLNode(doc, "Tags_Source", _TagSource->text()));
		root.appendChild(time_source);

		root.appendChild(_join_widget->toXMLElement(doc));
            
		QString xml = doc.toString();
		file << xml.toLatin1().data();
		file.close();
	}
}

void QjuiWindow::saveAsXML()
{
    QString path=QFileDialog::getSaveFileName(this, "Save as...", QString(), _saveset_filter);

    if(!path.isEmpty())
    {
		if(!path.contains(SAVE_EXTENSION) && !path.contains(".xml")){
			path += SAVE_EXTENSION;
		}
		
		_filename=path;

		saveXML();
	}
}

void QjuiWindow::restoreXML(QString path, bool replace)
{
	if( !QFile::exists(path) ) return;

    _filename = path;
    
	QDomDocument doc;
	QFile myFile(path);
	doc.setContent(&myFile);
	myFile.close();
        
	QDomElement docElem = doc.documentElement();
	if(!docElem.isNull())
	{
		if(replace) deleteAllItems();

		string guiID="0.0.00";
		QDomElement el= docElem.firstChildElement();

		QString s;
		if(getXMLNodeText(el, "QSAS_Version", s))
		{
			guiID=s.toLatin1().data();
		}

		QDomElement join_options=docElem.firstChildElement("Join_Options");
		if(!join_options.isNull())
		{
			if(getXMLNodeText(join_options, "Algorithm", s))
			{
				_algorithm->setCurrentIndex(s.toInt());
			}
			if(getXMLNodeText(join_options, "Width", s))
			{
				_width_field->setText(s);
			}
            if(getXMLNodeText(join_options, "Gap_Handling", s))
            {
                _gap->setCurrentIndex(s.toInt());
            }
            if(getXMLNodeText(join_options, "End_Handling", s))
            {
                _end->setCurrentIndex(s.toInt());
            }
            if(getXMLNodeText(join_options, "Gap_Threshold", s))
            {
                _gap_field->setText(s);
            }
            if(getXMLNodeText(join_options, "Min_Box_Num", s))
            {
                _box_min_field->setText(s);
            }
				
		}

		QDomElement time_source=docElem.firstChildElement("Time_Source");
		if(!time_source.isNull())
		{
			if(getXMLNodeText(time_source, "Interval", s))
			{
				_TimeIvlSelect->setCurrentIndex(s.toInt());
			}
			if(getXMLNodeText(time_source, "Object", s))
			{
				_IvlSource->setText(s);
			}
			if(getXMLNodeText(time_source, "Time_Tags", s))
			{
				_TimeTagSelect->setCurrentIndex(s.toInt());
				setTTsource(s.toInt());
			}
			if(getXMLNodeText(time_source, "Tags_Source", s))
			{
				_TagSource->setText(s);
			}
		}

		QDomElement join_list=docElem.firstChildElement("List");
		if(!join_list.isNull())
		{
			_join_widget->fromXMLElement(join_list);
		}
	}
	return;
}

void QjuiWindow::restore(bool replace)
{
	QString path;

    QString restore_filter = _saveset_filter;
    restore_filter += ";; All files (*)";
	
    path=QFileDialog::getOpenFileName(this, "Open...", QString(), restore_filter);

    if(!path.isEmpty())
    {
		restoreXML(path, replace);
	}
}

void QjuiWindow::saveSession(QString folder){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::saveSession\n");
	fflush(stdout);
 }
 

  if(folder.isEmpty()) return;
  // keep original name
  QString fileN = QString(_filename.toLatin1().data());
  
  // set session folder and save
  this->_filename = QString(folder.toLatin1().data());
  this->_filename += QString("/saveSet");
  QString extn = SAVE_EXTENSION;
  this->_filename += extn;

  saveXML();

  // put file name back
  _filename = QString(fileN.toLatin1().data());
}


void QjuiWindow::restoreSession(QString folder){

	if ( QuSlotDebug() )
	{
		printf( "QjuiWindow::restoreSession \n");
		fflush(stdout);
	}
    
	if(folder.isEmpty()) return;

	QString fileN =  QString(folder.toLatin1().data());
	fileN += QString("/saveSet");
    fileN += SAVE_EXTENSION;
    
	restoreXML(fileN, true);
  
}

 void QjuiWindow::deleteAllItems(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::deleteAllItems \n");
	fflush(stdout);
 }
 
 
   _join_widget->selectAll();
   _join_widget->deleteListItems();
}

void QjuiWindow::verboseToggle(){ 

 if(QuSlotDebug())
 {
	printf("QjuiWindow::verboseToggle \n");
	fflush(stdout);
 }
	 
	verbose = !verbose;
	verboseAction->setChecked(verbose);

}

//--------------------------
 
void QjuiWindow::DoJoin(){ 

	if(QuSlotDebug())
	{
		printf("QjuiWindow::DoJoin \n");
		fflush(stdout);
	}
 
	// keep settings
	keepChoices();
  
	DvJoinList JoinedList;
    
	if( joinObjects(JoinedList) ){

		status("Placing on WL...",10000);
    
		// put resulting objects on the Working List
		DvNode *node = JoinedList.first();
		while (node)  {

			// skip empty and failed slots quietly
    
       		DvObject_var out_obj = node->obj();
     
			if( out_obj.is_ok() ){
     
      			// Note Multijoin sets Xrefs and time tags on output objects
	
				// put joined obj on W List
         		QString joined_name = QString(node->name().c_str());
         		joined_name += "_J";
		 		status("putting "+joined_name+" on working list",10000);
         		QwuiPutOnWL(out_obj, joined_name);

       		}
			node = node->next;
      }  
      status("JOIN Finished", 10000);
   }
} 



// ---------------------------------------------------

bool QjuiWindow::writeFile(){ 

	if(QuSlotDebug())
	{
		printf("QjuiWindow::writeFile\n");
		fflush(stdout);
	}

  
	// keep settings for default behaviour
	keepChoices();
   
	DvJoinList JoinedList;
  
    if(noJoin->isChecked()){
 
        int numRows=_join_widget->count();
        if(numRows==0) return false;
        
        QjuiWidgetItem* item;
        
        for(int i=0; i<numRows; ++i)
        {
            item=(QjuiWidgetItem*)(_join_widget->item(i));
            DvObject_var obj = getWLObject(item->WLcontext.toLatin1().data());
            if (obj.is_nil() )
            {
                status("object no longer exists",10000);
                return false;
            }
            
            JoinedList.append(item->WLcontext.toLatin1().data(), obj);
            
        }
        if(pad->isChecked()){
            size_t nRecords = 0;
            // find max series length
            DvNode *node = JoinedList.first();
            while(node) {
                if(node->obj()->seqSize() > nRecords) nRecords = node->obj()->seqSize();
                node = node->next;
            }

            // pad if needed
            node = JoinedList.first();
            while(node) {
                if(node->obj()->seqSize() < nRecords) node->obj() = node->obj()->resize(nRecords);
                node = node->next;
            }
            
        }

        
    }
    else{
        // try to join objects, if it fails we still want to write file
        if( !(joinObjects(JoinedList)) ){
            // DEPEND_0 not set or no overlap: copy inputs to outputs
		
            status("Failed to join variables...",10000);

            int numRows=_join_widget->count();
            if(numRows==0) return false;
  
            QjuiWidgetItem* item;

            for(int i=0; i<numRows; ++i)
            {
                item=(QjuiWidgetItem*)(_join_widget->item(i));
                DvObject_var obj = getWLObject(item->WLcontext.toLatin1().data());
                if (obj.is_nil() )
                {
                    status("object no longer exists",10000);
                    return false;
                }

                JoinedList.append(item->WLcontext.toLatin1().data(), obj);

            }
  
        }
    }
	
	if(JoinedList.size() == 0) {
		status("Empty list...",10000);
		return false;
	}

	status("Writing file...",10000);

	dismissBtn->setEnabled(false);
	WriteBtn->setEnabled(false);

	//   Write file
	QexWriteFile (this, JoinedList);

	dismissBtn->setEnabled(true);
	WriteBtn->setEnabled(true);

    return true;

}


bool QjuiWindow::joinObjects(DvJoinList &JoinedList)
{

	if(QuSlotDebug())
	{
		printf("QjuiWindow::joinObjects\n");
		fflush(stdout);
	}
 
	JoinedList.clear(); // in case of previous call
	
  	DvJoinList ToJoinList;
  
	// Get Time tags
     
	DvObject_var target_D0 = getTimeTags();
	DvEvent target_interval = target_D0->getTimeRange();

    if(target_D0.is_nil() ){
    	status("No Target Tags",50000);
    	return false;
	}
  
	// for each item on Join List, set choices
	int numRows=_join_widget->count();
	if(numRows==0)  return false;
  
	QjuiWidgetItem* item;

	for(int i=0; i<numRows; ++i)
	{
		item=(QjuiWidgetItem*)(_join_widget->item(i));
		bool OK;

		DvObject_var obj = getWLObject(item->WLcontext.toLatin1().data());
		if (obj.is_nil() )
		{
			status("object no longer exists",10000);
			return false;
		}

		// ensure some overlap with target tt
		if( !(obj->getTimeRange().intersects(target_interval)) )
		{
			(void)QMessageBox::warning(this,"Join","at least one series has no overlap with target time interval.\n Attempting to save unjoined. Only CDF allows different length series.");
			qApp->processEvents();
			return false;
		}

      
        // gap tolerance in seconds
        
        double gap=item->I_gap_field.toDouble(&OK);
        if(gap > 1.e-20) obj->change_xref(DV_GAP_WIDTH, gap);
 		
		// boxcar width in seconds
      
		double boxcar_width=item->I_width_field.toDouble(&OK);

        // boxcar min num points
        
        int minBox = item->I_minBox_field.toInt(&OK);
        if(minBox > 0) obj->change_xref(DV_MIN_BOXCAR, minBox);

		// interpolation method
		DvString method( (_algorithm->itemText(item->I_algorithm)).toLatin1().data() );
		obj->change_xref(DV_JOIN_METHOD, method);	
			
		if(method == DvString(DV_BOXCAR) ){
			obj->change_xref(DV_BOX_WIDTH, boxcar_width);
		}
		
        // gap handling option
        
        DvString gap_opt( (_gap->itemText(item->I_gap)).toLatin1().data() );
        obj->change_xref(DV_GAP_OPTION, gap_opt);

        // end handling option
        
        DvString end_opt( (_end->itemText(item->I_end)).toLatin1().data() );
        obj->change_xref(DV_END_OPTION, end_opt);
			
		// create node
		ToJoinList.append(item->WLcontext.toLatin1().data(), obj);

	}
	// Do actual join
     
    status("performing join...", 10000);
	ToJoinList.MultiJoin(target_D0, JoinedList);

    return true;


}

 void QjuiWindow::defaultAll(){
 
  _width_field->setText(_width_field_Def);
  _gap->setCurrentIndex(_gap_Def);
  _end->setCurrentIndex(_end_Def);
  _gap_field->setText(_gap_field_Def);
  _box_min_field->setText(_box_min_field_Def);
  _algorithm->setCurrentIndex(_algorithm_Def);
  _TimeIvlSelect->setCurrentIndex(_TimeIvlSelect_Def);
  _IvlSource->setText(_IvlSource_Def);
  _TimeTagSelect->setCurrentIndex(_TimeTagSelect_Def);
  _TagSource->setText(_TagSource_Def);
  _join_widget->setItemChoices(-1);

 
}

 void QjuiWindow::setDefaults()
{
  _width_field_Def = QString("60");
  _width_field_cpy = _width_field_Def;
  
  _gap_Def = 0;
  _gap_cpy = _gap_Def;

  _end_Def = 4;
  _end_cpy = _end_Def;
  
  _gap_field_Def = QString("6");
  _gap_field_cpy = _gap_field_Def;

  _box_min_field_Def = QString("3");
  _box_min_field_cpy = _box_min_field_Def;

    _algorithm_Def = 0;
  _algorithm_cpy = _algorithm_Def;
  
  _IvlSource_Def = QString(" ");
  _IvlSource_cpy = _IvlSource_Def;
  
  _TimeIvlSelect_Def = 0;
  _TimeIvlSelect_cpy = _TimeIvlSelect_Def;
  
  _TagSource_Def = QString(" ");
  _TagSource_cpy = _TagSource_Def;
  
  // re-set in join or export, but here to allow safe initialisation
  _TimeTagSelect_Def = 2;
  _TimeTagSelect_cpy = _TimeTagSelect_Def;
}

 void QjuiWindow::keepChoices(){

 if(QuSlotDebug())
 {
	printf("QjuiWindow::keepChoices \n");
	fflush(stdout);
 }
 
 
  _width_field_cpy = _width_field->text();
  _gap_cpy = _gap->currentIndex();
  _end_cpy = _end->currentIndex();
  _gap_field_cpy = _gap_field->text();
  _box_min_field_cpy = _box_min_field->text();
  _algorithm_cpy = _algorithm->currentIndex();
  _IvlSource_cpy = _IvlSource->text();
  _TimeIvlSelect_cpy = _TimeIvlSelect->currentIndex();
  _TagSource_cpy = _TagSource->text();
  _TimeTagSelect_cpy = _TimeTagSelect->currentIndex();
 
}

void QjuiWindow::applyChoices(){

	if(QuSlotDebug())
	{
		printf("QjuiWindow::applyChoices \n");
		fflush(stdout);
	}
 
	_width_field->setText(_width_field_cpy);
    _gap->setCurrentIndex(_gap_cpy);
    _end->setCurrentIndex(_end_cpy);
    _gap_field->setText(_gap_field_cpy);
    _box_min_field->setText(_box_min_field_cpy);
	_algorithm->setCurrentIndex(_algorithm_cpy);
 	_TimeIvlSelect->setCurrentIndex(_TimeIvlSelect_cpy);
	_IvlSource->setText(_IvlSource_cpy);
	_TimeTagSelect->setCurrentIndex(_TimeTagSelect_cpy);
	_TagSource->setText(_TagSource_cpy);
	_join_widget->setItemChoices(-1);
 
}

void QjuiWindow::setTTivl(int item){ 

	if(QuSlotDebug())
	{
		printf("QjuiWindow::setTTivl %d\n", item);
		fflush(stdout);
	}
 
	if(item == 0) {
    	// from data
		IvlSource_label->setText(" ");
     	_IvlSource->setToolTip("not active");
    	_IvlSource->setText(" "); // force signal so that colour updates
	}
	else if(item == 1){
    	IvlSource_label->setText("Interval Object");
     	_IvlSource->setToolTip("drag and drop object here");
    	_IvlSource->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
    	_IvlSource->addAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_SEQ, QS_ANY));
    	_IvlSource->setText("");
    	_IvlSource->reValidate(true);
	}
}

// ---------------------------------

void QjuiWindow::tagsHadDrop(){ 
	if(QuSlotDebug())
	{
		printf("QjuiWindow::tagsHadDrop \n");
		fflush(stdout);
	}
	TagSource_label->setText("Target Tag Object");
	_TagSource->setToolTip("drag and drop object here");
	_TagSource->setAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_SEQ, QS_ANY));
	_TimeTagSelect->setCurrentIndex(1);
	_TimeTagSelect->update();
}

// ---------------------------------

void QjuiWindow::setTTsource(int item){ 

	if(QuSlotDebug())
	{
		printf("QjuiWindow::setTTsource %d\n", item);
		fflush(stdout);
	}
 
	if(item == 0) {
   	 	// create regular
   		TagSource_label->setText("Spacing");
    	_TagSource->setToolTip("enter spacing here (time is in seconds)");
    	_TagSource->setText(" ");
		_TagSource->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	}
	else if(item == 1){
    	TagSource_label->setText("Target Tag Object");
    	_TagSource->setToolTip("drag and drop object here");
    	_TagSource->setAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_SEQ, QS_ANY));
	}
	else if(item == 2){
    	TagSource_label->setText(" ");
    	_TagSource->setToolTip("Ignored: using first object in join list");
    	_TagSource->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_SEQ, QS_ANY));
    	_TagSource->addAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_SEQ, QS_ANY));
	}
	_TagSource->reValidate(true); // updates T display if valid
}

// ---------------------------------

void QjuiWindow::setIntervalOption(){ 
 	_TimeIvlSelect->setCurrentIndex(1);
	setTimeFeedback();
}


// ---------------------------------
 
void QjuiWindow::setTimeFeedback(){ 

	if(QuSlotDebug())
	{
		printf("QjuiWindow::setTimeFeedback \n");
		fflush(stdout);
	} 
 
	DvEvent ivl = this->getInterval();
	int Ntags=-1;
	double Spacing=-1.;
	double Tinterval;
  
	// set interval
 
	if ( ivl.empty() ) {
  		double start;
		double end;
  		if(this->getScalarInterval(start, end)){
			Tinterval = end - start;
    		Start_field->setText(QString("%1").arg(start));
    		End_field->setText(QString("%1").arg(end));
		}
		else{
			Ntags_field->setText("??");
    		spacing_field->setText("??");
    		Start_field->setText("??");
    		End_field->setText("??");
    		Tinterval = -1.0;
		}
	}
	else{
    	Start_field->setText(QString(ivl.start().getISOstring().c_str()));
    	End_field->setText(QString(ivl.end().getISOstring().c_str()));
    	Tinterval = ivl.duration_abs();
	}
  
	// set tags
  
	if( _TimeTagSelect->currentIndex() == 0)
	{
    	// create regular tags
    	bool OK;
    	Spacing = _TagSource->text().toDouble(&OK);
    	if(!OK || Spacing < 1.e-20) {
      		Spacing = -1.0;
      		Ntags = -1;
    	}
    	else
      		Ntags = (int) (Tinterval/Spacing + 1.0);
    
	}
	else
	{
    	// tags from object (time or scalar)
    	DvObject_var ts = getTagsFromObj();
		if(ts.is_ok()){
    		Spacing = ts->get_spacing();
		}
    	if(Spacing > 0) Ntags = (int) (fabs(Tinterval/Spacing) + 1.0);
	}
  
	QString txt;
	if(Ntags > 0) Ntags_field->setText(txt.setNum(Ntags));
	else Ntags_field->setText("??");
  
	if(Spacing > 0) spacing_field->setText(txt.setNum(Spacing));
	else spacing_field->setText("??");
  
}

// --------------------------------

bool QjuiWindow::getScalarInterval(double &start, double &end){

	if( _TimeIvlSelect->currentIndex() == 0)
	{
    	// interval from data
    	return this->getScalarIntervalFromData( start, end);
	}
	else
	{
    	// interval from object
    	DvObject_var ivl_obj = _IvlSource->getObject();
    	if ( ivl_obj.is_ok() ) {
      
        	DvObject_var D0 = ivl_obj->getDep0();
        	if(D0.is_nil()) return false;
     	
			start = D0->asDouble(0);
			end = D0->asDouble(D0->seqSize()-1);
			return true;
		}

	} 
	return false; 
}

// --------------------------------

DvEvent QjuiWindow::getInterval(){

  DvEvent ivl;
  if( _TimeIvlSelect->currentIndex() == 0)
  {
    // interval from data
    ivl = this->getIntervalFromData();

  }
  else
  {
    // interval from object
    DvObject_var ivl_obj = _IvlSource->getObject();
    if ( ivl_obj.is_ok() ) {
		if(ivl_obj->is_event() ) ivl = ivl_obj->event(0);
		else  ivl = ivl_obj->getTimeRange(); 
	}

  } 
  this->status(ivl.getISOstring().c_str(), 10000);
  return ivl; 
}

// --------------------------------

DvEvent QjuiWindow::getIntervalFromData()
{
	DvEvent ivl;
	DvEvent ivlUnion;
	ivlUnion.set_start(ivlUnion.end()); // empty() will return true

	int numRows=_join_widget->count();

	QjuiWidgetItem * item;
	for(int i=0; i<numRows; ++i)
	{
		item=(QjuiWidgetItem*)_join_widget->item(i);
		DvObject_var ivl_obj = getWLObject(item->WLcontext.toLatin1().data());
		if ( ivl_obj.is_ok() )
		{
			ivl = ivl_obj->getTimeRange();
      	}

		if(ivlUnion.empty() ) ivlUnion = ivl;
      	else
      	{
        	ivlUnion = ivlUnion.event_union(ivl);
      	}
	
	}
	
  	return ivlUnion;
}

// --------------------------------

bool QjuiWindow::getScalarIntervalFromData(double &start, double &end)
{
	bool OK = false;
	double obj_start, obj_end;
	int numRows=_join_widget->count();
	if(numRows < 1) return false;
	
	QjuiWidgetItem * item;
	for(int i=0; i<numRows; ++i)
	{
		item=(QjuiWidgetItem*)_join_widget->item(i);
		DvObject_var ivl_obj = getWLObject(item->WLcontext.toLatin1().data());
		if(ivl_obj.is_ok() ) {
			DvObject_var D0 = ivl_obj->getDep0();
		
			if( D0.is_ok() ){
				obj_start = D0->asDouble(0);
				obj_end = D0->asDouble(D0->seqSize()-1);
				
				if(OK){
					if(start < end){
						if( start > obj_start ) start = obj_start;
						if( end < obj_end ) end = obj_end;
					}
					else{
						if( start < obj_start ) start = obj_start;
						if( end > obj_end ) end = obj_end;
					}
				}
				else{
					// first valid data
					start = obj_start;
					end = obj_end;
				}
			}
			OK=true;
		}
	}
  
	return OK;
}

// --------------------------------

DvObject_var QjuiWindow::getTagsFromObj()
{
	DvObject_var ts;
	// tags from object
    DvObject_var tag_obj = _TagSource->getObject();

	if(_TimeTagSelect->currentIndex() ==  2 || tag_obj.is_nil()){
		// use first object in list
		QjuiWidgetItem* item=(QjuiWidgetItem*)(_join_widget->item(0));
		if(item != 0) tag_obj = getWLObject(item->WLcontext.toLatin1().data());		
	}
    if(tag_obj.is_ok()) ts = tag_obj->getDep0();


	return ts;
}


// --------------------------------

DvObject_var QjuiWindow::getTimeTags(){

	DvObject_var tags;
	DvEvent ivl;
	double Spacing;
	double Tinterval;
	double start, end;

	// set interval

	ivl = this->getInterval();
	
	if ( ivl.empty() ) {
	
		// look for scalar Depend_0
		
  		if(this->getScalarInterval(start, end) == false) return tags; // tags will be nil
		
		Tinterval = end - start;
		// set scalar tags

  		if( _TimeTagSelect->currentIndex() == 0)
  		{
    		// create regular tags
    		bool OK;
    		Spacing = _TagSource->text().toDouble(&OK);
    		if(!OK || fabs(Spacing) < 1.e-20) return tags;

			if(Tinterval < 0) Spacing = -fabs(Spacing); // ensure spacing -ve if decreasing sequence
			
    		int Ntags = (int) (fabs(Tinterval/Spacing) + 1.0);
	
			tags = new DvObject(start, Ntags);
			tags->assign_regular(Spacing); // start is unchanged, other values incremented regularly
		}
  		else
  		{
    		// tags from object
    		DvObject_var ss = getTagsFromObj();
    		if(ss.is_nil())  return tags;    // will be .is_nil()
			
    		// truncate to interval if necessary
				
			int recL, recU;
			if( ss->getRecBounds(recL, recU,  start,  end) ){
				tags = ss->subSequence(recL, recU);
    		}
		}
  	}
  	else {
		Tinterval = ivl.duration_abs();
  		// set tags

  		if( _TimeTagSelect->currentIndex() == 0)
  		{
    		// create regular tags
    		bool OK;
    		Spacing = _TagSource->text().toDouble(&OK);
    		if(!OK || Spacing < 1.e-20) return tags;

    		int Ntags = (int) (Tinterval/Spacing + 1.0);
	
			tags = new DvObject(ivl.start(), Ntags);
			tags->assign_regular(Spacing); // start is unchanged, other values incremented regularly
		}
  		else
  		{
    		// tags from object
    		DvObject_var ts = getTagsFromObj();
    		if(ts.is_nil())  return tags;    // will be .is_nil()

    		// truncate to interval if necessary
			tags = ts->subSequence(ivl);
		}
	}

	return tags;

}


void QjuiWindow::messageHandler(QString txt){

  this->status(txt, 10000);
}

QDomElement QjuiWidgetItem::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("Join_Item");
    xml.appendChild(createTextXMLNode(doc, "Object", WLcontext));
    xml.appendChild(createTextXMLNode(doc, "Algorithm", I_algorithm));
    xml.appendChild(createTextXMLNode(doc, "Width", I_width_field));
    xml.appendChild(createTextXMLNode(doc, "Gap_Handling", I_gap));
    xml.appendChild(createTextXMLNode(doc, "Gap_Threshold", I_gap_field));

    return xml;
}

void QjuiWidgetItem::fromXMLElement(QDomElement& el)
{
    if(!el.isNull())
    {
		QString s;
		if(getXMLNodeText(el, "Object", s))
		{
			setText(s);
			WLcontext = s;
		}
//         QDomElement object=el.firstChildElement("Object");
//         if(!object.isNull())
//         {
//             setText(object.firstChild().nodeValue());
//             WLcontext = object.firstChild().nodeValue();
//         }

		if(getXMLNodeText(el, "Algorithm", s))
		{
			I_algorithm=s.toInt();
		}
		if(getXMLNodeText(el, "Width", s))
		{
			I_width_field=s;
		}
		if(getXMLNodeText(el, "Gap_Handling", s))
		{
			I_gap=s.toInt();
		}
		if(getXMLNodeText(el, "Gap_Threshold", s))
		{
			I_gap_field=s;
		}
//         QDomElement algo=el.firstChildElement("Algorithm");
//         if(!algo.isNull()) I_algorithm=algo.firstChild().nodeValue().toInt();
// 
//         QDomElement width=el.firstChildElement("Width");
//         if(!width.isNull()) I_width_field=width.firstChild().nodeValue();
// 
//         QDomElement gap=el.firstChildElement("Gap_Handling");
//         if(!gap.isNull()) I_gap=gap.firstChild().nodeValue().toInt();
// 
//         QDomElement gapT=el.firstChildElement("Gap_Threshold");
//         if(!gapT.isNull()) I_gap_field=gapT.firstChild().nodeValue();
    }
}


QDomElement QjuiWidget::toXMLElement(QDomDocument& doc) const
{
    QDomElement xml=doc.createElement("List");
    for(int i=0; i<count(); ++i)
    {
        xml.appendChild(((QjuiWidgetItem*)item(i))->toXMLElement(doc));
    }
    return xml;
}

void QjuiWidget::fromXMLElement(QDomElement& el)
{
    if(!el.isNull())
    {
        QDomElement item=el.firstChildElement("Join_Item");
        while(!item.isNull())
        {
            QjuiWidgetItem *i=new QjuiWidgetItem("", this);
            i->fromXMLElement(item);
            item=item.nextSiblingElement("Join_Item");
        }
    }
}

