//
// Data Slice Widget
//
// Nov 2012: Tony Allen
//

#include <stdlib.h>

#include <qpushbutton.h>
#include <qlayout.h>
#include "qops.h"
#include "qwui.h"
#include "DvObj.h"

extern qwuiWindow* qsasWindow;

using namespace QSAS;

MergeTSWindow::MergeTSWindow(QWidget *parent) 
	: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	if(QuSlotDebug())
	{
		printf("MergeTSWindow::MergeTSWindow \n");
		fflush(stdout);
	}
  	this->setWindowTitle("Merge Time Series tool");
  
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slots
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QVBoxLayout * slotLayout=new QVBoxLayout;
	slotPanel->setTitle("TS Data Objects to Merge");
	
	objField1 = new QguWLinput(slotPanel,"objField1");
	objField1->setAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_SEQ, QS_TT));
	objField1->setEnabled(true);
	objField1->setMinimumWidth(char_width*55);
	slotLayout->addWidget(objField1);
	
	objField2 = new QguWLinput(slotPanel,"objField2");
	objField2->setAcceptable(Qcondition(QS_NARROW, QS_ANY, QS_ANY, QS_SEQ, QS_TT));
	objField2->setEnabled(true);
	objField2->setMinimumWidth(char_width*55);
	slotLayout->addWidget(objField2);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setText("MergedTS");
	name_field->setMinimumWidth(char_width*50);
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	doBtn = new QPushButton( "Merge Objects ");
	doBtn->setMinimumSize(doBtn->sizeHint());
	connect( doBtn, SIGNAL(clicked()), this, SLOT(merge()) );
	doBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(doBtn);
	doBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
		
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}


void MergeTSWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("MergeTSWindow::setObject \n");
	fflush(stdout);
 }
  
   objField1->setText(WLctxt);
   objField1->reValidate();
   
}


void MergeTSWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("MergeTSWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( doBtn->isDefault()  ) {
		   doBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   doBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(doBtn->isDefault()) merge();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void MergeTSWindow::merge(){
      
	if(QuSlotDebug())
	{
		printf("MergeTSWindow::merge \n");
		fflush(stdout);
	}

	DvObject_var dobj1 = objField1->getObject();
	DvObject_var dobj2 = objField2->getObject();
 
	DvObject_var resObj = dobj1->mergeWith(dobj2);

	if( resObj.is_ok()) // ensures error message is not overwritten by empty obj rejected
	{
		QString newName = name_field->text();
		QwuiPutOnWL(resObj, newName);
	}
	
	this->close();
	return;
}

// ----------------------------------------------

ArrayUnaryWindow::ArrayUnaryWindow(int op) 
	: QMainWindow(0) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{

	if(QuSlotDebug())
	{
		printf("ArrayUnaryWindow::ArrayUnaryWindow \n");
		fflush(stdout);
	}
    opFlag = op;
    
	this->setWindowTitle("");

	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
    if(opFlag == -1) slotPanel->setTitle("Matrix Inverse of Square 2D Array (same as using Arithmetic->Inverse method on a square array)");
    else if(opFlag == 0) slotPanel->setTitle("Trace of Square 2D Array");
    else if(opFlag == 1) slotPanel->setTitle("Transpose of 2D Array, M x N -> N x M ");
    else if(opFlag == 2) slotPanel->setTitle("Determinant of Square 2D Array ");
    else if(opFlag == 3) slotPanel->setTitle("Adjoint of Square 2D Array ");
    else if(opFlag == 4) slotPanel->setTitle("Normalize Vector ");
    
    
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_2D, QS_ANY, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Evaluate ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOp()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);
	saveBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

void ArrayUnaryWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}



void ArrayUnaryWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate(true);
   
}

void ArrayUnaryWindow::setNewName(){
     
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::setNewName \n");
	fflush(stdout);
 }

   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
       QString newName = QString(objName);
       
     int ptr = newName.lastIndexOf("/");
     if (ptr >= 0) newName.remove(0, ptr+1);
	 ptr = newName.indexOf(" [");
	 if (ptr > 0) newName.truncate(ptr);
       
     if(opFlag == 0) newName += "_Trace";
     else  if(opFlag == 1) newName += "_Transpose";
     else  if(opFlag == 2) newName += "_Det";
     else  if(opFlag == 3) newName += "_Adj";
     else  if(opFlag == -1) newName += "_Inv";
     else  if(opFlag == 4) newName += "_unit";
       
     this->name_field->setText(newName);
   }
   

}

void ArrayUnaryWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOp();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void ArrayUnaryWindow::acceptOp(){
      
	if(QuSlotDebug())
	{
		printf("ArrayTraceWindow::acceptOp \n");
		fflush(stdout);
	}
  
	DvObject_var dobj = objField->getObject();

    DvObject_var retObj;
    if(opFlag == -1) retObj = dobj->matrixInverse();
    else if(opFlag == 0) retObj = dobj->trace();
    else if(opFlag == 1) retObj = dobj->transpose();
    else if(opFlag == 2) retObj = dobj->det();
    else if(opFlag == 3) retObj = dobj->adjoint();
    else if(opFlag == 4) retObj = dobj->normalize();

	QString newName = name_field->text();
	if( retObj.is_nil() ) QuMsgBar(retObj->getError(0).c_str(), true);
	else QwuiPutOnWL(retObj, newName); 

	this->close();
	return;
}



// ----------------------------------------------

ArrayBinaryWindow::ArrayBinaryWindow(int op) 
: QMainWindow(0) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
    
    if(QuSlotDebug())
    {
        printf("PiecewiseWindow::PiecewiseWindow \n");
        fflush(stdout);
    }
  
    opFlag = op;
    
    this->setWindowTitle("");
    
    this->setPalette(qApp->palette());
    
    QFontMetrics qgu_font_mt(qApp->font());
    int char_width = qgu_font_mt.width("n");
    
    setAttribute(Qt::WA_DeleteOnClose);
    
    msgBar = statusBar();
    
    QGroupBox * mainWidget=new QGroupBox(this);
    QVBoxLayout * mainLayout=new QVBoxLayout;
    
    // Data slot
    
    QGroupBox * slotPanel=new QGroupBox(mainWidget);
    QVBoxLayout * slotLayout=new QVBoxLayout;
    if(opFlag == 0) slotPanel->setTitle("Piecewise Array Multiplication");
    else if(opFlag == 1) slotPanel->setTitle("Piecewise Array Division");
    else if(opFlag == 2) slotPanel->setTitle("Element at index i, A(i), in range [1,n]");
    else if(opFlag == 3) slotPanel->setTitle("Element at indices, A(i,j,...,n). Number of indices must match dimension. Indices count from 1");
    else if(opFlag == 4) slotPanel->setTitle("Remove element at indix i of 1D array Indices count from 1");
    else if(opFlag == 5) slotPanel->setTitle("Outer product R(i,j, ..., k,l,...) = A(i,j,...) * B(k,l,...) ");
    else if(opFlag == 6) slotPanel->setTitle("Inner product (same as dot() and multiply() where appropriate) ");
    else if(opFlag == 7) slotPanel->setTitle(" aTan2(y,x) Degrees");
    else if(opFlag == 8) slotPanel->setTitle(" aTan2(y,x) Radians");
    
    objA = new QguWLinput(slotPanel,"objA");
    objA->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_ANY, QS_ANY));
    objA->setEnabled(true);
    objA->setMinimumWidth(char_width*55);
    slotLayout->addWidget(objA);

    objB = new QguWLinput(slotPanel,"objB");
    if(opFlag == 2 || opFlag == 4 ) {
        objB->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
        objB->setText("1");
    }
    else if(opFlag == 3) {
        objB->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
        objB->setText("1, 2, 3");
    }
    else objB->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_ANY, QS_ANY));
    objB->setEnabled(true);
    objB->setMinimumWidth(char_width*55);
    slotLayout->addWidget(objB);
    
    slotPanel->setLayout(slotLayout);
    mainLayout->addWidget(slotPanel);
    
    // "Save as" panel
    
    QGroupBox * name_panel=new QGroupBox(mainWidget);
    QHBoxLayout * name_layout=new QHBoxLayout;
    
    QLabel *name_label = new QLabel("Save as ", name_panel);
    name_layout->addWidget(name_label);
    
    name_field = new QguWLinput(name_panel, "name_field");
    name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
    name_field->setEnabled(true);
    if(opFlag == 0) name_field->setText("Multiplied");
    else if(opFlag == 1) name_field->setText("Divided");
    else if(opFlag == 2) name_field->setText("Element_i");
    else if(opFlag == 3) name_field->setText("Element");
    else if(opFlag == 4) name_field->setText("subDimension");
    else if(opFlag == 5) name_field->setText("OuterProduct");
    else if(opFlag == 6) name_field->setText("InnerProduct");
    else if(opFlag == 7) name_field->setText("angleTanDeg");
    else if(opFlag == 8) name_field->setText("angleTanRad");

    name_field->setMinimumWidth(char_width*50);
    name_layout->addWidget(name_field);
    
    name_panel->setLayout(name_layout);
    mainLayout->addWidget(name_panel);
    
    // Buttons Layout
    
    QHBoxLayout * buttons_layout=new QHBoxLayout;
    
    saveBtn = new QPushButton( "Evaluate ");
    saveBtn->setMinimumSize(saveBtn->sizeHint());
    connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOp()) );
    saveBtn->setFocusPolicy(Qt::StrongFocus);
    buttons_layout->addWidget(saveBtn);
    saveBtn->setDefault(true);
    
    dismissBtn = new QPushButton( "Dismiss");
    dismissBtn->setMinimumSize(dismissBtn->sizeHint());
    connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
    buttons_layout->addWidget(dismissBtn);
    
    mainLayout->addLayout(buttons_layout);	
    
    mainWidget->setLayout(mainLayout);
    setCentralWidget(mainWidget);
    
    this->setMinimumSize(this->sizeHint());
  
}



void ArrayBinaryWindow::setObject(const char * WLctxt){
    
    if(QuSlotDebug())
    {
        printf("ArrayTraceWindow::setObject \n");
        fflush(stdout);
    }
    
    objA->setText(WLctxt);
    objA->reValidate(true);
    
}


void ArrayBinaryWindow::keyPressEvent(QKeyEvent * e )
{  
    
    if(QuSlotDebug())
    {
        printf("ArrayTraceWindow::keyPressEvent \n");
        fflush(stdout);
    }
    
    // use escape key to toggle default buttons
    if(e->key() == Qt::Key_Escape){
        if( saveBtn->isDefault()  ) {
            saveBtn->setDefault(false);
            dismissBtn->setDefault(true);
        }
        else if(dismissBtn->isDefault()  )  {
            dismissBtn->setDefault(false);
            saveBtn->setDefault(true);
        }
        return;
    }
    // use return or enter key to perform default
    if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
        if(saveBtn->isDefault()) acceptOp();
        else if (dismissBtn->isDefault()) close();
        return;
    }
    
    // else pass on to list view 
    e->ignore(); // let selection change
    
}

void ArrayBinaryWindow::acceptOp(){
    
    if(QuSlotDebug())
    {
        printf("ArrayTraceWindow::acceptOp \n");
        fflush(stdout);
    }
   
    DvObject_var dobjA = objA->getObject();
    DvObject_var dobjB = objB->getObject();

    DvObject_var retObj;

    if(opFlag == 0) retObj = dobjA->multElements(dobjB);
    else if(opFlag == 1) retObj = dobjA->divideElements(dobjB);
    else if(opFlag == 2) {
        int i = dobjB->asInt()-1;
        if(i<0) {
            statusBar()->showMessage("Array indices must be > 1 (not C style)");
            return;
        }
        if(dobjA->nDims() != 1 || i >= dobjA->Dims()[0]) {
            statusBar()->showMessage("Array index out of range");
            return;
        }
        retObj = dobjA->element((size_t)i);
    }
    else if(opFlag == 3) { 
        // of the form "1,2,...,n"
        size_t idx=0;
        size_t n = dobjA->nDims();
        int j=0;
        
        DvString str_idx = dobjB->asStr();
        for(size_t i=0; i<n-1; i++) {
            j = str_idx.before(',').toInt()-1;
            if(j < 0) {
                statusBar()->showMessage("Array indices must be > 1 (not C style)");
                return;
            }
           idx += (size_t)j;
            str_idx = str_idx.after(',');
            idx *= dobjA->dims[i+1];
        }
        j = str_idx.toInt()-1;
        if(j < 0) {
            statusBar()->showMessage("Array indices must be > 1 (not C style)");
            return;
        }
        idx += j;
     
        retObj = dobjA->element(idx);
    }
    else if(opFlag == 4) {
        int i = dobjB->asInt()-1;
        if(i<0) {
            statusBar()->showMessage("Array indices must be > 1 (not C style)");
            return;
        }
        if(dobjA->nDims() != 1 || i >= dobjA->Dims()[0]) {
            statusBar()->showMessage("Array index out of range");
            return;
        }
        retObj = dobjA->subDimension((size_t)i, true);
    }
    else if(opFlag == 5) {
        retObj = dobjA->outerProduct(dobjB);
    }
    else if(opFlag == 6) {
        retObj = dobjA * dobjB;
    }
    else if(opFlag == 7) {
        retObj = dobjA->atan2(dobjB, true);
    }
    else if(opFlag == 8) {
        retObj = dobjA->atan2(dobjB, false);
    }
    else{
        return;
    }
   
    QString newName = name_field->text();
    if( retObj.is_nil() ) QuMsgBar(retObj->getError(0).c_str(), true);
    else QwuiPutOnWL(retObj, newName); 
  
    this->close();
    return;
}

// ----------------------------------------------

ArrayTriWindow::ArrayTriWindow(int op) 
: QMainWindow(0) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
    
    if(QuSlotDebug())
    {
        printf("PiecewiseWindow::PiecewiseWindow \n");
        fflush(stdout);
    }
    
    opFlag = op;
    
    this->setWindowTitle("");
    
    this->setPalette(qApp->palette());
    
    QFontMetrics qgu_font_mt(qApp->font());
    int char_width = qgu_font_mt.width("n");
    
    setAttribute(Qt::WA_DeleteOnClose);
    
    msgBar = statusBar();
    
    QGroupBox * mainWidget=new QGroupBox(this);
    QVBoxLayout * mainLayout=new QVBoxLayout;
    
    // Data slot
    
    QGroupBox * slotPanel=new QGroupBox(mainWidget);
    QVBoxLayout * slotLayout=new QVBoxLayout;
    if(opFlag == 0) slotPanel->setTitle("Sub Matrix, ( matrix with row i and col j removed). Indices start at 1.");
        
    objA = new QguWLinput(slotPanel,"objA");
    objA->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_ANY, QS_ANY));
    objA->setEnabled(true);
    objA->setMinimumWidth(char_width*50);
    slotLayout->addWidget(objA);
 
    slotPanel->setLayout(slotLayout);
    mainLayout->addWidget(slotPanel);

    QGroupBox * slotBPanel=new QGroupBox(mainWidget);
    QHBoxLayout * slotBLayout=new QHBoxLayout;

    lab_i = new QLabel("      i");
    slotBLayout->addWidget(lab_i, Qt::AlignRight);

    objB = new QguWLinput(slotBPanel,"objB");
    objB->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
    objB->setText("1");
    
    objB->setEnabled(true);
    objB->setMinimumWidth(char_width*20);
    slotBLayout->addWidget(objB, Qt::AlignLeft);

    lab_j = new QLabel("      j");
    slotBLayout->addWidget(lab_j, Qt::AlignRight);

    objC = new QguWLinput(slotBPanel,"objC");
    objC->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
    objC->setText("1");
    
    objC->setEnabled(true);
    objC->setMinimumWidth(char_width*20);
    slotBLayout->addWidget(objC, Qt::AlignLeft);
    
    slotBLayout->setStretch(0,1);
    slotBLayout->setStretch(1,20);
    slotBLayout->setStretch(2,1);
    slotBLayout->setStretch(3,20);
    
    slotBPanel->setLayout(slotBLayout);
    mainLayout->addWidget(slotBPanel);

    // "Save as" panel
    
    QGroupBox * name_panel=new QGroupBox(mainWidget);
    QHBoxLayout * name_layout=new QHBoxLayout;
    
    QLabel *name_label = new QLabel("Save as ", name_panel);
    name_layout->addWidget(name_label);
    
    name_field = new QguWLinput(name_panel, "name_field");
    name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
    name_field->setEnabled(true);
    if(opFlag == 0) name_field->setText("subMat");
   
    
    name_field->setMinimumWidth(char_width*50);
    name_layout->addWidget(name_field);
    
    name_panel->setLayout(name_layout);
    mainLayout->addWidget(name_panel);
    
    // Buttons Layout
    
    QHBoxLayout * buttons_layout=new QHBoxLayout;
    
    saveBtn = new QPushButton( "Evaluate ");
    saveBtn->setMinimumSize(saveBtn->sizeHint());
    connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOp()) );
    saveBtn->setFocusPolicy(Qt::StrongFocus);
    buttons_layout->addWidget(saveBtn);
    saveBtn->setDefault(true);
    
    dismissBtn = new QPushButton( "Dismiss");
    dismissBtn->setMinimumSize(dismissBtn->sizeHint());
    connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
    buttons_layout->addWidget(dismissBtn);
    
    mainLayout->addLayout(buttons_layout);	
    
    mainWidget->setLayout(mainLayout);
    setCentralWidget(mainWidget);
    
    this->setMinimumSize(this->sizeHint());
    
}



void ArrayTriWindow::setObject(const char * WLctxt){
    
    if(QuSlotDebug())
    {
        printf("ArrayTraceWindow::setObject \n");
        fflush(stdout);
    }
    
    objA->setText(WLctxt);
    objA->reValidate(true);
    
    DvObject_var dobjA = objA->getObject();
    if(dobjA.is_ok() && dobjA->nDims() == 2){
        DvString labi("i (1,");
        labi += dobjA->Dims()[0];
        labi += ")";
        lab_i->setText(labi.c_str());
        
        DvString labj("j (1,");
        labj += dobjA->Dims()[1];
        labj += ")";
        lab_j->setText(labj.c_str());
    }
}


void ArrayTriWindow::keyPressEvent(QKeyEvent * e )
{  
    
    if(QuSlotDebug())
    {
        printf("ArrayTraceWindow::keyPressEvent \n");
        fflush(stdout);
    }
    
    // use escape key to toggle default buttons
    if(e->key() == Qt::Key_Escape){
        if( saveBtn->isDefault()  ) {
            saveBtn->setDefault(false);
            dismissBtn->setDefault(true);
        }
        else if(dismissBtn->isDefault()  )  {
            dismissBtn->setDefault(false);
            saveBtn->setDefault(true);
        }
        return;
    }
    // use return or enter key to perform default
    if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
        if(saveBtn->isDefault()) acceptOp();
        else if (dismissBtn->isDefault()) close();
        return;
    }
    
    // else pass on to list view 
    e->ignore(); // let selection change
    
}

void ArrayTriWindow::acceptOp(){
    
    if(QuSlotDebug())
    {
        printf("ArrayTraceWindow::acceptOp \n");
        fflush(stdout);
    }
    
    DvObject_var dobjA = objA->getObject();
    DvObject_var dobjB = objB->getObject();
    DvObject_var dobjC = objC->getObject();
    
    DvObject_var retObj;
    
    if(opFlag == 0) {
        int i = dobjB->asInt() - 1;
        int j = dobjC->asInt() - 1;
        if(i < 0 || j < 0) { 
            statusBar()->showMessage("Array indices must be > 1 (not C style)");
            return;
        }
        if(i >= dobjA->dims[0]  ) { 
            statusBar()->showMessage("Array index i too large");
            return;
        }
        if( j >= dobjA->dims[1] ) { 
            statusBar()->showMessage("Array index j too large");
            return;
        }
        retObj = dobjA->subMatrix((size_t)i, (size_t)j);
    }
    
    QString newName = name_field->text();
    if( retObj.is_nil() ) QuMsgBar(retObj->getError(0).c_str(), true);
    else QwuiPutOnWL(retObj, newName); 
    
    this->close();
    return;
}





// ----------------------------------------------

AngleDepWindow::AngleDepWindow(int angTop) 
	: QMainWindow() 
{

	if(QuSlotDebug())
	{
		printf("AngleDepWindow::AngleDepWindow \n");
		fflush(stdout);
	}
    top = angTop;
	newCentre = false;
	this->setWindowTitle(" ");

	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	newCentreFld=NULL;
	
	if(angTop == 360) this->setWindowTitle("Convert Angle Depend to use 0 to 360 deg");
	else if(angTop == 180) this->setWindowTitle("Convert Angle Depend to use -180 to 180 deg");
	else if(angTop == 90) this->setWindowTitle("Convert Angle Depend to use -90 to 90 deg");
	else if(angTop == 0) this->setWindowTitle("Convert Angle Depend to use 0 to 180 deg");
	else {
		newCentre = true;
		this->setWindowTitle("Shift Angle Depend to new centre");
	}
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Data Object");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition());
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	connect(objField, SIGNAL(hadDropEvent()), this, SLOT(setAccept()));
	slotLayout->addWidget(objField);
	
	QLabel *dep_label = new QLabel("Depend", slotPanel);
	slotLayout->addWidget(dep_label);
	depField = new QSpinBox(slotPanel);
	depField->setRange(1,4);
	slotLayout->addWidget(depField);
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);

	if(newCentre){
		QGroupBox * centrePanel=new QGroupBox(mainWidget);
		QHBoxLayout * centreLayout=new QHBoxLayout;
		centrePanel->setTitle("Centre Angle Depend at...");
	
		newCentreFld = new FloatSpinBox(-360., 360., 2, 10., centrePanel, "centre" );
		centreLayout->addWidget(newCentreFld);
		centrePanel->setLayout(centreLayout);
		mainLayout->addWidget(centrePanel);
	}
	
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	name_field->setText("modifiedObj");
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Save ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);
	saveBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

void AngleDepWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}



void AngleDepWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("DataSliceWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   setAccept(); // ensure correct drop down if opened with preselected object
   objField->reValidate();
   
}

void AngleDepWindow::setAccept(){
    
	if(QuSlotDebug())
	{
		printf("AngleDepWindow::setAccept \n");
		fflush(stdout);
	}
  
	DvObject_var dobj = objField->getObject();

	// special case of event table
	if(dobj->is_event()) 
	{
		objField->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_ANY, QS_ANY));
		objField->reValidate();
	}
    else objField->setAcceptable(Qcondition());
   
}

void AngleDepWindow::setNewName(){
     
 if(QuSlotDebug())
 {
	printf("AngleDepWindow::setNewName \n");
	fflush(stdout);
 }

   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
       QString newName = QString(objName);
       
     int ptr = newName.lastIndexOf("/");
     if (ptr >= 0) newName.remove(0, ptr+1);
	 ptr = newName.indexOf(" [");
	 if (ptr > 0) newName.truncate(ptr);
     newName += QString("_M");
     this->name_field->setText(newName);
   }
   

}

void AngleDepWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("AngleDepWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOperation();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void AngleDepWindow::acceptOperation(){
      
	if(QuSlotDebug())
	{
		printf("AngleDepWindow::acceptOperation \n");
		fflush(stdout);
	}
  
	DvObject_var dobj = objField->getObject();
	
	int i = depField->value();
	if(i > dobj->nDims()){
		QString text = QString("dependency %1 out of range, object dimension only %2D").arg(i).arg(dobj->nDims());
		statusBar()->showMessage(text);
		return;
	}
	
	QString newName = name_field->text();
	
	if( dobj.is_ok() ) 
	{   
		// copy object
		DvObject_var newObject = new DvObject(dobj);

		// find DEPEND_i
		QString dep = QString("DEPEND_%1").arg(i);
		if( !newObject->xref_exists(dep.toLatin1().data()) ) {
			QString text = QString("Object has no Depend_%1").arg(i);
			statusBar()->showMessage(text);
			return;
		}
		
		DvObject_var xref = newObject->get_xref(dep.toLatin1().data());
			
		// If angle DEPEND modify
		DvString angle = xref->angleUnitStr();
		if( angle.empty() ) {
			QString text = QString("dependency %1 is not an angle").arg(i);
			statusBar()->showMessage(text);
			return;
		}

		if(newCentre){
			double centre = 0.0;
			if(newCentreFld) centre = newCentreFld->value();
			
			DvObject_var newXref = new DvObject(xref);
			*newXref -= centre;
			newObject->change_xref(dep.toLatin1().data(), newXref);
		}
		else {
			DvObject_var newXref = xref->angleMod(top, angle);
			newObject->change_xref(dep.toLatin1().data(), newXref);
		}
		
		QwuiPutOnWL(newObject, newName);
			

	}
	else QuMsgBar("NULL input object", true);
	
	this->close();
	return;
}

void AngleDepWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("AngleDepWindow::dismiss \n");
		fflush(stdout);
	}
	
	this->close();
	return;
}



// -------------Derivative Window class ---------------------------------

DerivativeWindow::DerivativeWindow() 
	: QMainWindow() 
{

	if(QuSlotDebug())
	{
		printf("DerivativeWindow::DerivativeWindow \n");
		fflush(stdout);
	}
	this->setWindowTitle("Derivative Estimator");
	
  	QMenu * help = new QMenu( this );
	this->menuBar()->addMenu(help );
	help->addAction("Differentiation in QSAS", this, SLOT( launchHelp() ));
	help->setTitle(" Help ");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
     
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Differentiate...");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_ANY, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// Method panel
	
	QGroupBox * method_panel=new QGroupBox(mainWidget);
	QVBoxLayout * method_layout=new QVBoxLayout;
	
	derivMethod = new QComboBox(method_panel);
	derivMethod->addItem(DV_DERIV_3PT);
	derivMethod->addItem(DV_DERIV_5PT );
	method_layout->addWidget( derivMethod, 0,Qt::AlignLeft );
	
	
	methodLabl = new QLabel("(f(x+h) - f(x-g) ) / (h + g) \nTwo point estimate used at edges and gaps");
	method_layout->addWidget( methodLabl, 1, Qt::AlignLeft );

	connect( derivMethod,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setMethod(const QString &) ) );
	
	method_panel->setLayout(method_layout);
	mainLayout->addWidget(method_panel);

	// Gap panel
	
	QGroupBox * gap_panel=new QGroupBox(mainWidget);
	QHBoxLayout * gap_layout=new QHBoxLayout;
	
	QLabel *gapLabl = new QLabel("Gap Threshold: ");
	gap_layout->addWidget( gapLabl, 0, Qt::AlignLeft );

	gapSizeField = new QguWLinput(gap_panel,"gapField");	
	gapSizeField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
	gapSizeField->setToolTip("Treat spacing above this size as a gap (time gaps are in seconds)");
	gapSizeField->setDouble(6.0);

	gap_layout->addWidget( gapSizeField, 1, Qt::AlignLeft );
		
	gap_panel->setLayout(gap_layout);
	mainLayout->addWidget(gap_panel);

// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Do Derivative ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(doDerivative()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);
	saveBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

DerivativeWindow::~DerivativeWindow() 
{
}
// -------------

void DerivativeWindow::launchHelp() 
{
	 if(QuSlotDebug())
 	{
		printf("DerivativeWindow::launchHelp \n");
		fflush(stdout);
	 }
 
  	qsasWindow->help_window->manEntry("QSAS_analysis.html","#differentiation");

}
// -------------

void DerivativeWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}

// -------------

void DerivativeWindow::setMethod(const QString & text){
    
	if( text.contains("3") ) {
		methodLabl->setText("(f(x+h) - f(x-g) ) / (h + g) \nTwo point estimate used at edges and gaps");
		gapSizeField->setEnabled(true);
	}
	else 	{
		methodLabl->setText("(-f(x+2h) + 8f(x+h) - 8f(x-h) + f(x-2h) ) / 12h \nWill join onto regularly spaced sequence first");
		gapSizeField->setEnabled(false);
	}
}


// -------------

void DerivativeWindow::setNewName(){
     
	if(QuSlotDebug())
	{
		printf("DerivativeWindow::setNewName \n");
		fflush(stdout);
	}
	
	// first set default gap
	DvObject_var dobj = objField->getObject();

	DvObject_var d0 = dobj->getDep0();
		
	double spacing=0;
	if(d0.is_ok() && d0->seqSize() > 1) spacing = d0->asDouble(1) - d0->asDouble(0);
	
	if(spacing > 0) gapSizeField->setDouble(spacing *1.5);
	
	// set output name default
   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
		QString newName = QString(objName);
       
		int ptr = newName.lastIndexOf("/");
		if (ptr >= 0) newName.remove(0, ptr+1);
		ptr = newName.indexOf(" [");
		if (ptr > 0) newName.truncate(ptr);

		this->name_field->setText("d_"+newName);
   }

}

// -------------

void DerivativeWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("DerivativeWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) doDerivative();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

// -------------

void DerivativeWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("DerivativeWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate(true);
   
}

// -------------

void DerivativeWindow::doDerivative(){
      
	if(QuSlotDebug())
	{
		printf("DerivativeWindow::doDerivative \n");
		fflush(stdout);
	}
	qApp->processEvents();
	
	double gapSize = gapSizeField->value();
	DvString method = DvString(DV_DERIV_3PT);
	if(derivMethod->currentIndex() == 1) method = DvString(DV_DERIV_5PT);
	
	DvObject_var dobj = objField->getObject();
	
	// do derivative (DVOS handles metadata)
	
	DvObject_var retObj = dobj->differentiate(method, gapSize);
	
	if( retObj.is_nil() )  {
		statusBar()->showMessage("NULL object from derivative");
		return;
	}

	QString newName = name_field->text();
	QwuiPutOnWL(retObj, newName);

	qApp->processEvents();
	
	this->close();

}



// -------------Create Regular Window class ---------------------------------

CreateRegularWindow::CreateRegularWindow() 
: QMainWindow() 
{
    
    if(QuSlotDebug())
    {
        printf("CreateRegularWindow::CreateRegularWindow \n");
        fflush(stdout);
    }
    this->setWindowTitle("Create Regular Object");
    
    this->setPalette(qApp->palette());
    
    QFontMetrics qgu_font_mt(qApp->font());
    int char_width = qgu_font_mt.width("n");
    
    setAttribute(Qt::WA_DeleteOnClose);
    
    QGroupBox * mainWidget=new QGroupBox(this);
    QVBoxLayout * mainLayout=new QVBoxLayout;
    
    // Type panel
    
    QGroupBox * type_panel=new QGroupBox(mainWidget);
    QHBoxLayout * type_layout=new QHBoxLayout;
    
    type_panel->setTitle("Object Type");
    
    objType = new QComboBox(type_panel);
    objType->addItem(DV_TIME);
    objType->addItem(DV_DOUBLE);
    objType->addItem(DV_INT);
    type_layout->addWidget( objType, 0,Qt::AlignLeft );
    
    QLabel * numberLbl = new QLabel("Number of records: ");
    type_layout->addWidget( numberLbl, 1, Qt::AlignRight );

    numberField = new QguWLinput(type_panel,"numberField");	
    numberField->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
    numberField->setToolTip("number of entries to create");
    type_layout->addWidget( numberField, 1, Qt::AlignLeft );
    numberField->setText("10");
    
    connect( objType,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setType(const QString &) ) );
    
    type_panel->setLayout(type_layout);
    mainLayout->addWidget(type_panel);
    

    // start panel
    
    QGroupBox * start_panel=new QGroupBox(mainWidget);
    QHBoxLayout * start_layout=new QHBoxLayout;
    
    QLabel *startLabl = new QLabel("Start at: ");
    start_layout->addWidget( startLabl, 0, Qt::AlignLeft );
    
    startField = new QguWLinput(start_panel,"startField");	
    startField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
    startField->setToolTip("Value of first record, accepts text or object");
    startField->setText("2001-07-01 12:00:00.00");
    start_layout->addWidget( startField, 1, Qt::AlignLeft );
        
    start_panel->setLayout(start_layout);
    mainLayout->addWidget(start_panel);

    // data panel
    
    QGroupBox * data_panel=new QGroupBox(mainWidget);
    QHBoxLayout * data_layout=new QHBoxLayout;
    
    QLabel *spacingLabl = new QLabel("Spacing: ");
    data_layout->addWidget( spacingLabl, 0, Qt::AlignLeft );
    
    spacingField = new QguWLinput(data_panel,"spacingField");	
    spacingField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
    spacingField->setToolTip("regular spacing between values");
    spacingField->setDouble(4.0);
    data_layout->addWidget( spacingField, 1, Qt::AlignLeft );
    
    unitLabl = new QLabel(" (s)");
    data_layout->addWidget( unitLabl, 0, Qt::AlignLeft );
    
    data_panel->setLayout(data_layout);
    mainLayout->addWidget(data_panel);

    // "Save as" panel
    
    QGroupBox * name_panel=new QGroupBox(mainWidget);
    QHBoxLayout * name_layout=new QHBoxLayout;
    
    QLabel *name_label = new QLabel("Save as ", name_panel);
    name_layout->addWidget(name_label);
    
    name_field = new QguWLinput(name_panel, "name_field");
    name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
    name_field->setEnabled(true);
    name_field->setMinimumWidth(char_width*50);
    userSet = false;
    name_field->setText("RegularSpaced");
    connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
    name_layout->addWidget(name_field);
    
    name_panel->setLayout(name_layout);
    mainLayout->addWidget(name_panel);
    
    // Buttons Layout
    
    QHBoxLayout * buttons_layout=new QHBoxLayout;
    
    saveBtn = new QPushButton( "Create Object ");
    saveBtn->setMinimumSize(saveBtn->sizeHint());
    connect( saveBtn, SIGNAL(clicked()), this, SLOT(createObj()) );
    saveBtn->setFocusPolicy(Qt::StrongFocus);
    buttons_layout->addWidget(saveBtn);
    saveBtn->setDefault(true);
    
    dismissBtn = new QPushButton( "Dismiss");
    dismissBtn->setMinimumSize(dismissBtn->sizeHint());
    connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
    buttons_layout->addWidget(dismissBtn);
    
    mainLayout->addLayout(buttons_layout);	
    
    mainWidget->setLayout(mainLayout);
    setCentralWidget(mainWidget);
    
    this->setMinimumSize(this->sizeHint());
    
}



// -------------

void CreateRegularWindow::setUserInput(){
    if(name_field->lineBox->hasFocus()){
        
        if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
        else userSet = true; // user typed input, disable update from data slot
    }
    // else ignore
}

// -------------

void CreateRegularWindow::setType(const QString & text){
    
    if( text.contains(DV_TIME) ) {
        startField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
        spacingField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
        unitLabl->setText(" (s)");
    }
    else if( text.contains(DV_DOUBLE) )	{
        
        startField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
        spacingField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_NOT_SEQ, QS_ANY));
        unitLabl->setText(" ");        
    }
    else{
        startField->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
        spacingField->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
        unitLabl->setText(" ");        
    }
}


// -------------

void CreateRegularWindow::keyPressEvent(QKeyEvent * e )
{  
    
    if(QuSlotDebug())
    {
        printf("CreateRegularWindow::keyPressEvent \n");
        fflush(stdout);
    }
    
    // use escape key to toggle default buttons
    if(e->key() == Qt::Key_Escape){
        if( saveBtn->isDefault()  ) {
            saveBtn->setDefault(false);
            dismissBtn->setDefault(true);
        }
        else if(dismissBtn->isDefault()  )  {
            dismissBtn->setDefault(false);
            saveBtn->setDefault(true);
        }
        return;
    }
    // use return or enter key to perform default
    if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
        if(saveBtn->isDefault()) createObj();
        else if (dismissBtn->isDefault()) close();
        return;
    }
    
    // else pass on to list view 
    e->ignore(); // let selection change
    
}



// -------------

void CreateRegularWindow::createObj(){
    
    if(QuSlotDebug())
    {
        printf("CreateRegularWindow::CreateObj \n");
        fflush(stdout);
    }
    qApp->processEvents();

    
    DvObject_var numObj = numberField->getObject();
    int n = numObj->asInt();

    DvObject_var stObj = startField->getObject();        
    DvObject_var spObj = spacingField->getObject();        

    DvObject_var retObj;
    if( objType->currentText().contains(DV_TIME) ) {
        
        retObj = new DvObject(stObj->asTime(), (size_t) n);
        retObj->assign_regular(spObj->asDouble());
        
    }
    else if( objType->currentText().contains(DV_DOUBLE) )	{
       
        retObj = new DvObject(stObj->asDouble(), (size_t) n);
        retObj->assign_regular(spObj->asDouble());       

    }
    else{
        
        retObj = new DvObject(stObj->asInt(), (size_t) n);
        retObj->assign_regular(spObj->asInt());
        
    }
    
    QString newName = name_field->text();
    QwuiPutOnWL(retObj, newName);
    
    qApp->processEvents();
    
    this->close();
    
}




// -------------Regression Window class ---------------------------------

RegressionWindow::RegressionWindow() 
	: QMainWindow() 
{

	if(QuSlotDebug())
	{
		printf("RegressionWindow::RegressionWindow \n");
		fflush(stdout);
	}
   
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Linear Regression");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_SEQ, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save best fit line as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	name_field->setText("LinearFit");
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Save ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOp()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);
	saveBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

void RegressionWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}



void RegressionWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate();
   
}

void RegressionWindow::setNewName(){
     
 if(QuSlotDebug())
 {
	printf("ArrayTraceWindow::setNewName \n");
	fflush(stdout);
 }

   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
	 QString newName = QString(objName);
       
     int ptr = newName.lastIndexOf("/");
     if (ptr >= 0) newName.remove(0, ptr+1);
	 ptr = newName.indexOf(" [");
	 if (ptr > 0) newName.truncate(ptr);
     newName += "_LinFit";
     this->name_field->setText(newName);
   }
   

}

void RegressionWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("RegressionWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOp();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void RegressionWindow::acceptOp(){
      
	if(QuSlotDebug())
	{
		printf("RegressionWindow::acceptOp \n");
		fflush(stdout);
	}
  
	DvObject_var dobj = objField->getObject();
	
	DvObject_var fitLine = dobj->regression();

	if(fitLine.is_nil()) {
		statusBar()->showMessage("NULL object from Linear Fit");
		return;
	}
	
	
	QString newName = name_field->text();
	QwuiPutOnWL(fitLine, newName);

	this->close();
	return;
}




// ----------------------------------------------
//-----------------------------------------------


T1T2ToSecWindow::T1T2ToSecWindow(QWidget *parent) 
	: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	// converts DEPEND_0 timetags to scalar offset (default from ttag start time)
	// and attaches as xref (default is DEPEND_0)
	
	if(QuSlotDebug())
	{
		printf("T1T2ToSecWindow::T1T2ToSecWindow \n");
		fflush(stdout);
	}
  
  	this->setWindowTitle("Convert Time Difference to seconds Tend - Tstart");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * startPanel=new QGroupBox(mainWidget);
	QHBoxLayout * startLayout=new QHBoxLayout;
	startPanel->setTitle("Start Time Object");
	
	startField = new QguWLinput(startPanel,"startField");
	startField->setToolTip("Accepts Time (can extract from interval or time tags)");
	startField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
	startField->setEnabled(true);
	startField->setMinimumWidth(char_width*55);
	startLayout->addWidget(startField);
	
	startPanel->setLayout(startLayout);
	mainLayout->addWidget(startPanel);

	
	QGroupBox * endPanel=new QGroupBox(mainWidget);
	QHBoxLayout * endLayout=new QHBoxLayout;
	endPanel->setTitle("End Time Object");
	
	endField = new QguWLinput(endPanel,"endField");
	endField->setToolTip("Accepts Time (can extract from interval or time tags)");
	endField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
	endField->setEnabled(true);
	endField->setMinimumWidth(char_width*55);
	endLayout->addWidget(endField);
	
	endPanel->setLayout(endLayout);
	mainLayout->addWidget(endPanel);
	
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	name_field->setText("difference");
	userSet = false;
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Calculate");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);
	saveBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}



void T1T2ToSecWindow::setObjectStart(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("T1T2ToSecWindow::setObject \n");
	fflush(stdout);
 }
  
   startField->setText(WLctxt);
   startField->reValidate(true);
   
}

void T1T2ToSecWindow::setObjectEnd(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("T1T2ToSecWindow::setObject \n");
	fflush(stdout);
 }
  
   endField->setText(WLctxt);
   endField->reValidate(true);
   
}


void T1T2ToSecWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("T1T2ToSecWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOperation();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void T1T2ToSecWindow::acceptOperation(){
      
	if(QuSlotDebug())
	{
		printf("T1T2ToSecWindow::acceptOperation \n");
		fflush(stdout);
	}
	
	QString name = name_field->text();
	if (name.isEmpty() ) name = "diffSec";
	    
	DvObject_var t1 = startField->getObject();
  	DvObject_var t2 = endField->getObject();
		
	if(t1.is_ok() && t2.is_ok())
	{
		DvObject_var val = new DvObject(0.0);
		val->dbl(0) = t2->time(0) - t1->time(0);
			
		DvString SI_Str("1>s");
		val->change_xref(SI_CONVERSION, SI_Str);
			
		DvString unit_Str("sec");
		val->change_xref(UNITS, unit_Str);

		DvString name_Str(name.toLatin1().data());
		val->change_xref(FIELDNAM, name_Str);

		QwuiPutOnWL(val, name);
		return;
	}
	
	this->close();
	return;
}

void T1T2ToSecWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("T1T2ToSecWindow::dismiss \n");
		fflush(stdout);
	}

	this->close();
	return;
}

//-----------------------------------------------


TT1TT2ToSecWindow::TT1TT2ToSecWindow(QWidget *parent) 
	: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	// Finds delay (difference) between timetags 
	
	if(QuSlotDebug())
	{
		printf("TT1TT2ToSecWindow::TT1TT2ToSecWindow \n");
		fflush(stdout);
	}
  
  	this->setWindowTitle("Find Difference in Time Tags to sequence, tt1[i] - tt2[i] (seconds)");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * startPanel=new QGroupBox(mainWidget);
	QHBoxLayout * startLayout=new QHBoxLayout;
	startPanel->setTitle("Start Time Object");
	
	startField = new QguWLinput(startPanel,"startField");
	startField->setToolTip("Accepts Time Sequence (can extract from time tags)");
	startField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_ANY, QS_ANY));
	startField->setEnabled(true);
	startField->setMinimumWidth(char_width*55);
	startLayout->addWidget(startField);
	
	startPanel->setLayout(startLayout);
	mainLayout->addWidget(startPanel);

	
	QGroupBox * endPanel=new QGroupBox(mainWidget);
	QHBoxLayout * endLayout=new QHBoxLayout;
	endPanel->setTitle("End Time Object");
	
	endField = new QguWLinput(endPanel,"endField");
	endField->setToolTip("Accepts Time sequence(can extract from time tags)");
	endField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_ANY, QS_ANY));
	endField->setEnabled(true);
	endField->setMinimumWidth(char_width*55);
	endLayout->addWidget(endField);
	
	endPanel->setLayout(endLayout);
	mainLayout->addWidget(endPanel);
	
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	name_field->setText("difference");
	userSet = false;
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Calculate");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);
	saveBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}



void TT1TT2ToSecWindow::setObjectStart(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("TT1TT2ToSecWindow::setObject \n");
	fflush(stdout);
 }
  
   startField->setText(WLctxt);
   startField->reValidate(true);
   
}

void TT1TT2ToSecWindow::setObjectEnd(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("TT1TT2ToSecWindow::setObject \n");
	fflush(stdout);
 }
  
   endField->setText(WLctxt);
   endField->reValidate(true);
   
}


void TT1TT2ToSecWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("TT1TT2ToSecWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOperation();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void TT1TT2ToSecWindow::acceptOperation(){
      
	if(QuSlotDebug())
	{
		printf("TT1TT2ToSecWindow::acceptOperation \n");
		fflush(stdout);
	}
	
	QString name = name_field->text();
	if (name.isEmpty() ) name = "diffSecSeq";
	    
	DvObject_var t1 = startField->getObject();
  	DvObject_var t2 = endField->getObject();
	
	if(t1.is_nil() || t2.is_nil()){
		statusBar()->showMessage("Empty object");
		return;
	}

	if(t1->seqSize() != t2->seqSize()){
		statusBar()->showMessage("Sequences have different lengths");
		return;
	}
	
	if(!t1->is_time() || !t2->is_time()){
		statusBar()->showMessage("Not time object");
		return;
	}

	DvObject_var val = new DvObject(0.0, t2->seqSize());
	for(int i=0; i<t2->seqSize(); i++){
		val->dbl(i) = t2->time(i) - t1->time(i);
	}
			
	DvString SI_Str("1>s");
	val->change_xref(SI_CONVERSION, SI_Str);
			
	DvString unit_Str("sec");
	val->change_xref(UNITS, unit_Str);

	DvString name_Str(name.toLatin1().data());
	val->change_xref(FIELDNAM, name_Str);

	QwuiPutOnWL(val, name);

		
	this->close();
	return;
}

void TT1TT2ToSecWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("TT1TT2ToSecWindow::dismiss \n");
		fflush(stdout);
	}

	this->close();
	return;
}


//-----------------------------------------------


TIToSecWindow::TIToSecWindow(QWidget *parent) 
	: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	// converts DEPEND_0 timetags to scalar offset (default from ttag start time)
	// and attaches as xref (default is DEPEND_0)
	
	if(QuSlotDebug())
	{
		printf("TIToSecWindow::TIToSecWindow \n");
		fflush(stdout);
	}
  
  	this->setWindowTitle("Convert Time Difference to seconds Tend - Tstart");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Data Object");
	
	objField = new QguWLinput(slotPanel,"startField");
	objField->setToolTip("Accepts Time (can extract from interval or time tags)");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_TI, QS_0D, QS_NOT_SEQ, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	name_field->setText("duration");
	userSet = false;
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Calculate");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);
	saveBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}



void TIToSecWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("TIToSecWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate(true);
   
}


void TIToSecWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("TIToSecWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOperation();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void TIToSecWindow::acceptOperation(){
      
	if(QuSlotDebug())
	{
		printf("TIToSecWindow::acceptOperation \n");
		fflush(stdout);
	}
	
	QString name = name_field->text();
	if (name.isEmpty() ) name = "duration";
	    
	DvObject_var dobj = objField->getObject();
	
	if(dobj.is_ok() ){
		
		if(dobj->is_event() )
		{
			DvObject_var val = new DvObject(dobj->event(0).duration());
			
			DvString SI_Str("1>s");
			val->change_xref(SI_CONVERSION, SI_Str);
			
			DvString unit_Str("sec");
			val->change_xref(UNITS, unit_Str);

			DvString name_Str(name.toLatin1().data());
			val->change_xref(FIELDNAM, name_Str);

			QwuiPutOnWL(val, name);
			return;
		}
	}
		
	this->close();
	return;
}

void TIToSecWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("TIToSecWindow::dismiss \n");
		fflush(stdout);
	}

	this->close();
	return;
}


//-----------------------------------------------


TTagToScalarWindow::TTagToScalarWindow(QWidget *parent) 
	: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
	// converts DEPEND_0 timetags to scalar offset (default from ttag start time)
	// and attaches as xref (default is DEPEND_0)
	
	if(QuSlotDebug())
	{
		printf("TTagToScalarWindow::TTagToScalarWindow \n");
		fflush(stdout);
	}
  
  	this->setWindowTitle("Convert Time Tags to seconds from Start Time");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
   
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Data Object");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition());
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewOffset()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// Offset slot
	
	QGroupBox * offsetPanel=new QGroupBox(mainWidget);
	QHBoxLayout * offsetLayout=new QHBoxLayout;
	offsetPanel->setTitle("Start Time");
	
	offsetField = new QguWLinput(offsetPanel,"objField");
	offsetField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
	offsetField->setEnabled(true);
	offsetField->setMinimumWidth(char_width*55);
	offsetLayout->addWidget(offsetField);
	connect( offsetField,  SIGNAL( edited_OK() ), this, SLOT( setUserOffset() ) );
	
	offsetPanel->setLayout(offsetLayout);
	mainLayout->addWidget(offsetPanel);
	
	// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save metadata as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	name_field->setText(DEPEND_0);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserName() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Set as new DEPEND_0 ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);
	saveBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	//Done
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

void TTagToScalarWindow::setUserName(){

  if(name_field->lineBox->text() == QString(DEPEND_0)) saveBtn->setText("Set as new DEPEND_0 ");
  else saveBtn->setText("Save new metadata ");
  
}


void TTagToScalarWindow::setUserOffset(){
     
 if(QuSlotDebug())
 {
	printf("TTagToScalarWindow::setUserOffset \n");
	fflush(stdout);
 }

  if(offsetField->lineBox->hasFocus()){
	
     if(offsetField->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }

}

void TTagToScalarWindow::setNewOffset(){
     
	if(QuSlotDebug())
	{
		printf("TTagToScalarWindow::setNewOffset \n");
		fflush(stdout);
	}

	if (userSet) return;
   
	DvObject_var dobj = objField->getObject();
   
	if(!dobj.is_nil()){

		DvObject_var tt = dobj->getTimeTags();
		if(tt.is_ok() && tt->seqSize() > 0){
			offsetField->setText(tt->time(0).getISOstring(0).c_str());
		}
	}

}


void TTagToScalarWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("TTagToScalarWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate(true);
   
}


void TTagToScalarWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("TTagToScalarWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) acceptOperation();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void TTagToScalarWindow::acceptOperation(){
      
	if(QuSlotDebug())
	{
		printf("TTagToScalarWindow::acceptOperation \n");
		fflush(stdout);
	}
	DvObject_var dobj = objField->getObject();
  	DvObject_var offset = offsetField->getObject();
	
	if(dobj.is_ok() && offset.is_ok()){

		DvObject_var tt = dobj->getTimeTags();
		
		if(tt.is_ok()){
			
			DvObject_var ss = tt - offset;
			
			DvString unitStr("sec");
			ss->change_xref(UNITS, unitStr);
			DvString SI_Str("1>s");
			ss->change_xref(SI_CONVERSION, SI_Str);
			DvString NameStr("Sec after ");
			NameStr += offset->asStr(0);
			ss->change_xref(FIELDNAM, NameStr);
						
			DvString setXrefName = name_field->lineBox->text().toLatin1().data();
			if(setXrefName != DEPEND_0) ss->change_xref(DEPEND_0, tt);

			ss->change_xref(PTR_VAR_NAME, setXrefName);

			dobj->change_xref(setXrefName, ss);

		}
	}
	
	this->close();
	return;
}

void TTagToScalarWindow::dismiss(){
      
	if(QuSlotDebug())
	{
		printf("TTagToScalarWindow::dismiss \n");
		fflush(stdout);
	}

	this->close();
	return;
}

//-----------------------------------------------


ScalarToTTagWindow::ScalarToTTagWindow(QWidget *parent) 
: QMainWindow(/*parent*/) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{
    // converts scalar offset and start time to tt
    
    if(QuSlotDebug())
    {
        printf("ScalarToTTagWindow::ScalarToTTagWindow \n");
        fflush(stdout);
    }
    
    this->setWindowTitle("Convert scalar sequence and start time to timetags");
    
    this->setPalette(qApp->palette());
    
    QFontMetrics qgu_font_mt(qApp->font());
    int char_width = qgu_font_mt.width("n");
    
    setAttribute(Qt::WA_DeleteOnClose);
    
    msgBar = statusBar();
    
    QGroupBox * mainWidget=new QGroupBox(this);
    QVBoxLayout * mainLayout=new QVBoxLayout;
    
    // Data slot
    
    QGroupBox * slotPanel=new QGroupBox(mainWidget);
    QHBoxLayout * slotLayout=new QHBoxLayout;
    slotPanel->setTitle("Scalar Sequence");
    
    objField = new QguWLinput(slotPanel, "objField");
    objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_0D, QS_ANY, QS_ANY));
    objField->setEnabled(true);
    objField->setMinimumWidth(char_width*55);
    slotLayout->addWidget(objField);
    
    slotPanel->setLayout(slotLayout);
    mainLayout->addWidget(slotPanel);
    
    // Offset slot
    
    QGroupBox * offsetPanel=new QGroupBox(mainWidget);
    QHBoxLayout * offsetLayout=new QHBoxLayout;
    offsetPanel->setTitle("Start Time");
    
    offsetField = new QguWLinput(offsetPanel,"objField");
    offsetField->setAcceptable(Qcondition(QS_NARROW, QS_TIME, QS_0D, QS_NOT_SEQ, QS_ANY));
    offsetField->setEnabled(true);
    offsetField->setMinimumWidth(char_width*55);
    offsetLayout->addWidget(offsetField);
    
    offsetPanel->setLayout(offsetLayout);
    mainLayout->addWidget(offsetPanel);
    
    // "Save as" panel
    
    QGroupBox * name_panel=new QGroupBox(mainWidget);
    QHBoxLayout * name_layout=new QHBoxLayout;
    
    QLabel *name_label = new QLabel("Save time tags as ", name_panel);
    name_layout->addWidget(name_label);
    
    name_field = new QguWLinput(name_panel, "name_field");
    name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
    name_field->setEnabled(true);
    name_field->setMinimumWidth(char_width*50);
    name_field->setText("timetags");
    name_layout->addWidget(name_field);
    
    name_panel->setLayout(name_layout);
    mainLayout->addWidget(name_panel);
    
    // Buttons Layout
    
    QHBoxLayout * buttons_layout=new QHBoxLayout;
    
    saveBtn = new QPushButton( "Create ");
    saveBtn->setMinimumSize(saveBtn->sizeHint());
    connect( saveBtn, SIGNAL(clicked()), this, SLOT(acceptOperation()) );
    saveBtn->setFocusPolicy(Qt::StrongFocus);
    buttons_layout->addWidget(saveBtn);
    saveBtn->setDefault(true);
    
    dismissBtn = new QPushButton( "Dismiss");
    dismissBtn->setMinimumSize(dismissBtn->sizeHint());
    connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
    buttons_layout->addWidget(dismissBtn);
    
    mainLayout->addLayout(buttons_layout);
    
    //Done
    
    
    mainWidget->setLayout(mainLayout);
    setCentralWidget(mainWidget);
    
    this->setMinimumSize(this->sizeHint());
    
}


void ScalarToTTagWindow::setObject(const char * WLctxt){
    
    if(QuSlotDebug())
    {
        printf("ScalarToTTagWindow::setObject \n");
        fflush(stdout);
    }
    
    objField->setText(WLctxt);
    objField->reValidate(true);
    
}


void ScalarToTTagWindow::keyPressEvent(QKeyEvent * e )
{  
    
    if(QuSlotDebug())
    {
        printf("ScalarToTTagWindow::keyPressEvent \n");
        fflush(stdout);
    }
    
    // use escape key to toggle default buttons
    if(e->key() == Qt::Key_Escape){
        if( saveBtn->isDefault()  ) {
            saveBtn->setDefault(false);
            dismissBtn->setDefault(true);
        }
        else if(dismissBtn->isDefault()  )  {
            dismissBtn->setDefault(false);
            saveBtn->setDefault(true);
        }
        return;
    }
    // use return or enter key to perform default
    if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
        if(saveBtn->isDefault()) acceptOperation();
        else if (dismissBtn->isDefault()) close();
        return;
    }
    
    // else pass on to list view 
    e->ignore(); // let selection change
    
}

void ScalarToTTagWindow::acceptOperation(){
    
    if(QuSlotDebug())
    {
        printf("ScalarToTTagWindow::acceptOperation \n");
        fflush(stdout);
    }
    DvObject_var dobj = objField->getObject();
    DvObject_var offset = offsetField->getObject();
    
    if(dobj.is_ok() && offset.is_ok()){
        
        DvObject_var tt = offset + dobj;
        QString newName = name_field->text();
        QwuiPutOnWL(tt, newName);
    }
    
    this->close();
    return;
}

void ScalarToTTagWindow::dismiss(){
    
    if(QuSlotDebug())
    {
        printf("ScalarToTTagWindow::dismiss \n");
        fflush(stdout);
    }
    
    this->close();
    return;
}



// -------------Integral Window class ---------------------------------

IntegralWindow::IntegralWindow() 
	: QMainWindow() 
{

	if(QuSlotDebug())
	{
		printf("IntegralWindow::IntegralWindow \n");
		fflush(stdout);
	}
	this->setWindowTitle("Integral Estimator");
	
  	QMenu * help = new QMenu( this );
	this->menuBar()->addMenu(help );
	help->addAction("Integration in QSAS", this, SLOT( launchHelp() ));
	help->setTitle(" Help ");
	
	this->setPalette(qApp->palette());

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
     
	QGroupBox * mainWidget=new QGroupBox(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Data slot
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	slotPanel->setTitle("Integrate...");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ANY, QS_ANY, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(setNewName()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// Method panel
	
	QGroupBox * method_panel=new QGroupBox(mainWidget);
	QVBoxLayout * method_layout=new QVBoxLayout;
	method_panel->setTitle("Integration form");
	
	integralMethod = new QComboBox(method_panel);
	integralMethod->addItem("Definite Integral");
	integralMethod->addItem("Indefinite Intergal" );
	method_layout->addWidget( integralMethod, 0,Qt::AlignLeft );
	
	
	methodLabl = new QLabel("Total Sum (f(x)*h) over range");
	method_layout->addWidget( methodLabl, 1, Qt::AlignLeft );

	connect( integralMethod,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setMethod(const QString &) ) );
	
	method_panel->setLayout(method_layout);
	mainLayout->addWidget(method_panel);

	// Gap panel
	
	QGroupBox * gap_panel=new QGroupBox(mainWidget);
	QVBoxLayout *gap_layout=new QVBoxLayout;
	gap_panel->setTitle("Interpolation");
	
	gapMethod = new QComboBox(gap_panel);	
	gapMethod->addItem("Nearest Neighbour (histogram)" );
	gapMethod->addItem("Linear interp (trapezoidal)" );
	gapMethod->addItem("Ignore gaps" );
	connect( gapMethod,  SIGNAL( currentIndexChanged(const QString &) ), this, SLOT( setGapMethod(const QString &) ) );
	

	gap_layout->addWidget( gapMethod, 2, Qt::AlignLeft );
	gapMethodLabl = new QLabel("Divide gap into two bins using Nearest Neighbour value");
	gap_layout->addWidget( gapMethodLabl, 3, Qt::AlignLeft );

	
	gap_panel->setLayout(gap_layout);
	mainLayout->addWidget(gap_panel);


// "Save as" panel
	
	QGroupBox * name_panel=new QGroupBox(mainWidget);
	QHBoxLayout * name_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Save as ", name_panel);
	name_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setEnabled(true);
	name_field->setMinimumWidth(char_width*50);
	userSet = false;
	connect( name_field,  SIGNAL( edited_OK() ), this, SLOT( setUserInput() ) );
	name_layout->addWidget(name_field);
	
	name_panel->setLayout(name_layout);
	mainLayout->addWidget(name_panel);
	
	// Buttons Layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	saveBtn = new QPushButton( "Do Integral ");
	saveBtn->setMinimumSize(saveBtn->sizeHint());
	connect( saveBtn, SIGNAL(clicked()), this, SLOT(doIntegral()) );
	saveBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveBtn);
	saveBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
	
}

IntegralWindow::~IntegralWindow() 
{
}
// -------------

void IntegralWindow::launchHelp() 
{
	 if(QuSlotDebug())
 	{
		printf("IntegralWindow::launchHelp \n");
		fflush(stdout);
	 }
 
  	qsasWindow->help_window->manEntry("QSAS_analysis.html","#integration");

}
// -------------

void IntegralWindow::setUserInput(){
  if(name_field->lineBox->hasFocus()){
	
     if(name_field->lineBox->text().isEmpty() ) userSet = false; // allow refresh from drop to object slot
	 else userSet = true; // user typed input, disable update from data slot
  }
  // else ignore
}

// -------------

void IntegralWindow::setMethod(const QString & text){
    
	if( text.contains("Definite") ) {
		methodLabl->setText("Total Sum (f(t)*h), over range");
	}
	else  {
		methodLabl->setText("Cumulative Sum (f(t)*h), as function of Depend_0 (time or scalar) ");
	}
	
	
}

// -------------

void IntegralWindow::setGapMethod(const QString & text){
    
	if( text.contains("Nearest") ) {
		gapMethodLabl->setText("Histogram using Nearest Neighbour value");
	}
	else if( text.contains("Linear") ) {
		gapMethodLabl->setText("Trapeziodal fit between data points ");
	}
	else {
		gapMethodLabl->setText("Ignore gaps in sums (use delta_plus/minus) - integral incomplete ");
	}
	
}


// -------------

void IntegralWindow::setNewName(){
     
	if(QuSlotDebug())
	{
		printf("IntegralWindow::setNewName \n");
		fflush(stdout);
	}

	
	// set output name default
   if (userSet) return;
   QString objName = QString(this->objField->textObject().toLatin1().data());
   
   if(!objName.isEmpty()){
		QString newName = QString(objName);
       
		int ptr = newName.lastIndexOf("/");
		if (ptr >= 0) newName.remove(0, ptr+1);
		ptr = newName.indexOf(" [");
		if (ptr > 0) newName.truncate(ptr);

		this->name_field->setText("I_"+newName);
   }

}

// -------------

void IntegralWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("IntegralWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( saveBtn->isDefault()  ) {
		   saveBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   saveBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(saveBtn->isDefault()) doIntegral();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

// -------------

void IntegralWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("IntegralWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate(true);
   
}

// -------------

void IntegralWindow::doIntegral(){
	  
	if(QuSlotDebug())
	{
		printf("IntegralWindow::doIntegral \n");
		fflush(stdout);
	}
	qApp->processEvents();
		
	DvObject_var dobj = objField->getObject();
	
	DvString gapMthd = DV_NN;
	if(gapMethod->currentIndex() == 1) gapMthd = DV_LINEAR;
	if(gapMethod->currentIndex() == 2) gapMthd = DV_REMOVE;

	DvString intMthd = DV_DEF_INTEGRAL;
	if(integralMethod->currentIndex() == 1) intMthd = DV_INDEF_INTEGRAL;
	
	
	DvObject_var retObj = dobj->integrate(gapMthd, intMthd);
	

	if( retObj.is_nil() ) {
		statusBar()->showMessage("NULL object from integral, putting on WL for error checking");
		QString newName = name_field->text();
		QwuiPutOnWL(retObj, newName);
		return;
	}


	QString newName = name_field->text();
	QwuiPutOnWL(retObj, newName);
      
	qApp->processEvents();
	
	this->close();

}

// Class MakeVectorWindow
//------------------------

MakeVectorWindow::MakeVectorWindow(QWidget *parent) 
	: QMainWindow(/*parent*/NULL) // NULL because if it's called from the new calculator, it will be scaled/rotated as its parent widget, and window decorations could be missing
{

	if(QuSlotDebug())
	{
		printf("MakeVectorWindow::MakeVectorWindow \n");
		fflush(stdout);
	}
  
	this->setPalette(qApp->palette());
	this->setWindowTitle("Add vector metadata to 3 element Data Array ");

	QFontMetrics qgu_font_mt(qApp->font());
	int char_width = qgu_font_mt.width("n");
   
	setAttribute(Qt::WA_DeleteOnClose);
		
	// Status bar
	
	msgBar = statusBar();
	
	// Main widget
	
	QWidget * mainWidget=new QWidget(this);
	QVBoxLayout * mainLayout=new QVBoxLayout;
	
	// Slot widget
	
	QGroupBox * slotPanel=new QGroupBox(mainWidget);
	QHBoxLayout * slotLayout=new QHBoxLayout;
	
	slotPanel->setTitle(" Data Array Object ");
	
	objField = new QguWLinput(slotPanel,"objField");
	objField->setAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_1D, QS_ANY, QS_ANY));
	objField->setEnabled(true);
	objField->setMinimumWidth(char_width*55);
	connect(objField, SIGNAL(edited_OK()), this, SLOT(testArray()));
	slotLayout->addWidget(objField);
	
	slotPanel->setLayout(slotLayout);
	mainLayout->addWidget(slotPanel);
	
	// Frame panel
	
	QGroupBox * frame_panel=new QGroupBox(mainWidget);
	QHBoxLayout * frame_layout=new QHBoxLayout;
	
	QLabel *frame_label = new QLabel("Frame    ", frame_panel);
	frame_layout->addWidget(frame_label);
	
	frame_field = new QguWLinput(frame_panel, "frame_field");
	frame_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	frame_field->setEnabled(true);
	frame_field->setMinimumWidth(char_width*25);
	frame_field->setText("gse");
	frame_layout->addWidget(frame_field);
	
	frame_panel->setLayout(frame_layout);
	mainLayout->addWidget(frame_panel);
	
	// Representation panel
	
	QGroupBox * rep_panel=new QGroupBox(mainWidget);
	QHBoxLayout * rep_layout=new QHBoxLayout;
	
	QLabel *rep_label = new QLabel("Representation ", rep_panel);
	rep_layout->addWidget(rep_label);
	
	rep_field = new QComboBox(rep_panel);
	rep_field->setEnabled(true);
	rep_field->addItem("xyz");
	rep_field->addItem("rlp (rad)");
	rep_field->addItem("rlp (deg)");
	rep_field->addItem("rtp (rad)");
	rep_field->addItem("rtp (deg)");
	rep_layout->addWidget(rep_field);
	
	rep_panel->setLayout(rep_layout);
	mainLayout->addWidget(rep_panel);
	
	// Buttons layout
	
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
	doBtn = new QPushButton( "Convert into Vector");
	doBtn->setMinimumSize(doBtn->sizeHint());
	connect( doBtn, SIGNAL(clicked()), this, SLOT(makeVector()) );
	doBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(doBtn);
	doBtn->setDefault(true);

	dismissBtn = new QPushButton( "Dismiss");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	buttons_layout->addWidget(dismissBtn);
	
	mainLayout->addLayout(buttons_layout);
	
	
	
	mainWidget->setLayout(mainLayout);
	setCentralWidget(mainWidget);
	
	this->setMinimumSize(this->sizeHint());
}


void MakeVectorWindow::setObject(const char * WLctxt){
    
 if(QuSlotDebug())
 {
	printf("MakeVectorWindow::setObject \n");
	fflush(stdout);
 }
  
   objField->setText(WLctxt);
   objField->reValidate();

}


void MakeVectorWindow::testArray(){
    
 if(QuSlotDebug())
 {
	printf("MakeVectorWindow::setObject \n");
	fflush(stdout);
 }
  
   // if input not 3 comp array disable doBtn based on component subset
   DvObject_var dobj = objField->getObject();
   if(dobj->arraySize() != 3)
   {
	   doBtn->setEnabled(false);
	   return; // not a matrix object, cannot convert
   }
   doBtn->setEnabled(true);
}


void MakeVectorWindow::keyPressEvent(QKeyEvent * e )
{  
     
 if(QuSlotDebug())
 {
	printf("MakeVectorWindow::keyPressEvent \n");
	fflush(stdout);
 }
  
	  // use escape key to toggle default buttons
  if(e->key() == Qt::Key_Escape){
	  if( doBtn->isDefault()  ) {
		doBtn->setDefault(false);
		   dismissBtn->setDefault(true);
	  }
	  else if(dismissBtn->isDefault()  )  {
		   dismissBtn->setDefault(false);
		   doBtn->setDefault(true);
	  }
	  return;
  }
  // use return or enter key to perform default
  if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
     if(doBtn->isDefault()) makeVector();
	 else if (dismissBtn->isDefault()) close();
     return;
  }

  // else pass on to list view 
  e->ignore(); // let selection change

}

void MakeVectorWindow::makeVector(){
      
  if(QuSlotDebug())
  {
	printf("MakeVectorWindow::makeVector \n");
	fflush(stdout);
  }

  DvObject_var dobj = objField->getObject();
  DvString name = objField->text().toLatin1().data();
  
  // attach xrefs here
  DvString repTxt = rep_field->currentText().toLatin1().data();
  DvString frameTxt = frame_field->text().toLatin1().data();
  DvString text = "vector>";
  text += frameTxt;

  if( repTxt.contains("rlp") ){
	   DvString SI_conv = dobj->getXrefText(SI_CONVERSION);
       if( repTxt.contains("deg") ){
	     SI_conv = SI_conv.before(",");
		 SI_conv += ", deg, deg";
       }
	   else{
		 SI_conv = SI_conv.before(",");
		 SI_conv += ", rad, rad";	   
	   }
	   dobj->change_xref(SI_CONVERSION, SI_conv);
       text +=  "_rlp";
	   dobj->setFrameAttr(text);
	   dobj = dobj->getVecFromRLP();
	   QwuiPutOnWL(dobj, QString(name.c_str()), true); // replace object
  }
  else if(repTxt.contains("rtp") > 0){
	    DvString SI_conv = dobj->getXrefText(SI_CONVERSION);
       if( repTxt.contains("deg") ){
	     SI_conv = SI_conv.before(",");
		 SI_conv += ", deg, deg";
       }
	   else{
		 SI_conv = SI_conv.before(",");
		 SI_conv += ", rad, rad";	   
	   }
	   dobj->change_xref(SI_CONVERSION, SI_conv);
       text +=  "_rtp";
	   dobj->setFrameAttr(text);
	   dobj = dobj->getVecFromRTP();
	   QwuiPutOnWL(dobj, QString(name.c_str()), true); // replace object

  }
  else {
  	text += "_xyz";
	dobj->setFrameAttr(text);
  }

  this->close();
  return;
}

// -----------------------------
