/***********************************************************************
*           See Accompanying licence file QSASlicence                 *
***********************************************************************/
#include <stdlib.h>

#include <qstring.h>

#include "qtpl.h"
#include "qtpl_parser.h"
#include "qtpl_io.h"
#include "qdutil.h"
#include "qplug_if.h"

using namespace QSAS; 

// class QtplParserFile

QtplParserFile::QtplParserFile(QString fileNameAndPath){
   
   openedOK = false;
   int p;
   
   contentBuf = QtplReadFile(fileNameAndPath);
   
   if( !contentBuf.isEmpty() ){     
     // go through and replace \n by newline
     p=0;
     while ( p<contentBuf.length()){
        if(contentBuf[p] == '\\' && contentBuf[p+1] == 'n'){
	   contentBuf = contentBuf.replace(p,2,"\n");
	}
	p++;
     }
	//   cout << contentBuf.toLatin1().data() << endl;
     QString nextWord = getWord(contentBuf);
     if (nextWord == "EXTERNAL") nextWord = getWord(contentBuf);
     function_name = QString(nextWord.toLatin1().data());
     
     QString headBlock = getToken(contentBuf, '(', ')');
     
     while ( headBlock.length() != 0){
       QString ioType = getWord(headBlock);
       if(headBlock.length() == 0) break;
       
       // remove spaces and comma
       stripSpace(headBlock);
       QString dataType = getWord(headBlock);
       if(headBlock.length() == 0) break;

       // remove spaces and comma
       stripSpace(headBlock);
       QString vName = getWord(headBlock);
       
       QtplParserVar *newVar = new QtplParserVar(vName, ioType, dataType);
       qtpl_vars.push_back(newVar);
       
       // remove spaces and comma
       stripSpace(headBlock);
     }
   
     // the remaining text should all be encloed in {}
     // get this block and put back in content buffer
     // tolerate if {} are both missing

     QString mainBlock = getToken(contentBuf, '{', '}');
     
     if ( !(mainBlock.isEmpty() )) contentBuf = mainBlock;
     
     QString function = getWord(contentBuf);
     
     if( function != function_name){
       // issue warning to help debug
       QplugAppendTextDisplay("Function name differs from plugin name \n -> ");
       QplugAppendTextDisplay(function.toLatin1().data());
       QplugAppendTextDisplay("\n-> ");
       QplugAppendTextDisplay(function_name.toLatin1().data());
       QplugAppendTextDisplay("\n using former\n");
       function_name = function;
     }
     
     QString arguments = getToken(contentBuf, '{', '}');
     while (arguments.length() != 0){
         
       QString parameter = getWord(arguments);
       QString value = getToken(arguments, '{', '}' );

       if(parameter.isNull() || value.isNull()){
         QplugAppendTextDisplay("Unknown function parameter... ");
         QplugAppendTextDisplay(parameter.toLatin1().data());
         QplugAppendTextDisplay("\n");
	 return;
       }
       
       // remove all white space and quotes from front and end
       stripSpace(value);
       if (value[0] == '"') value.remove(0,1);
       if (value[value.length()-1] == '"') value.truncate(value.length() -1);
       stripSpace(value);
        
       if( parameter == "ENTRY_NAME"){
         entry_name = value;
       }
       else if( parameter == "OBJECT_FILE"){
         object_file = value;
       }
       else if( parameter == "DESCRIPTION"){
         description = value;
       }
       else if( parameter == "PUBLIC_NAME"){
         public_name = value;
       }
       else{
         QplugAppendTextDisplay("Unknown function parameter... ");
         QplugAppendTextDisplay(parameter.toLatin1().data());
         QplugAppendTextDisplay("\n");
       } 
       
       // remove spaces and comma
       stripSpace(arguments);  
     }
     
     // get arguments for each IO variable 
     // not necessarily in order, so search list each time
     
     while(contentBuf.length() != 0){
       QString testName = getWord(contentBuf);
       QString varArgs = getToken(contentBuf, '{', '}');
       if (testName.isEmpty() || varArgs.isEmpty()) return;

       bool found = false;
       for (int i=0; i<qtpl_vars.size(); i++){
         if(testName == qtpl_vars[i]->name){
	   found = true;
	   qtpl_vars[i]->QtplGetParserVar(varArgs);
	 }
       } 
       
       if(!found){
         QplugAppendTextDisplay("Variable... ");
         QplugAppendTextDisplay(testName.toLatin1().data());
         QplugAppendTextDisplay("\n not set in argument list \n");
	 return;
       }
       
       stripSpace(contentBuf);
     }
     
     openedOK = true;
   }
   else{
      openedOK = false;
   }  

   return;
}


// class qtplParserVar

QtplParserVar::QtplParserVar(QString varName, QString io, QString type){
			     
  name = QString(varName.toLatin1().data());  // force deep copy
  ioType = QString(io.toLatin1().data());      // is this input or output slot
  var_type = QString(type.toLatin1().data()); // data type expected
}

bool QtplParserVar::QtplGetParserVar(QString &varBlock){
  
   //returns false if variable block did not parse OK
   
  // QString varBlock = getToken(sbuf, '{', '}');
   
   while (varBlock.length() > 0){
     QString parameter = getWord(varBlock);
     if(parameter.isNull()) return false;
     
     QString value = getToken(varBlock, '{', '}');
     if(value.isNull()) return false;

     // remove all white space and quotes
     stripSpace(value);
     if (value[0] == '"') value.remove(0,1);
     if (value[value.length()-1] == '"') value.truncate(value.length() -1);
     
     // set values of parameters as we find them
     if(parameter == "PUBLIC_NAME")
        public_name = QString(value.toLatin1().data());
	
     else if(parameter == "DEFAULT_VALUE")
        default_value = QString(value.toLatin1().data());
	
     else if(parameter == "DESCRIPTION")
        description = QString(value.toLatin1().data());

     else {
       QplugAppendTextDisplay("Unknown IO parameter... ");
       QplugAppendTextDisplay(parameter.toLatin1().data());
       QplugAppendTextDisplay("\n");
     }
     
     // remove any trailing junk to ensure clean exit of while
     stripSpace(varBlock);  
   }
   return true;
}

void stripSpace(QString &sbuf){
  
 while(sbuf.length() >0 ){
   int slen = sbuf.length();
   QChar nextChar = sbuf[0];
   if(nextChar.isSpace() || nextChar == ',') sbuf.remove(0,1);
   if(sbuf.length() == 0) break;
   
   nextChar = sbuf[sbuf.length()-1];
   if(nextChar.isSpace() || nextChar == ',') sbuf.truncate(sbuf.length()-1);
   if(sbuf.length() == 0) break;
   
   if(slen == sbuf.length()) break;
 }
 
}

QString getWord(QString &sbuf){

  // finds and returns the next word in the buffer and removes 
  // it from the buffer. White space is removed. 
  // the character { can also end a word
  
  // returns null string if no word found
  
  int p;
  QString word;
  QString null_word;
  
  stripSpace(sbuf);
  
  for(p=0; p<sbuf.length();p++){
    QChar nextChar = sbuf[p];
    if (nextChar.isSpace() || nextChar == '{' || nextChar == ',') break;
  }
  if (p==0) return null_word;
  
  word = QString(sbuf.left(p).toLatin1().data());
  sbuf.remove(0,p);
  
  return word;
}

QString getToken(QString &sbuf, QChar from, QChar to){

  // returns the string contained in paired {}
  // and removes this token from the input sbuf
 
  // returns null string if unable to parse text for variable

  QString token;
  QString null_token;
  
  int p = sbuf.indexOf(from);
  if (p==-1) {
    return null_token;
  }
  
  token = QString(sbuf.toLatin1().data()); // force deep copy
  
  token.remove(0,p+1);
  sbuf.remove(0,p+1);
  int bracketCount = 1;
  p = -1;
  while(bracketCount > 0){
    p++;
    if ( p == token.length() ) {
      QplugAppendTextDisplay("Cannot find closing bracket for block starting...");
      QString startingWith(from);
      startingWith += token.left(30);
      QplugAppendTextDisplay(startingWith.toLatin1().data());
      QplugAppendTextDisplay("\n");
      return null_token;
    }
    if( token[p] == from ) bracketCount++;
    if( token[p] == to ) bracketCount--;
  }
  token.truncate(p);
  sbuf.remove(0,p+1);
  return token;
  
}

