
/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

#include "qplug_if.h"
#include "DvObj.h"

extern "C" QplugReturnStatus DvToUnits(vector<DvObject_var>&, vector<DvObject_var>&);

QplugReturnStatus DvToUnits(vector<DvObject_var>& inputs, vector<DvObject_var>& outputs)
{
	DvObject_var in_ptr;
	DvObject_var out_ptr;
	DvObject_var SI_target;
	DvObject_var Units_target;

	DvString SIcTarget;
	DvString UnitsTarget;

	// Get name of output object
	DvString out_name;
 	out_name = GetSlotText(0);
 	if (out_name.empty()) {
    	out_name = "InNewUnits";
    	SetSlotText(0, out_name.c_str());
	}

	if(inputs.size() < 3) {
    	QplugAppendTextDisplay ("3 inputs required\n");
    	return QPLUG_FAILURE; 
 	} 
 
	//	Unpack the input data objects

	in_ptr = inputs[0];
 	SI_target = inputs[1];
	Units_target = inputs[2];

	if(in_ptr.is_nil() || SI_target.is_nil() || Units_target.is_nil()){
    	QplugAppendTextDisplay ( "Invalid input object \n");
    	return QPLUG_FAILURE; 
	}

	if( SI_target->is_str() ){
		 SIcTarget = SI_target->str(0);
	}
 	else
 	{
 		SIcTarget = SI_target->getXrefText(SI_CONVERSION);
	}

	if( !in_ptr->sameBaseSI(SIcTarget.c_str()) ){
    	QplugAppendTextDisplay ( "Not same base SI units \n");
    	return QPLUG_FAILURE; 
	}

	if( Units_target->is_str() ){
		 UnitsTarget = Units_target->str(0);
	}
 	else
 	{
 		UnitsTarget = Units_target->getXrefText(UNITS);
	}
 

	// create output object
    // this creates new converted object, copies xrefs across and sets SI_conversion

	out_ptr = in_ptr->changeUnitsTo(SIcTarget, UnitsTarget);
  
 
	if (out_ptr->nError() > 0) QplugAppendTextDisplay( out_ptr->getError().c_str() );

	//	..add  the output series to the list

	outputs.push_back(out_ptr);

	QplugAppendTextDisplay ("Convert to Base SI - Done.\n");
  
	return QPLUG_SUCCESS;

}


