/**********************************************************************
* SCCS id: @(#)CorCoefPlug.h	1.1 14:32:33 12/17/98 
***********************************************************************
* PROJECT:    Cluster UK CDHF
* COMPONENT:  Correlation Coefficient Plugin
* MODULE:     CorCoefPlug.h header file 
* LANGUAGE:   C++
* PURPOSE:    header file
* DESCRIPTION:header file
* HISTORY:    SJ Schwartz Dec 1998
*/

#ifndef CORCOEFPLUG
#define CORCOEFPLUG   // if not defined, define and include rest of file.

#ifndef __cplusplus
#define __cplusplus
#endif

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "DvObj.h"
#include "qdutil.h"
#include "IndexTSSubset.h"
#include "CorCoefPlug_utils.h"
#include "CorCoef.h"
//#include "qtui_if.h"


using namespace QSAS;

//#define VERBOSE_CC   // for extra comments from plug-in

typedef struct CorCoefPlug_ptrs_s{    // this structure holds EVERYTHING!
	DvObject_var xdo_vec;
	DvObject_var xdo;
	float *x_vx;                // x data series and tags
	float *x_vy;
	float *x_vz;
	float *x;               
	double *tx; 
	int nxdata_tot;               // total no. of x data points
	int nxsub;                    // number in subsetted interval
	int ixstart;                  // tx[istart] <=> start of interval
	int xvecset;

	DvObject_var ydo_vec;
	DvObject_var ydo;
	float *y_vx;             // all the same stuff for y
	float *y_vy;
	float *y_vz;
	float *y;                
	double *ty; 
	int nydata_tot;
	int nysub;
	int iystart;
	int yvecset;

	double dt_spacing;	

	DvObject_var interval_do;
	double tstart;
	double tend;
	float dt_lag_max;             // from input

	float *rho;                  // output arrays
	float *rho_vx;              
	float *rho_vy;              
	float *rho_vz;   
	int rvecset;           
	float *dt_lag;
	int   out_array_size;
	int   nlags;
} CorCoefPlug_ptrs;

static const char* CorCoefPlugReturnMessage[5] = 
   { "CorCoefPlug>> Error zero undefined\n",
     "CorCoefPlug>> Input type check/load failed\n",
     "CorCoefPlug>> Subset failure - fatal\n",
     "CorCoefPlug>> Failed to create output data object\n",
     "CorCoefPlug >> Success\n"};

typedef enum CorCoefPlugReturnStatus_e{
  CORCOEF_PLUG_ERROR_UNDEFINED,
  CORCOEF_PLUG_CHECK_INPUT_ERROR,
  CORCOEF_PLUG_SUBSET_ERROR,
  CORCOEF_PLUG_CREATE_OUTPUT_FAIL,
  CORCOEF_PLUG_SUCCESS
} CorCoefPlugReturnStatus;


/* C++ prototypes */

CorCoefPlugReturnStatus  CorCoefPlug_ini(CorCoefPlug_ptrs *);
CorCoefPlugReturnStatus  CorCoefPlug_checkload_input(vector <DvObject_var>& inputs, CorCoefPlug_ptrs *);
CorCoefPlugReturnStatus  CorCoefPlug_SubsetXY (CorCoefPlug_ptrs *);
CorCoefPlugReturnStatus  CorCoefPlug_create_out_DOs(vector <DvObject_var>& outputs, CorCoefPlug_ptrs *);
QplugReturnStatus  CorCoefPlug_cleanup  (QplugReturnStatus , CorCoefPlug_ptrs *);
     
extern "C" QplugReturnStatus  DvCorCoefStart(vector <DvObject_var>&, vector <DvObject_var>&);

                                  
#endif    // end define CORCOEFPLUG

