#include <stdio.h>
#include <math.h>

#ifdef HAVE_IEEEFP_H
 #include <ieeefp.h>
#endif

#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "qdutil.h"
#include "DvObj.h"


using namespace QSAS;

// local prototypes
extern "C" QplugReturnStatus DvStripLT (vector <DvObject_var>&, vector <DvObject_var>&);
extern "C" QplugReturnStatus DvStripGT (vector <DvObject_var>&, vector <DvObject_var>&);
extern "C" QplugReturnStatus DvStripNaN (vector <DvObject_var>&, vector <DvObject_var>&);
extern "C" QplugReturnStatus DvStripFills (vector <DvObject_var>&, vector <DvObject_var>&);
extern "C" QplugReturnStatus DvConvertFills (vector <DvObject_var>&, vector <DvObject_var>&);
extern "C" QplugReturnStatus DvNumericMask (vector <DvObject_var>&, vector <DvObject_var>&);
extern "C" QplugReturnStatus DvStringMask (vector <DvObject_var>&, vector <DvObject_var>&);
extern "C" QplugReturnStatus DvApplyMask (vector <DvObject_var>&, vector <DvObject_var>&);

// -------------------------------------------------

QplugReturnStatus DvNumericMask( vector <DvObject_var>& inputs, vector <DvObject_var>& outputs )
{
	 
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data objects

	DvObject_var dobj = inputs[0];
	DvObject_var cond = inputs[1];
	DvObject_var userValue = inputs[2];
	   
	if(dobj.is_nil() || userValue.is_nil() || cond.is_nil() ){
		QplugAppendTextDisplay( "Missing input object in filter\n" );
		return QPLUG_FAILURE; 
	}
	
	if(dobj->arraySize() > 1){
		QplugAppendTextDisplay( "Only scalar object handled\n" );
		return QPLUG_FAILURE; 
	}
	
	double value = userValue->asDouble();
	DvString condition = cond->asStr();
	

	// initialise mask 
	DvObject_var msk = new DvObject((int)1, dobj->seqSize());

	if( condition.clean() == "=" || condition.clean() == "==")
	{
		if(dobj->is_int()){
			int ivalue = userValue->asInt();
			for(size_t i=0;i<dobj->seqSize();i++){
				if(dobj->itg(i) == ivalue) msk->itg(i) = 0;
			}
		}
		else{
			for(size_t i=0;i<dobj->seqSize();i++){
				if(dobj->asDouble(i) == value) msk->itg(i) = 0;
			}
		}
	}
	else if( condition.clean() == ">")
	{
		for(size_t i=0;i<dobj->seqSize();i++){

			if(dobj->asDouble(i) > value) msk->itg(i) = 0;
		}
	}
	else if( condition.clean() == "<")
	{
		for(size_t i=0;i<dobj->seqSize();i++){

			if(dobj->asDouble(i) < value) msk->itg(i) = 0;
		}
	}
	else if( condition.clean() == ">=")
	{
		for(size_t i=0;i<dobj->seqSize();i++){

			if(dobj->asDouble(i) >= value) msk->itg(i) = 0;
		}
	}
	else if( condition.clean() == "<=")
	{
		for(size_t i=0;i<dobj->seqSize();i++){

			if(dobj->asDouble(i) <= value) msk->itg(i) = 0;
		}
	}
	else if( condition.clean() == "!=")
	{
		for(size_t i=0;i<dobj->seqSize();i++){

			if(dobj->asDouble(i) != value) msk->itg(i) = 0;
		}
	}
	
	DvObject_var qdo = dobj->getDep0();
	if ( qdo.is_ok() ) msk->change_xref(DEPEND_0 , qdo );
	
		
	outputs.push_back(msk);
	QplugAppendTextDisplay ("Mask created: Done\n");
	return QPLUG_SUCCESS;

}

// -------------------------------------------------

QplugReturnStatus DvStringMask( vector <DvObject_var>& inputs, vector <DvObject_var>& outputs )
{
	 
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data objects

	//  Unpack the input data objects

	DvObject_var dobj = inputs[0];
	DvObject_var cond = inputs[1];
	DvObject_var userValue = inputs[2];
	   
	if(dobj.is_nil() || userValue.is_nil() || cond.is_nil() ){
		QplugAppendTextDisplay( "Missing input object in filter\n" );
		return QPLUG_FAILURE; 
	}
	
	if(dobj->arraySize() > 1 ){
		QplugAppendTextDisplay( "Only scalar stringt object handled\n" );
		return QPLUG_FAILURE; 
	}
	
	DvString value = userValue->asStr();
	DvString condition = cond->asStr();
	

	// initialise mask 
	DvObject_var msk = new DvObject((int)1, dobj->seqSize());

	if( condition.clean() == "=" || condition.clean() == "==")
	{
		for(size_t i=0;i<dobj->seqSize();i++){

			if(dobj->asStr(i) == value) msk->itg(i) = 0;
		}
	}
	else if( condition.clean() == "!=")
	{
		for(size_t i=0;i<dobj->seqSize();i++){

			if(dobj->asStr(i) != value) msk->itg(i) = 0;
		}
	}
	else if( condition.clean() == "contain" || condition.clean() == "contains")
	{
		for(size_t i=0;i<dobj->seqSize();i++){

			if(dobj->asStr(i).contains(value)) msk->itg(i) = 0;
		}
	 }
	else if( condition.clean() == "!contain" || condition.clean() == "!contains")
	{
		for(size_t i=0;i<dobj->seqSize();i++){

			if( !dobj->asStr(i).contains(value)) msk->itg(i) = 0;
		}
	}
	
	DvObject_var qdo = dobj->getDep0();
	if ( qdo.is_ok() ) msk->change_xref(DEPEND_0 , qdo );
	
		
	outputs.push_back(msk);
	QplugAppendTextDisplay ("Mask created: Done\n");
	return QPLUG_SUCCESS;

}

// -------------------------------------------------

QplugReturnStatus DvApplyMask( vector <DvObject_var>& inputs, vector <DvObject_var>& outputs )
{
	 
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data objects

	DvObject_var dobj = inputs[0];
	DvObject_var mask = inputs[1];
	   
    if(dobj.is_nil() || mask.is_nil()  ){
        QplugAppendTextDisplay( "Missing input object in apply mask\n" );
        return QPLUG_FAILURE; 
    }
    
    
  	// create output object
	mask = mask->Join(dobj);

	if(mask.is_nil()){
		QplugAppendTextDisplay( "Failed to join mask onto object\n" );
		return QPLUG_FAILURE; 
	}
	    
	// initialise DvMask
	DvMask msk(mask->seqSize());

	for(size_t i=0;i<mask->seqSize();i++)
	{
		if(mask->itg(i) < 0.99){
			msk[i] = false;
		}
	}

	DvObject_var obj_out = new DvObject(dobj);

	obj_out->apply_mask(msk);
	  
	outputs.push_back(obj_out);
	QplugAppendTextDisplay ("Mask Applied: Done\n");
	return QPLUG_SUCCESS;


}


// -------------------------------------------------

QplugReturnStatus DvStripGT( vector <DvObject_var>& inputs, vector <DvObject_var>& outputs )
{
	 
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data objects

	DvObject_var dobj = inputs[0];
	DvObject_var userValue = inputs[1];
	   

	if(dobj.is_nil() || userValue.is_nil()  ){
		QplugAppendTextDisplay( "Missing input object in remove greater than filter\n" );
		return QPLUG_FAILURE; 
	}
 
	DvObject_var limitVal = userValue; // shallow copy
	if(limitVal->seqSize() == 1)
	{
		limitVal = new DvObject(userValue->asDouble(), dobj->seqSize());
	 
	}
	// we now have a sequence of limit values 
	    
	// initialise mask
	DvMask msk(dobj->seqSize());

	for(size_t i=0;i<dobj->seqSize();i++)
	{
		double limit = limitVal->asDouble(i);

		for(size_t j=0; j<dobj->arraySize(); j++)
		{
			if(dobj->asDouble(i, j) > limit){
				msk[i] = false;
				break;
			}
		}
	}

	DvObject_var obj_out = new DvObject(dobj);

	obj_out->apply_mask(msk);
	  
	outputs.push_back(obj_out);

	QplugAppendTextDisplay ("GT Filter: Done\n");
	return QPLUG_SUCCESS;

}


QplugReturnStatus DvStripLT( vector <DvObject_var>& inputs, vector <DvObject_var>& outputs )
{
	 
	 
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data objects

	DvObject_var dobj = inputs[0];
	DvObject_var userValue = inputs[1];
	   

	if(dobj.is_nil() || userValue.is_nil()  ){
		QplugAppendTextDisplay( "Missing input object in remove less than filter\n" );
		return QPLUG_FAILURE; 
	}
 
	DvObject_var limitVal = userValue; // shallow copy
	if(limitVal->seqSize() == 1)
	{
		limitVal = new DvObject(userValue->asDouble(), dobj->seqSize());
	 
	}
	// we now have a sequence of limit values 
	    
	// initialise mask
	DvMask msk(dobj->seqSize());

	for(size_t i=0;i<dobj->seqSize();i++)
	{
		double limit = limitVal->asDouble(i);

		for(size_t j=0; j<dobj->arraySize(); j++)
		{
			if(dobj->asDouble(i, j) < limit){
				msk[i] = false;
				break;
			}
		}
	}

	DvObject_var obj_out = new DvObject(dobj);

	obj_out->apply_mask(msk);
	  
	outputs.push_back(obj_out);

	QplugAppendTextDisplay ("LT Filter: Done\n");
	return QPLUG_SUCCESS;

}

QplugReturnStatus DvStripNaN( vector <DvObject_var>& inputs, vector <DvObject_var>& outputs )
{
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data object

	DvObject_var dobj = inputs[0];
	
	if(dobj.is_nil()  ){
		QplugAppendTextDisplay( "Invalid input object in strip NaN filter\n" );
		return QPLUG_FAILURE; 
	}
 	
	if(dobj->not_dbl())
	{
		QplugAppendTextDisplay( "Input object must be double data\n" );
		return QPLUG_FAILURE;
	}

	// initialise mask
	DvMask msk(dobj->seqSize());

	for(size_t i=0;i<dobj->seqSize();i++)
	{
		for(size_t j=0; j<dobj->arraySize(); j++)
		{
			if( !isfinite(dobj->asDouble(i, j)) ){
				msk[i] = false;
				break;
			}
		}
	}

	DvObject_var obj_out = new DvObject(dobj);

	obj_out->apply_mask(msk);
	  
	outputs.push_back(obj_out);
	
		
	QplugAppendTextDisplay ("NaN Filter: Done\n");
	return QPLUG_SUCCESS;


}



QplugReturnStatus DvStripFills( vector <DvObject_var>& inputs, vector <DvObject_var>& outputs )
{
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data object

	DvObject_var dobj = inputs[0];
	
	if(dobj.is_nil()  ){
		QplugAppendTextDisplay( "Invalid input object in strip NaN filter\n" );
		return QPLUG_FAILURE; 
	}
 	
	// get Fillval
    DvObject_var fill_xref = dobj->get_xref(FILL_VALUE);
	if(fill_xref.is_nil()){
		QplugAppendTextDisplay( "No Fillval attribute in object\n" );
		return QPLUG_FAILURE; 
	}

	// initialise mask
	DvMask msk(dobj->seqSize());

	if(dobj->is_dbl() && fill_xref->arraySize() == 1){
		double fill = fill_xref->asDouble();
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( isFill(dobj->dbl(i, j), fill) ){
					msk[i] = false;
					break;
				}
			}
		}
	}
	else if(dobj->is_dbl() && fill_xref->arraySize() == dobj->arraySize()){
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				double fill = fill_xref->asDouble(j);
				if( isFill(dobj->dbl(i, j), fill) ){
					msk[i] = false;
					break;
				}
			}
		}
	}
	else if(dobj->is_int() && fill_xref->arraySize() == 1){
		int fill = fill_xref->asInt();
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( dobj->itg(i, j) == fill ){
					msk[i] = false;
					break;
				}
			}
		}
	}
	else if(dobj->is_int() && fill_xref->arraySize() == dobj->arraySize()){
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				int fill = fill_xref->asInt(j);
				if( dobj->itg(i, j) == fill ){
					msk[i] = false;
					break;
				}
			}
		}
	}
	else if(dobj->is_str() && fill_xref->arraySize() == 1){
		DvString fill = fill_xref->asStr();
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( dobj->str(i, j) == fill ){
					msk[i] = false;
					break;
				}
			}
		}
	}
	else if(dobj->is_int() && fill_xref->arraySize() == dobj->arraySize()){
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				DvString fill = fill_xref->asStr(j);
				if( dobj->str(i, j) == fill ){
					msk[i] = false;
					break;
				}
			}
		}
	}
	else if(dobj->is_time() && fill_xref->arraySize() == 1){
		DvTime fill = fill_xref->asTime();
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( dobj->time(i, j) == fill ){
					msk[i] = false;
					break;
				}
			}
		}
	}
	else if(dobj->is_time() && fill_xref->arraySize() == dobj->arraySize()){
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				DvTime fill = fill_xref->asTime(j);
				if( dobj->time(i, j) == fill ){
					msk[i] = false;
					break;
				}
			}
		}
	}
	else if(dobj->is_event() && fill_xref->arraySize() == 1 && fill_xref->is_event()){
		DvEvent fill = fill_xref->event();
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( dobj->event(i, j) == fill ){
					msk[i] = false;
					break;
				}
			}
		}
	}
	else if(dobj->is_event() && fill_xref->arraySize() == dobj->arraySize() && fill_xref->is_event()){
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				DvEvent fill = fill_xref->event(j);
				if( dobj->event(i, j) == fill ){
					msk[i] = false;
					break;
				}
			}
		}
	}
	else{
		QplugAppendTextDisplay( "Dimensions or type combination not handled\n" );
		return QPLUG_FAILURE; 
	}
	
	DvObject_var obj_out = new DvObject(dobj);

	obj_out->apply_mask(msk);
	  
	outputs.push_back(obj_out);

	QplugAppendTextDisplay ("Remove fill values Filter: Done\n");
	return QPLUG_SUCCESS;
}

// -------------------------------------------------

QplugReturnStatus DvConvertFills( vector <DvObject_var>& inputs, vector <DvObject_var>& outputs )
{
	QplugAppendTextDisplay("\n\n");

	//  Unpack the input data objects

	DvObject_var dobj = inputs[0];
	DvObject_var userValue = inputs[1];
	
	if(dobj.is_nil() || userValue.is_nil()  ){
		QplugAppendTextDisplay( "Missing input object\n" );
		return QPLUG_FAILURE; 
	}
 	
	// get Fillval
    DvObject_var fill_xref = dobj->get_xref(FILL_VALUE);
	if(fill_xref.is_nil()){
		QplugAppendTextDisplay( "No Fillval attribute in object\n" );
		return QPLUG_FAILURE; 
	}   
	 

	DvObject_var obj_out = new DvObject(dobj);

	if(dobj->is_dbl() && fill_xref->arraySize() == 1){
		double fill = fill_xref->asDouble();
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( isFill(dobj->dbl(i, j), fill) ){
					obj_out->dbl(i, j) = userValue->asDouble();
				}
			}
		}
	}
	else if(dobj->is_dbl() && fill_xref->arraySize() == dobj->arraySize()){
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				double fill = fill_xref->asDouble(j);
				if( isFill(dobj->dbl(i, j), fill) ){
					obj_out->dbl(i, j) = userValue->asDouble();
				}
			}
		}
	}
	else if(dobj->is_int() && fill_xref->arraySize() == 1){
		int fill = fill_xref->asInt();
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( dobj->itg(i, j) == fill ){
					obj_out->itg(i, j) = userValue->asInt();
				}
			}
		}
	}
	else if(dobj->is_int() && fill_xref->arraySize() == dobj->arraySize()){
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				int fill = fill_xref->asInt(j);
				if( dobj->itg(i, j) == fill ){
					obj_out->itg(i, j) = userValue->asInt();
				}
			}
		}
	}
	else if(dobj->is_str() && fill_xref->arraySize() == 1){
		DvString fill = fill_xref->asStr();
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( dobj->str(i, j) == fill ){
					obj_out->str(i, j) = userValue->asStr();
				}
			}
		}
	}
	else if(dobj->is_int() && fill_xref->arraySize() == dobj->arraySize()){
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				DvString fill = fill_xref->asStr(j);
				if( dobj->str(i, j) == fill ){
					obj_out->str(i, j) = userValue->asStr();
				}
			}
		}
	}
	else if(dobj->is_time() && fill_xref->arraySize() == 1){
		DvTime fill = fill_xref->asTime();
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( dobj->time(i, j) == fill ){
					obj_out->time(i, j) = userValue->asTime();
				}
			}
		}
	}
	else if(dobj->is_time() && fill_xref->arraySize() == dobj->arraySize()){
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				DvTime fill = fill_xref->asTime(j);
				if( dobj->time(i, j) == fill ){
					obj_out->time(i, j) = userValue->asTime();
				}
			}
		}
	}
	else if(dobj->is_event() && fill_xref->arraySize() == 1 && fill_xref->is_event() && userValue->is_event()){
		DvEvent fill = fill_xref->event();
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				if( dobj->event(i, j) == fill ){
					obj_out->event(i, j) = userValue->event();
				}
			}
		}
	}
	else if(dobj->is_event() && fill_xref->arraySize() == dobj->arraySize() && fill_xref->is_event() && userValue->is_event()){
		for(size_t i=0;i<dobj->seqSize();i++)
		{
			for(size_t j=0; j<dobj->arraySize(); j++)
			{
				DvEvent fill = fill_xref->event(j);
				if( dobj->event(i, j) == fill ){
					obj_out->event(i, j) = userValue->event();
				}
			}
		}
	}
	else{
		QplugAppendTextDisplay( "Dimensions or type combination not handled\n" );
		return QPLUG_FAILURE; 
	}
	
	obj_out->change_xref(FILL_VALUE, userValue);

	outputs.push_back(obj_out);

	QplugAppendTextDisplay ("Convert fill values Filter: Done\n");
	return QPLUG_SUCCESS;

 }
