
/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/
/* PROJECT: Cluster UK CDHF
*
* COMPONENT:    Centre FPI timetags using delta_plus/minus if available else dobj name
* 
* CAUTION: name parsing is unique to fpi L2 products (and perhaps l1b)?
* RETURN:
*    TYPE:    QplugReturnStatus
*    VALUES:
*
*
*
* HISTORY:   SJS 8 May 2016; 
*/

#include "qplug_if.h"
#include "DvObj.h"

#define DEBUG_CENTRE_TT

QplugReturnStatus CentreObjTimetags (DvObject_var dobj_ptr){
	DvObject_var tags_ptr, dep0, dplus_var, dminus_var, dt_shift_var, delta_var;
	bool delta_ok=false;
	DvString obj_name = dobj_ptr->getXrefText("ObjectName");//

#ifdef DEBUG_CENTRE_TT
cout << " got object with objname = " << obj_name.c_str() << endl;
#endif	
	if(dobj_ptr->xref_exists("DEPEND_0")){
		dep0 = dobj_ptr->get_xref("DEPEND_0");
		tags_ptr = new DvObject(dep0);  // work with copy
		// check for delta_plus & minus; should always be true as QSAS
		// will add them I believe.
#ifdef DEBUG_CENTRE_TT
cout << " got object with objname = " << obj_name.c_str() << endl;
if (tags_ptr->is_nil()){
	cout << "tags_ptr is nil!! " << endl;
} else {
cout << "tags_ptr not nil " << endl;
}

cout << "time interval start is " <<   tags_ptr->getTimeRange().iso_srep() << endl;
#endif	
		if(tags_ptr->xref_exists("DELTA_PLUS") && tags_ptr->xref_exists("DELTA_MINUS")){
			dplus_var = tags_ptr->get_xref("DELTA_PLUS");
			dminus_var= tags_ptr->get_xref("DELTA_MINUS");

			if ( dplus_var->sameBaseSI("1>s") && 
			  (dplus_var->getSIC().trimmed() == dminus_var->getSIC().trimmed())){
			  //ok; both dplus and dminus have good SI_conversion;
				delta_ok = true;
			} else {  // otherwise construct them
				delta_ok = false;
				QplugAppendTextDisplay("Timetag Delta_plus/minus of "+obj_name+" missing SI_CONVERSION; using object name, resolution to set them");			
			}
		} else {
			delta_ok = false;
			QplugAppendTextDisplay("No delta_plus/minus for timetags of "+obj_name+"; will construct from object name");
		}
		
		if (!delta_ok){
			// construct dplus/minus 
			//unless dminus exists and is >0 in which case give up
			if (tags_ptr->xref_exists("DELTA_MINUS")){
				dminus_var = tags_ptr->get_xref("DELTA_MINUS");
				if (dminus_var->asDouble() > 0.0){
					QplugAppendTextDisplay(obj_name+" has delta_minus but not zero; can't construct from obj name; give up");
					return QPLUG_FAILURE;
				}
			}
			// orig fpi timetags are always start times, so delta_minus = 0
			dminus_var = new DvObject(0.0);
			dminus_var->change_xref("SI_CONVERSION","1>s");
			dplus_var = new DvObject(dminus_var);
			if (obj_name.contains("fast")){          //fast survey has 4.5s res
				*dplus_var +=4.5; 
			} else if (obj_name.contains("brst")){
				if (obj_name.contains("des")){
					*dplus_var +=0.03;                // brst electrons have 0.03s
				} else if (obj_name.contains("dis")){
					*dplus_var +=0.150;               // brst ions have 0.150s
				} else {
					QplugAppendTextDisplay(obj_name+" can't be parsed for timetag delta_plus/minus; giving up");
					return QPLUG_FAILURE;
				}
			}					
		}
		dt_shift_var = new DvObject(dplus_var);
		dt_shift_var -=dminus_var;
		*dt_shift_var *=0.5;
		tags_ptr += dt_shift_var;
		delta_var = new DvObject(dplus_var);
		delta_var +=dminus_var;
		*delta_var *=0.5;
		delta_var->change_xref("UNITS","s");
		delta_var->change_xref("FIELDNAM","epoch delta");
		tags_ptr->change_xref("DELTA_PLUS",delta_var);
		tags_ptr->change_xref("DELTA_MINUS",delta_var);
		dobj_ptr->change_xref("DEPEND_0",tags_ptr);
		dobj_ptr->change_xref("CentredTags","Timetags centred by MMS_FPI_CDF_moments plugin");
		QplugAppendTextDisplay("TimeTags centred for "+dobj_ptr->getXrefText("ObjectName"));
		return QPLUG_SUCCESS;
	} else {
		QplugAppendTextDisplay("No timetags for "+dobj_ptr->getXrefText("ObjectName")+"; timetag centering aborted.");
		return QPLUG_FAILURE;
	}
		
}

