#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#include "qplug_if.h"
#include "qdutil.h"
#include "DvObj.h"

using namespace QSAS;

// Make the function accessible from external modules 
extern "C" QplugReturnStatus DvQarRemainder( vector<DvObject_var>& inputs, vector<DvObject_var>& outputs )
{

   // Define some variables
 
   // Display initial message in QpluGui window
   QplugAppendTextDisplay( "Remainder Plugin 1.1 Mar 2016\n" );
   
   // ( 1 ) Unpack the input data objects
   // This will consist of two series
   // Serie 1: List of values to be phased
   // Serie 2: List of phases

   DvObject_var value_in     = inputs[0]; // List of QdObject_var objects 
   DvObject_var divider_in   = inputs[1];
   DvObject_var Reminder_out;
 
   if(value_in.is_nil()  || divider_in.is_nil())  {
        QplugAppendTextDisplay ("Remainder>> ERROR - input not valid\n");
        return QPLUG_FAILURE;    
   }
   
   // ( 2 ) Get name of output objects:
   DvString outName = GetSlotText(0);
   if ( outName.empty() )
   {
      SetSlotText( 0, "Remainder" );
   }


   // ( 4 ) Display what is happening
   QplugAppendTextDisplay ("Remainder>> Calculating.\n");
   
   Reminder_out = value_in->remainder(divider_in);
   

   // FRAME
   if(value_in->isThreeVector())
    QplugAppendTextDisplay ("WARNING: Remainder treats each component independently");

  
   // FIELDNAME
     Reminder_out->change_xref( FIELDNAM, "Remainder" );
   

   // Add output object to  QSAS working list
   outputs.push_back( Reminder_out );
   
   
   QplugAppendTextDisplay( "Remainder: Calculation completed\n" );  
  
   return ( QPLUG_SUCCESS );
}


