/***********************************************************************
*           See Accompanying licence file QSAS_licence                 *
***********************************************************************/

/* PROJECT: Cluster UK CDHF
*
* COMPONENT:    TSSubset
* 
* MODULE:	QSAS Subsetting of Time Series
*
* LANGUAGE:	C++
*
* FUNCTION:
*
* PURPOSE: Subset a time series (any type)
* 
* DESCRIPTION:
*
* ARGUMENTS:
*    IN:
*     ANY_TIME_SERIES    ts_in                 input vector time series 
*                                                  data object
*     TIME_INTERVAL         start_end             time interval data object
*                                                  giving start and end times
*                                                  surrounding output series
*    OUT:
*        ANY_TIME_SERIES ts_out
* GLOBALS:
*
* RETURN:
*    TYPE:     QplugReturnStatus
*    VALUES:   see qplug_if.h
*
* FUNCTIONS_USED:	 
*
* ALGORITHM:   pretty obvious
*
* HISTORY:		Steve Schwartz, 1996
*  Amended to use Qfui interface instead of Qai - A R Hare, 12 Sep 96.
*  Amended to use Qplug interface instead of Qfui - SJ Schwartz 21 May 97
*  Combined Scalar and Vector TS Subset using  "ANY_TIME_SERIES" args 21/5/97
*  Fixed memory leaks on failure and in attr. handling   27/6/97
*
***********************************************************************/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "qplug_if.h"
#include "DvObj.h"
#include "qdutil.h"

extern "C" QplugReturnStatus  DvTSSubset  ( vector<DvObject_var>& inputs, vector<DvObject_var>& outputs )
{

DvEvent start_end;
DvObject_var ts_in;
DvObject_var ts_out;


//
//	...first, unpack the input data objects
//

//	..first item is input Time Series

  ts_in = inputs[0];

//	2nd item is time interval for start and stop

  start_end = inputs[1]->getTimeRange();
 
  ts_out = ts_in->subSequence(start_end);
  
  outputs.push_back(ts_out);


  QplugAppendTextDisplay ("TSSubset >> data object created - exiting successfully\n");
  return QPLUG_SUCCESS;

}  // end TSSubset
