/*****************************************************************
* PROJECT:  Cluster UK CDHF
* COMPONENT:  Plug In library
* MODULE:  InvTrigoFunct
* LANGUAGE:  C++
* FUNCTION: QSAS InvTrigoFunct  Plug In
* PURPOSE: .
* 	
* ARGUMENTS:  
*    IN: ANY DATA OBJECT
*        integer: function identifier
*	 integer: deg/rad
*    OUT: 
*	ANY DATA OBJECT 
*
* RETURN: QplugReturnStatus
*
* History: version 1.0 Created Jul. 2002. 
* 	   version 1.1 July 2002.
* A. Serroukh, 
* Migrated to DVOS A.J.Allen March 2015
******************************************************************/

#include<stdio.h>
#include<math.h>
#include<string.h>
#include<stdlib.h>
#include<fstream>

#include "qplug_if.h"
#include "qdutil.h"
#include "DvObj.h"

using namespace QSAS;

/*****************************************************************/
extern "C" QplugReturnStatus DvInvTrigo(vector<DvObject_var>& inputs, vector<DvObject_var>& outputs){

    DvObject_var  any_obj = inputs[0];
    DvObject_var  obj_fct = inputs[1]; 
    DvObject_var  deg_or_rad= inputs[2];
   
    bool indeg = false;
    bool hyperbolic = true; 
    
    DvObject_var outObj; 
      
    //-----unpack and test the narrow operations ------------   
    if( any_obj.is_nil() ){
     QplugAppendTextDisplay("Error>> input neither const nor sequence.\n");
     return QPLUG_FAILURE;
    } 
               
    if(obj_fct.is_nil()){
        QplugAppendTextDisplay("Error>> undefined function\n");
        return QPLUG_FAILURE;
    }       
    int obj_fct_id = obj_fct->asInt();
    if(obj_fct_id < 4 ) hyperbolic = false; 

    if(deg_or_rad.is_nil()){
        QplugAppendTextDisplay("Error>> Deg/Rad unspecified.\n");
        return QPLUG_FAILURE;
    }        
    if(deg_or_rad->asInt() == 0) indeg = true;  
     
    //-----read and check what's in the SI_conversion ----------  
    
    if(any_obj->hasUnits()){
        QplugAppendTextDisplay("Warning: Data has units, \n");
        QplugAppendTextDisplay(any_obj->getXrefText(SI_CONVERSION).c_str());
    }
    
    
    if(!hyperbolic && deg_or_rad->asInt() != 0 && deg_or_rad->asInt() != 1 ){
        QplugAppendTextDisplay("Error>> must specify degree or rad for non-hyperbolic functions.\n");
        return QPLUG_FAILURE;	  
    }

    switch(obj_fct_id){
        case 0: 
            outObj =  any_obj->acos(indeg);
            break;
        case 1:  
            outObj = any_obj->asin(indeg);
            break;
        case 2:  
            outObj = any_obj->atan(indeg);
            break;
        case 3:   
            outObj = any_obj->acot(indeg);	      
            break;
        case 4:
            outObj = any_obj->acosh();
            break;
        case 5: 
            outObj = any_obj->asinh();
            break;	
        case 6:  
            outObj = any_obj->atanh();
            break;
        case 7:           
            outObj = any_obj->acoth();
            break;				
        default:         
            QplugAppendTextDisplay("Error>> Please choose from acos,asin,...\n");
            return QPLUG_FAILURE;
    } 
    
    if( outObj.is_nil() ){
       QplugAppendTextDisplay( "Error>> Invalid input, nil object\n");
       return QPLUG_FAILURE;
    }
     
    outputs.push_back(outObj);  
         		
    return QPLUG_SUCCESS;

}

