//  qbu.cc
//
// Data Browser Widget
//
// July 2002: Tony Allen
//

#include <stdlib.h>
#include "qt_version.h"

#include "qt_version.h"

#include "qbu.h"

#include "qdutil.h"
#include "qwui.h"

extern qwuiWindow* qsasWindow;
extern void QdaLaunchWindow(QString,QWidget*);


XrefNameEditor::XrefNameEditor(QObject *parent):
	QItemDelegate(parent)
{
	viewer = dynamic_cast<QbObjViewer*>(parent);
}

QWidget * XrefNameEditor::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
	QLineEdit *e=new QLineEdit(parent);
	e->setFrame(false);
	return e;
}

void XrefNameEditor::setEditorData(QWidget *editor, const QModelIndex &index) const
{
	QString value = index.model()->data(index, Qt::DisplayRole).toString();

	QLineEdit *edit = static_cast<QLineEdit*>(editor);
	edit->setText(value);
	
	// get object and current name for saving in setModelData()
	DvObject_var xrefObj;
	xrefObj = viewer->activeEditor->edited_var->get_xref(value.toLatin1().data());
	viewer->activeEditor->edited_var->change_xref("nameBeingChanged", xrefObj);
	viewer->activeEditor->edited_var->delete_xref(value.toLatin1().data());
	
}
		
void XrefNameEditor::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{

	QLineEdit *edit = static_cast<QLineEdit*>(editor);
	QString value = edit->text();

	model->setData(index, value, Qt::EditRole);	

	DvObject_var xrefObj;

	xrefObj = viewer->activeEditor->edited_var->get_xref("nameBeingChanged");
	viewer->activeEditor->edited_var->change_xref(value.toLatin1().data(), xrefObj);
	viewer->activeEditor->edited_var->delete_xref("nameBeingChanged");
		
	viewer->refreshList();
}

void XrefNameEditor::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &/*index*/) const
{
	editor->setGeometry(option.rect);
}


XrefListItem::XrefListItem(XrefListView* parent, const QString& text):
	QListWidgetItem(text, parent)
{
	setFlags (flags () | Qt::ItemIsEditable);
}


XrefListView::XrefListView(QWidget* parent):
	QListWidget(parent)
{
	setDragEnabled(true);
	viewport()->setAcceptDrops(true);
	setDropIndicatorShown(true);

	//setDragDropMode(QAbstractItemView::InternalMove);
	setDragDropMode(QAbstractItemView::DragDrop);
	setSelectionMode(QAbstractItemView::ExtendedSelection);
		
	delegate=new XrefNameEditor(parent);
	setItemDelegate(delegate);

	#if QT_VERSION >= 0x040300
	setEditTriggers(QAbstractItemView::NoEditTriggers);
	#endif

}

QMimeData* XrefListView::mimeData ( const QList<QListWidgetItem *> items ) const 
{
	QMimeData *mimeData = new QMimeData();
	
	QStringList idList;

	foreach (QListWidgetItem* item, items) {
		idList << item->text();
	}
	
	QString data=idList.join("//");

	mimeData->setData("application/xrefitems", data.toLatin1().data());
	mimeData->setText(items.first()->text());
	return mimeData;

}
 QStringList XrefListView::mimeTypes() const
{
	QStringList types;
	types << "application/xrefitems";
	return types;
}


void XrefListView::dragEnterEvent(QDragEnterEvent* event)
{
	if (event->mimeData()->hasFormat("application/wlitems"))
	{
		event->acceptProposedAction();
		activateWindow();
	}
}

void XrefListView::dropEvent(QDropEvent* event)
{
	if(!event->mimeData()->hasFormat("application/wlitems"))
		return;

	QStringList itemList=QString(event->mimeData()->data("application/wlitems")).split("//");

	foreach(QString item, itemList)
	{
		QStringList name_decomp=item.split("/");
		QString name=name_decomp[name_decomp.size()-1];

		if( name != viewer->objName() ) // stop drop of obj on its own xref list
		{
			DvObject_var xref_obj = getWLObject(item.toLatin1().data());
			if(!xref_obj.is_nil())
			{
				viewer->activeEditor->edited_var->change_xref(name.toLatin1().data(), xref_obj);
			}
		}
	}

	viewer->refreshList();
}


DataBrowseWindow::DataBrowseWindow(DvObject_var &dobj, QString WLcontext,
				   QWidget *parent, const char *name )
	: QMainWindow(0)
{
    
	QSizePolicy fixed(QSizePolicy::Fixed,QSizePolicy::Fixed);

	this->setPalette(qApp->palette());

	setAttribute(Qt::WA_DeleteOnClose);
  
	msgBar = statusBar();
	
	browser =0;
	browserVB=new QWidget(this);

    browserVBLayout=new QVBoxLayout;
	
	// Name panel
	
	name_panel=new QGroupBox;
	QHBoxLayout * name_panel_layout=new QHBoxLayout;
	
	QLabel *name_label = new QLabel("Name", name_panel);
	name_panel_layout->addWidget(name_label);
	
	name_field = new QguWLinput(name_panel, "name_field");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_panel_layout->addWidget(name_field);
	
	extractBtn = new QPushButton( "Extract...");
	connect( extractBtn, SIGNAL(clicked()), this, SLOT(extract()) );
	extractBtn->setEnabled(true);
	
	name_panel_layout->addWidget(extractBtn);

	
	name_panel->setLayout(name_panel_layout);
	browserVBLayout->addWidget(name_panel);
	

	// Record number panel
	
	recSlider=new QGroupBox;
	QHBoxLayout * recSliderLayout=new QHBoxLayout;
	
	QToolButton * leftEnd = new QToolButton;
	leftEnd->setText("|");
	leftEnd->setFixedSize(20,21);
	recSliderLayout->addWidget(leftEnd);
	
	QToolButton * leftSlide = new QToolButton( recSlider);
	leftSlide->setArrowType(Qt::LeftArrow);
	leftSlide->setFixedSize(20,21);
	recSliderLayout->addWidget(leftSlide);
	
	recBar = new QSlider(Qt::Horizontal, recSlider);
	recSliderLayout->addWidget(recBar);
	
	QToolButton * rightSlide = new QToolButton( recSlider);
	rightSlide->setArrowType(Qt::RightArrow);
	rightSlide->setFixedSize(20,21);
	recSliderLayout->addWidget(rightSlide);
	
	QToolButton * rightEnd = new QToolButton( recSlider);
	rightEnd->setText("|");
	rightEnd->setFixedSize(20,21);
	recSliderLayout->addWidget(rightEnd);
	
	recField = new QguWLinput(recSlider,"recField");
	recField->setAcceptable(Qcondition(QS_NARROW, QS_INT, QS_0D, QS_NOT_SEQ, QS_ANY));
	recField->setEnabled(true);
	recField->setFixedWidth(60);
	recField->setAlignment(Qt::AlignHCenter);
	recSliderLayout->addWidget(recField);
	
	connect( rightSlide, SIGNAL(clicked()), this, SLOT(incRec()) );
	connect( leftSlide,  SIGNAL(clicked()), this, SLOT(decRec()) );
	connect( rightEnd, SIGNAL(clicked()), this, SLOT(lastRec()) );
	connect( leftEnd,  SIGNAL(clicked()), this, SLOT(firstRec()) );
	connect( recBar,   SIGNAL(valueChanged(int)), this, SLOT(adjustRec(int)) );
	connect( recField, SIGNAL(edited_OK()), this, SLOT(inputRec()) );
	
	recSlider->setLayout(recSliderLayout);
	browserVBLayout->addWidget(recSlider);
	
	
 	// Buttons layout
	
	buttons_groupbox=new QGroupBox;
	QHBoxLayout * buttons_layout=new QHBoxLayout;
	
    
    parentsBtn = new QPushButton( " View Source Files... " );
    parentsBtn->setMinimumSize(parentsBtn->sizeHint());
    buttons_layout->addWidget(parentsBtn,5);
	
	
	saveEditsBtn = new QPushButton( "Save Edits");
	saveEditsBtn->setMinimumSize(saveEditsBtn->sizeHint());
	connect( saveEditsBtn, SIGNAL(clicked()), this, SLOT(saveEdits()) );
	saveEditsBtn->setEnabled(true);
	saveEditsBtn->setFocusPolicy(Qt::StrongFocus);
	buttons_layout->addWidget(saveEditsBtn, 5);
	saveEditsBtn->setToolTip("Saves result of editing (will over-write object of same name)");
    saveEditsBtn->setDefault(true);
	
	pad = new QLabel("                     ");
	buttons_layout->addWidget(pad, 5);
	pad->setVisible(false); // used only in time interval and event table editors
	
	dismissBtn = new QPushButton( "    Cancel    ");
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(dismiss()) );
	buttons_layout->addWidget(dismissBtn, 3, Qt::AlignRight);
	
	buttons_groupbox->setLayout(buttons_layout);
	
    // Object view widget
    
    setObject(dobj, WLcontext);

	//Menu
	
	editMenu = new QMenu( this );
	menuBar()->addMenu(editMenu );
	editMenu->setTitle("  Edit ");
	editMenu->addAction("New Metadata", browser, SLOT( newXref() ) );
	editMenu->addAction("Attach Metadata", browser, SLOT( attachXref() ) );
	editMenu->addAction("Add time tags", browser, SLOT( addTimeTags() ));
	editMenu->addSeparator();
	
	QAction *delAct = editMenu->addAction("Delete Metadata", browser, SLOT( deleteXref() ));
    QList<QKeySequence> keyList;
    keyList.append(Qt::Key_Delete);
    keyList.append(Qt::Key_Backspace);
    keyList.append(Qt::CTRL+Qt::Key_X);
    delAct->setShortcuts(keyList);
   
	menuBar()->addSeparator();
	QMenu * help = new QMenu( this );
	menuBar()->addMenu( help );
	help->setTitle("  Help ");
	help->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help->addSeparator();
	help->addAction("...Data Import", this, SLOT( launchManBrowser() ));
	help->addSeparator();
	help->addAction("Module Version", this, SLOT( launchVersion() ));
	
	browserVBLayout->addWidget(browser);
    
    // Error feedback

    if(dobj->nError() > 0){
        error_panel=new QGroupBox;
        error_panel->setTitle("Object has Errors");
        QVBoxLayout * error_panel_layout=new QVBoxLayout;
        for(size_t n=0; n<dobj->nError(); n++){
            QLabel *error_txt = new QLabel(dobj->getError(n).c_str(), error_panel);
            error_panel_layout->addWidget(error_txt);
        }
        
        clearBtn = new QPushButton("Clear Errors", error_panel);
        clearBtn->setMaximumWidth(150);
        error_panel_layout->addWidget(clearBtn);
        connect( clearBtn, SIGNAL(clicked()), this, SLOT(clearErrors()) );

        error_panel->setLayout(error_panel_layout);
        browserVBLayout->addWidget(error_panel);
        
    }
    
   
	browserVBLayout->addWidget(buttons_groupbox);
	browserVB->setLayout(browserVBLayout);
	
	setCentralWidget(browserVB);

	item=qsasWindow->wlview->itemFromPath(WLcontext);
	
}

void DataBrowseWindow::keyPressEvent(QKeyEvent * e )
{
  // use escape key to toggle default buttons
	if(e->key() == Qt::Key_Escape){
        if(saveEditsBtn->isDefault() ) {
			saveEditsBtn->setDefault(false);
			dismissBtn->setDefault(true);
		}
		else if(dismissBtn->isDefault()  )  {
			
            dismissBtn->setDefault(false);
            saveEditsBtn->setDefault(true);
			
		}
		return;
	}
  // use return or enter key to perform default
	if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
		if (saveEditsBtn->isDefault()) saveEdits();
		else if (dismissBtn->isDefault()) dismiss();
		return;
	}

  // else pass on to list view 
	e->ignore(); // let selection change

}


DataBrowseWindow::~DataBrowseWindow(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::~DataBrowseWindow \n");
		fflush(stdout);
	}
 
 
//  delete recField;
	if(browser != 0) delete browser;
}

void DataBrowseWindow::dismiss(){
	dismissBtn->setEnabled(false);
	close();
}

void DataBrowseWindow::launchManual(){ 

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::launchManual \n");
		fflush(stdout);
	}
 
	qsasWindow->help_window->manEntry("index.html");
}

void DataBrowseWindow::launchManBrowser(){ 

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::launchManBrowser \n");
		fflush(stdout);
	}
 
	qsasWindow->help_window->manEntry("QSAS_data_handling.html");
}


void DataBrowseWindow::launchVersion(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::launchVersion \n");
		fflush(stdout);
	}
 
  
	QString text;
	text  = QString("Version Numbers...\n");
	text += QString(QSAS_VERSION_ID);
	text += "\n Data Object Browser  ";
	text += QString(QBU_VERSION);
	QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );

}

void DataBrowseWindow::incRec(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::incRec \n");
		fflush(stdout);
	}
 
	if(browser->activeEditor->record < browser->activeEditor->maxRec){
		browser->activeEditor->record +=1;
	}
	else{
		browser->activeEditor->record = browser->activeEditor->maxRec;
	}
	recBar->setValue(browser->activeEditor->record);

}

void DataBrowseWindow::decRec(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::decRec \n");
		fflush(stdout);
	}
 
	if(browser->activeEditor->record > 0){
		browser->activeEditor->record -= 1;
	}
	else{
		browser->activeEditor->record = 0;
	}
	recBar->setValue(browser->activeEditor->record);
}

void DataBrowseWindow::lastRec(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::lastRec \n");
		fflush(stdout);
	}
 
 
	browser->activeEditor->record = browser->activeEditor->maxRec;
	recBar->setValue(browser->activeEditor->record);

}

void DataBrowseWindow::firstRec(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::firstRec \n");
		fflush(stdout);
	}
 
 
	browser->activeEditor->record = 0;
	recBar->setValue(browser->activeEditor->record);
}

void DataBrowseWindow::inputRec(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::inputRec \n");
		fflush(stdout);
	}
 
	bool ok;
	int rec = recField->text().toInt(&ok);
	if(ok){
		if(rec < 0) rec = 0;
		if(rec > browser->activeEditor->maxRec)  rec = browser->activeEditor->maxRec;
		recField->setText(tr("%1").arg(rec)); // reset field in case changed, does not cause re-validation
		browser->activeEditor->record = rec;
		recBar->setValue(rec);
		browser->activeEditor->updateForRec();
	}
}
 
void DataBrowseWindow::adjustRec(int rec){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::adjustRec  %d\n",  rec);
		fflush(stdout);
	}
 
 
	browser->activeEditor->record = rec;
	recField->setText(tr("%1").arg(rec));
	browser->activeEditor->updateForRec();
  
}

void DataBrowseWindow::extract(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::extract \n");
		fflush(stdout);
	}
	
	// ask user for selection
	DataSliceWindow *SliceWindow = new DataSliceWindow(0);
	if(parentBrowser)
	{
		DvObject_var x_obj;
		
        x_obj = parentObject->get_xref(editObjectName.toLatin1().data());
		
		QString tempName = QString("Temp/")+editObjectName;
		QuSetObjByName(x_obj, tempName.toLatin1().data()); // add to Wlist but not GUI WL
		SliceWindow->setObject(tempName.toLatin1().data());
	}
	else
	{
			SliceWindow->setObject(editObjectName.toLatin1().data());
	}
	SliceWindow->show();

}



void DataBrowseWindow::dragEnterEvent(QDragEnterEvent *event)
{
	if(QuSlotDebug()){ printf("DataBrowseWindow::dragEnterEvent \n"); fflush(stdout); }
	this->activateWindow();
	this->raise();
}

void DataBrowseWindow::saveEdits(){

	if(QuSlotDebug())
	{
		printf("DataBrowseWindow::saveEdits \n");
		fflush(stdout);
	}
	
	if(name_field->text().isEmpty()) {
		msgBar->showMessage("Name field is empty");
		return;
	}
	
	DvObject_var new_obj = browser->getUpdate();
	QString newName = name_field->text();

	if( parentObject.is_nil() )
	{
		// editing object from working list
		if(originalName != name_field->text() || item == 0)
		{
			// save as new name
			
			if ( !newName.contains("Time Intervals/") && new_obj->is_event() ){
				newName=QString("Time Intervals/")+newName;
			}
			else if( !newName.contains("Data/") ){
				newName=QString("Data/")+newName;
			}
		
			QwuiPutOnWL(new_obj, newName);
		
			this->dismiss();

			//originalName = name_field->text();
			//editObjectName = newName;
		}
		else
		{
		// replace dobj
			QuReplaceObject(editObjectName.toLatin1().data(), new_obj);
			emit objChanged(editObjectName);
			qsasWindow->wlview->update();
			
			this->dismiss();

		}
		
		
	}
	else {
		// xref object
		parentObject->change_xref(name_field->text().toLatin1().data(), new_obj);
		if(parentBrowser)
		{
			parentBrowser->browser->refreshList();
			parentBrowser->emitObjChanged((const QString &) parentBrowser->editObjectName);
		}
		
		// close since Xref being edited
		this->dismiss();
		return;
	}
	
	
	return;
}



void DataBrowseWindow::clearErrors()
{
    if(QuSlotDebug())
    {
        printf("DataBrowseWindow::clearErrors \n");
        fflush(stdout);
    }
    
    DvObject_var origObj = getWLObject(editObjectName.toLatin1().data());
    if(origObj.is_ok()) origObj->clearErr();
    
    browserVBLayout->removeWidget(error_panel);
    delete error_panel;
    
    this->resize(this->minimumSizeHint());
    this->adjustSize();
}



void DataBrowseWindow::setObject(DvObject_var &dobj, QString WLcontext){


    editObjectName = WLcontext;
      
	browser = new QbObjViewer(this, dobj, browserVB, WLcontext);
	if( browser == 0 || browser->activeEditor == 0){
		return;
	}
    connect( parentsBtn, SIGNAL(clicked()), browser, SLOT(openParentFile()) );

  
  // ask qsas main window to ask qda to open a file (for Windows port)
	connect(browser, SIGNAL(launchQDA_Window(const char *)), qsasWindow, SIGNAL(openQDA_Window(const char *)));
    
	msgBar->showMessage(WLcontext);
	QuMsgBar("", false);
  
  
  // initialise slider to first record
	browser->activeEditor->record = 0;
	recField->setText("0");
  
	recBar->setRange(0, browser->activeEditor->maxRec);
    int step = browser->activeEditor->maxRec / 10;
	if (step < 1) step = 1;
	recBar->setPageStep(step);

    if(dobj->seqSize() > 1 && !dobj->is_event() && browser->activeEditor->maxRec > 0)  recSlider->setEnabled(true);
	else {
		// single object or event list with own bar
		recSlider->setEnabled(false);
		recSlider->setVisible(false);

        if( dobj->is_event() ){
			// time interval or evant table (have own widgets)

			name_panel->setVisible(false);
			saveEditsBtn->setVisible(false);
			pad->setVisible(true);
		}
	}

    // set Xrefs
	DvNode * node = dobj->first_xref();
    while(node)
    {
        new XrefListItem(browser->XrefList, QString(node->name().c_str()));
        node = node->next;
    }


}



// generic viewer widget class

QbObjViewer::QbObjViewer(DataBrowseWindow * pWindow, DvObject_var dobj, QWidget *parent, QString name)
	: QGroupBox(parent)
{

	size_t len = dobj->seqSize();
	
	DvObject_var nilObj;
	activeEditor = 0;
	parentBrowser = pWindow;
	
	QHBoxLayout * viewerLayout=new QHBoxLayout;
  
	// is this a time interval object?
	if( dobj->is_event() && len == 1){
		TI_window = new QbTimeIntervalEditor(dobj, this, name);
		TI_window->adjustSize();
		TI_window->show();
		activeEditor = TI_window;
		viewerLayout->addWidget(TI_window);
		

	}
  
	//  time interval sequence object?  
	else if(dobj->is_event() && len > 1 ){
		TIS_window = new QbEventTableEditor(dobj, this, name);
		TIS_window->adjustSize();
		TIS_window->show();
		activeEditor = TIS_window;
		viewerLayout->addWidget(TIS_window);
	}
  
	else{
		activeEditor = new QbEditor(dobj, this);
		activeEditor->show();
		viewerLayout->addWidget(activeEditor);
	}
  
	  
	if (activeEditor == 0) {
		QuMsgBar("object type not supported", true);
		return;
	}
	
	Attribute_box = new QWidget(this);
	QVBoxLayout * Attribute_box_layout=new QVBoxLayout;
	Attribute_box_layout->setSpacing(2);

	XrefList=new XrefListView(this);
	XrefList->setMinimumHeight(200);
	XrefList->setMinimumWidth(160);
	XrefList->viewer = this;
	connect(XrefList, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(xrefEdit(QListWidgetItem*)));
	connect(qsasWindow->wlview, SIGNAL(xrefDropDetected(const QString&)), this, SLOT(copyXrefToWL(const QString &)));
	connect(qsasWindow->wlview, SIGNAL(refreshBrowser()), this, SLOT(refreshList()));


	renameBtn = new QPushButton( " Rename Attribute ", Attribute_box);
	renameBtn->setMinimumSize(renameBtn->sizeHint());
	connect( renameBtn, SIGNAL(clicked()), this, SLOT(renameAttribute()) );

	QLabel* attributesLabel=new QLabel("Attributes", this);
	attributesLabel->setAlignment(Qt::AlignCenter);
	QFont f=attributesLabel->font();
	f.setWeight(QFont::Bold);
	attributesLabel->setFont(f);
	Attribute_box_layout->addWidget(attributesLabel, Qt::AlignCenter);
	Attribute_box_layout->addWidget(XrefList);
	Attribute_box_layout->addWidget(renameBtn);
	Attribute_box->setLayout(Attribute_box_layout);
    
	viewerLayout->addWidget(Attribute_box);
	
	if(dobj->is_event()) Attribute_box->hide();
    
	fileList = new QMenu(this);

	setLayout(viewerLayout);
	adjustSize();
}


void QbObjViewer::copyXrefToWL(const QString &attrName){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::copyXrefToWL '%s'\n", attrName.toLatin1().data());
		fflush(stdout);
	}
	if(activeEditor->edited_var.is_nil()) return;
	DvObject_var xref_obj;

    xref_obj = activeEditor->edited_var->get_xref(attrName.toLatin1().data());
	
	DvObject_var new_obj = new DvObject(xref_obj);
	
	// save to WL
	
	QString newName = attrName;
	if(newName.startsWith('_'))
	{
		// WL objects must not contain reserved chars [,],{,}
		newName=QString("Cache_Obj");
	}
	
	if ( new_obj->is_event()){
		newName=QString("Time Intervals/")+newName;
	}
	else if( new_obj->seqSize() <= 1 ){
		newName=QString("Constants/")+newName;
	}
	else{
		newName=QString("Data/")+newName;
	}
		
	QwuiPutOnWL(new_obj, newName);
	
}

void QbObjViewer::refreshList(){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::refreshList \n");
		fflush(stdout);
	}

	XrefList->clear();
	if( activeEditor == NULL || activeEditor->edited_var.is_nil() ) return;

   // reset Xrefs
	DvNode *xnode = activeEditor->edited_var->first_xref();

	while(xnode)
	{
		new XrefListItem(XrefList, QString(xnode->name().c_str()));
		xnode = xnode->next;
	}
      
}



DvObject_var QbObjViewer::getUpdate(){
  
	return activeEditor->edited_var;
  
}

void QbObjViewer::xrefEdit ( QListWidgetItem *item )
{

	XrefListItem* xitem=(XrefListItem*)(item);
	if(QuSlotDebug())
	{
		printf("QbObjViewer::xrefEdit \n");
		fflush(stdout);
	}
 

	DataBrowseWindow *QBROWSER_widget_xref=0;
    
	if(item == 0) return;
	
	DvObject_var xref_obj=activeEditor->edited_var->get_xref(xitem->text().toLatin1().data());
	if(xref_obj.is_nil())
	{
		QuMsgBar("Unknown attribute ", xitem->text().toLatin1().data(), true);
		return;
	}
  
	QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
					       parentBrowser, xitem->text().toLatin1().data(),
							       xref_obj,
									       QString(item->text().toLatin1().data()));

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
     
	if (QBROWSER_widget_xref == 0 ) return;
  
	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this,  SLOT(refreshList()) );
//	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), parentBrowser,  SIGNAL(objChanged(const QString&)) );
		    
}

void QbObjViewer::addTimeTags(){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::addTimeTags \n");
		fflush(stdout);
	}
   
	if(activeEditor == 0 || activeEditor->edited_var.is_nil() ) {
		QuMsgBar("Object does not exist", true);
		return;
	}

	if(activeEditor->edited_var->xref_exists(DEPEND_0) ){
		QuMsgBar("Time Tags already exist", true);
		return;
	}
  
	int n_recs = activeEditor->edited_var->seqSize();
  
  	DvTime t(2001, 9, 4);
	DvObject_var T_var = new DvObject(t, n_recs);
    
	activeEditor->dep0Box->show();

	
	activeEditor->timeField->setTime( T_var->time(0) );

	activeEditor->edited_var->change_xref(DEPEND_0, T_var);
	
	parentBrowser->saveEdits();

}

void QbObjViewer::renameAttribute(){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::renameAttribute \n");
		fflush(stdout);
	}

	XrefList->setCurrentIndex(XrefList->currentIndex());
	XrefList->edit(XrefList->currentIndex());

}

void QbObjViewer::openParentFile(){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::openParentFile \n");
		fflush(stdout);
	}
 
  
	DvObject_var parentsXref;
	if(activeEditor == 0) return;
  
	if( (activeEditor->edited_var).is_nil() ) {
		QuMsgBar("No variable edited", true);
		return;
	}
    
    parentsXref = activeEditor->edited_var->get_xref("Parents");
	
  
	if ( parentsXref.is_nil() || !parentsXref->is_str()) return;
  
	fileList->clear();
	int listSize = parentsXref->seqSize();
	for (int i=0; i<listSize; i++){
		fileList->addAction( QString(parentsXref->str(i).c_str()));
	}
	fileList->addSeparator();
	fileList->addAction( QString("Cancel") );
	
	QAction* act=fileList->exec(QCursor::pos());
	if(act==NULL) return;
	if(act->text()==QString("Cancel")) return;
	
	emit launchQDA_Window(act->text().toLatin1().data());
}


void QbObjViewer::attachXref()
{

	if(QuSlotDebug())
	{
		printf("QbObjViewer::attachXref \n");
		fflush(stdout);
	}
	
	QbuAttachWindow * attachWindow = new QbuAttachWindow(this, "attachXrefChooser");
	attachWindow->dobj = activeEditor->edited_var;
	attachWindow->thisViewer = this;
	attachWindow->show();
}


void QbObjViewer::deleteXref()
{

	if(QuSlotDebug())
	{
		printf("QbObjViewer::deleteXref \n");
		fflush(stdout);
	}
  
	QList<QListWidgetItem *> items=XrefList->selectedItems();
	if(items.isEmpty()) return;
	foreach(QListWidgetItem* item, items)
	{
		
        activeEditor->edited_var->delete_xref(item->text().toLatin1().data() );
		
	}
  
	refreshList();
 
}

void QbObjViewer::newXref()
{

	if(QuSlotDebug())
	{
		printf("QbObjViewer::newXref \n");
		fflush(stdout);
	}
	
	static QPushButton *Sbtn;
	static QPushButton *Mbtn;
	static QPushButton *Vbtn;
	static QPushButton *tbtn;
	static QPushButton *Tbtn;
	static QPushButton *Txtbtn;
	static QPushButton *TxtArrbtn;
	static QWidget * typeBox;
   
	objTypeSelector = new QDialog(0);
	typeBox = new QWidget(objTypeSelector);

	QVBoxLayout * typeBoxLayout=new QVBoxLayout;
	typeBoxLayout->setMargin(QSAS_profiles->current()->GUImargins);
	typeBoxLayout->setSpacing(QSAS_profiles->current()->GUImargins);

	Txtbtn = new QPushButton(" Text ", typeBox); typeBoxLayout->addWidget(Txtbtn);
	TxtArrbtn = new QPushButton(" Text Array", typeBox); typeBoxLayout->addWidget(TxtArrbtn);
	Tbtn = new QPushButton(" Time Object ", typeBox); typeBoxLayout->addWidget(Tbtn);
	tbtn = new QPushButton(" Time Interval  ", typeBox);     typeBoxLayout->addWidget(tbtn);
	Sbtn = new QPushButton(" Scalar Constant  ", typeBox); typeBoxLayout->addWidget(Sbtn);
	Mbtn = new QPushButton(" Matrix Constant  ", typeBox); typeBoxLayout->addWidget(Mbtn);
	Vbtn = new QPushButton(" Vector Constant  ", typeBox); typeBoxLayout->addWidget(Vbtn);
	connect(Txtbtn,SIGNAL(clicked()),this,SLOT(createText()));
	connect(TxtArrbtn,SIGNAL(clicked()),this,SLOT(createTextArr()));
	connect(Tbtn,SIGNAL(clicked()),this,SLOT(createTime()));
	connect(tbtn,SIGNAL(clicked()),this,SLOT(createTInterval()));
	connect(Sbtn,SIGNAL(clicked()),this,SLOT(createConstant()));
	connect(Mbtn,SIGNAL(clicked()),this,SLOT(createArray()));
	connect(Vbtn,SIGNAL(clicked()),this,SLOT(createVConstant()));
	typeBox->setLayout(typeBoxLayout);
	typeBox->adjustSize();
     
	objTypeSelector->show();
}


void QbObjViewer::createText ( ){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createText \n");
		fflush(stdout);
	}
 
 
	DataBrowseWindow *QBROWSER_widget_xref;
	DvString txt("");
	DvObject_var dobj = new DvObject(txt);
  
	objTypeSelector->close();

	QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
					       parentBrowser, "new", dobj, "new");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
	if(QBROWSER_widget_xref == 0 ) return;
  
	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );

}


void QbObjViewer::createTime(){ 

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createTime \n");
		fflush(stdout);
	}
 
 	DvTime t("2001-09-05 00:00:00.000");
	DvObject_var dobj = new DvObject(t);
	DataBrowseWindow *QBROWSER_widget_xref;
  
	objTypeSelector->close();
  
  	DvString txt("1>s");
	DvObject_var SI_ptr = new DvObject(txt);
	dobj->change_xref(SI_CONVERSION, SI_ptr);
	QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
					       parentBrowser, "newTime", dobj, "newTime");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
	if(QBROWSER_widget_xref == 0 ) return;
  
	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );
  
}


void QbObjViewer::createTInterval ( ){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createTInterval \n");
		fflush(stdout);
	}
 
 
	printf("type not yet supported\n");
	fflush(stdout);
	objTypeSelector->close();

}


void QbObjViewer::createVConstant ( ){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createVConstant \n");
		fflush(stdout);
	}
 
 
	vector <size_t> mDim(1);
	mDim[0] = 3;
  
	DvObject_var dobj = new DvObject(0.0, mDim);
	DvString txt("vector>gse_xyz");
	dobj->change_xref(FRAME, txt);
  
	DataBrowseWindow *QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
			parentBrowser, "newVector", dobj, "newVector");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
	if(QBROWSER_widget_xref == 0 ) return;
  
	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );
 
	objTypeSelector->close();

}

void QbObjViewer::createArray(){ 

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createArray \n");
		fflush(stdout);
	}
   
	objTypeSelector->close();
  
	arrTypeSelector = new arrDialog(this, "arrayDimChooser", this, true);


	arrTypeSelector->arrType = QB_DOUBLE;
	arrTypeSelector->show();
}



const char * QbObjViewer::objName(){ 
   
	return parentBrowser->name_field->text().toLatin1().data();
  
}


void QbObjViewer::createTextArr(){ 

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createTextArr \n");
		fflush(stdout);
	}
   
	objTypeSelector->close();
  
	arrTypeSelector = new arrDialog(this, "arrayDimChooser", this, true);


	arrTypeSelector->arrType = QB_STRING;
	arrTypeSelector->show();
}


void QbObjViewer::createConstant ( ){

	if(QuSlotDebug())
	{
		printf("QbObjViewer::createConstant \n");
		fflush(stdout);
	}
 
 
	DataBrowseWindow *QBROWSER_widget_xref;
	DvObject_var dobj = new DvObject(0.0);

	objTypeSelector->close();

	QBROWSER_widget_xref = QbuLaunchBrowse(activeEditor->edited_var,
					       parentBrowser, "new",dobj, "new");

  // browser is not modal, so code falls through to here immediately
  // and connect is available before "save" is pressed
  
	if (QBROWSER_widget_xref == 0) return;
  
	connect(QBROWSER_widget_xref, SIGNAL(objChanged(const QString&)), this, SLOT(refreshList()) );

}


DataBrowseWindow * QbuLaunchBrowse(DvObject_var &source,  QWidget *parent, const char * objectName, DvObject_var &dobj, QString WLcontext)
{
	DataBrowseWindow *QBROWSER_widget;
      
	QBROWSER_widget = new DataBrowseWindow(dobj, WLcontext, parent, "editor");

	QBROWSER_widget->originalName = objectName;
	QBROWSER_widget->name_field->setText(objectName);
	if( QBROWSER_widget->browser == 0 || QBROWSER_widget->browser->activeEditor == 0) {
		QBROWSER_widget->close();
		return 0;
	}

	 QBROWSER_widget->parentObject = source;


	if( source.is_nil() ){
		QBROWSER_widget->setWindowTitle("Object Editor");
		QBROWSER_widget->parentsBtn->setEnabled(true);
		QBROWSER_widget->move(30,30);
		QBROWSER_widget->parentBrowser = 0;
	}
	else{
     // source means launched from a xref on another edit window
		QBROWSER_widget->setWindowTitle("Metadata Editor");
		QBROWSER_widget->parentsBtn->setEnabled(false);
		QBROWSER_widget->move(30,250);
		QBROWSER_widget->parentBrowser = dynamic_cast <DataBrowseWindow*> (parent);
	}
  

	QBROWSER_widget->show();
 
	return QBROWSER_widget;
 
}

QbuAttachWindow::QbuAttachWindow(QbObjViewer *parent, const char *name)
	:  QDialog(parent)
{

	QVBoxLayout * centreBox;
	QGroupBox * typeBox;
	QWidget * btnBox;
	QLabel *slotLabel;
	QLabel *slotSpace;
	QLabel *ObjSlotLabel;
	QLabel *infoSlot;
	QPushButton *cancelBtn;
	QPushButton *acceptBtn;
  
	centreBox = new QVBoxLayout(this);
	typeBox = new QGroupBox(" Choose Name and Object " , this);
	QVBoxLayout * typeBoxLayout=new QVBoxLayout;
	typeBox->setMinimumWidth(250);
	typeBoxLayout->setMargin(4);
       
  	ObjSlotLabel = new QLabel(typeBox);
	ObjSlotLabel->setText(" Drop object to attach ");
	typeBoxLayout->addWidget(ObjSlotLabel);
  
	object_field  = new QguWLinput(typeBox, "object_field");
	object_field->setAcceptable(Qcondition(QS_WL_OBJ));
	object_field->setMinimumWidth(150);
	typeBoxLayout->addWidget(object_field);
  
	slotSpace = new QLabel(typeBox);
	slotSpace->setText("  ");
	typeBoxLayout->addWidget(slotSpace);
  
	slotLabel = new QLabel(typeBox);
	slotLabel->setText(" Metadata Name ");
	typeBoxLayout->addWidget(slotLabel);
  
	name_field  = new QguWLinput(typeBox, "nameField");
	name_field->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	name_field->setText("new");
	name_field->setMinimumWidth(150);
	typeBoxLayout->addWidget(name_field);
  
	infoSlot = new QLabel(typeBox);
	infoSlot->setText("( Drag straight to metadata List attaches with same name as on WL)");
	typeBoxLayout->addWidget(infoSlot);
  
	connect(object_field,SIGNAL(edited_OK()),this, SLOT(updateName()));

	btnBox = new QWidget(this);
	QHBoxLayout * btnLayout=new QHBoxLayout(btnBox);
	btnLayout->setSpacing(QSAS_profiles->current()->GUImargins);

	btnLayout->setContentsMargins(0,0,0,0);

	cancelBtn = new QPushButton("Cancel", btnBox);
	acceptBtn = new QPushButton("Create...", btnBox);

	btnBox->layout()->addWidget(cancelBtn);
	btnBox->layout()->addWidget(acceptBtn);

	connect(cancelBtn,SIGNAL(clicked()),this,SLOT(close()));
	connect(acceptBtn,SIGNAL(clicked()),this,SLOT(attachObject()));
	centreBox->addWidget(typeBox);
	centreBox->addWidget(btnBox);
	
	typeBox->setLayout(typeBoxLayout);
	typeBox->adjustSize();
  
}
  
void QbuAttachWindow::updateName(){
	  
	QString newText = QString(object_field->textObject().toLatin1().data());
	int ptr = newText.lastIndexOf( "/");
	if (ptr >= 0) newText.remove(0, ptr+1);
     
	// disallow drop of itself into list
	if (thisViewer->objName() == newText ) {
		QuMsgBar(" drop of object as its own xref", true);
		object_field->setText("self-link");
		object_field->reValidate();
	} // stop drop of obj on its own xref list
	else{
		name_field->setText(newText);
		name_field->reValidate();
		QuMsgBar(" ", false);
	}
}
  
void  QbuAttachWindow::attachObject(){

	DvObject_var xref = object_field->getObject();
	dobj->change_xref(name_field->text().toLatin1().data(), xref );
	
	thisViewer->refreshList();
	this->close();

}

  arrDialog::arrDialog(QWidget *parent, const char *name, QbObjViewer* browser, bool modal)
	:  QDialog(parent)
{
	this->setModal(modal);
	parentViewer = browser;
	dimSlot = 0;
	QVBoxLayout * centreBox;
	QGroupBox * dimBox;

	folderPath=QString("Constants");
	
	QWidget * btnBox;

	QLabel *slotLabel;
	QPushButton *cancelBtn;
	QPushButton *acceptBtn;

	centreBox = new QVBoxLayout(this);
	dimBox = new QGroupBox(" Choose Array Dimensions ", this);
	QVBoxLayout * dimBoxLayout=new QVBoxLayout;
	dimBox->setMinimumWidth(250);
	dimBoxLayout->setMargin(QSAS_profiles->current()->GUImargins);
	
	dimSlot = new QguWLinput(dimBox, "dimSlot");
	dimSlot->setAcceptable(Qcondition(QS_NARROW, QS_TEXT, QS_0D, QS_NOT_SEQ, QS_ANY));
	dimSlot->addAcceptable(Qcondition(QS_NARROW, QS_DBL, QS_ARRAY, QS_ANY, QS_ANY));
	dimBoxLayout->addWidget(dimSlot);

	slotLabel = new QLabel(dimBox);
	slotLabel->setText("drop object with dimensions \n    or \ntype in array sizes, e.g. 3, 16, 36  or  3, 3");
	dimBoxLayout->addWidget(slotLabel);
	
	btnBox = new QWidget(this);
	QHBoxLayout * btnLayout=new QHBoxLayout(btnBox);

	btnLayout->setContentsMargins(0,0,0,0);
	cancelBtn = new QPushButton("Cancel", btnBox);
	acceptBtn = new QPushButton("Create...", btnBox);
	btnLayout->addWidget(cancelBtn);
	btnLayout->addWidget(acceptBtn);
	acceptBtn->setDefault(true);

	connect(cancelBtn,SIGNAL(clicked()), this, SLOT(close()));
	connect(acceptBtn,SIGNAL(clicked()), this, SLOT(editArray()));
	
	centreBox->addWidget(dimBox);
	centreBox->addWidget(btnBox);
	dimBox->setLayout(dimBoxLayout);
	dimBox->adjustSize();
}



// ---------------------------------------------------

void arrDialog::editArray(){ 

	if(QuSlotDebug())
	{
		printf("arrDialog::editArray \n");
		fflush(stdout);
	}
 
	DvObject_var target = dimSlot->getObject();
	if( target.is_nil() ){
		QuMsgBar("Bad Dimension Input", true);
		return;
	}
  
	vector <size_t> mDim = target->Dims();
	

	if( target->is_str() && target->arraySize() == 1) {
		DvString dimStr = target->str(0) ; // take copy
		int rankM = 0;
		mDim.clear(); // reset from string
		
		while( !dimStr.empty() ){
			DvString token = dimStr.front(",");
			dimStr = dimStr.after(',');
			int sizeN = token.toInt();
			mDim.push_back((size_t)sizeN);
			rankM++;
		}
	
		if(rankM == 0){
			QuMsgBar("Bad Dimension Input string", true);
			return;
		}
	}

	DataBrowseWindow *QBROWSER_widget=0;
	DataBrowseWindow *parentBrowser=0;
	if (arrType == QB_DOUBLE){
  
		DvObject_var dobj = new DvObject(0.0, mDim);

		DvObject_var parentObj;
		if( parentViewer != 0 )
		{
			parentObj = parentViewer->activeEditor->edited_var;
			parentBrowser = parentViewer->parentBrowser;
		}
     	DvString txt("1>(?)");
		DvObject_var SI_ptr = new DvObject(txt);
		dobj->change_xref(SI_CONVERSION, SI_ptr);
  
		QBROWSER_widget = QbuLaunchBrowse(parentObj, parentBrowser, "", dobj, "");
		QBROWSER_widget->name_field->setText("newArray"); // set here so that it doesn't think object exists on WL
	}
	else if (arrType == QB_STRING){
  		DvString nilStr("");
		DvObject_var dobj = new DvObject(nilStr, mDim);

	//	QString context("Constants/newTextArray");

		DvObject_var parentObj;
		if( parentViewer != 0 ) 
		{
			parentObj = parentViewer->activeEditor->edited_var;
			parentBrowser = parentViewer->parentBrowser;			
		}
  
		QBROWSER_widget = QbuLaunchBrowse(parentObj, parentBrowser, "", dobj, "");
		QBROWSER_widget->name_field->setText("newTextArray"); // set here so that it doesn't think object exists on WL
	}
	
    if (QBROWSER_widget != 0) 
	{
		if(parentViewer != 0){
		
			connect(QBROWSER_widget, SIGNAL(objChanged(const QString&)), parentViewer,  SLOT(refreshList()) );
  
		}

        this->hide();
		QBROWSER_widget->activateWindow();
		this->setParent(QBROWSER_widget); //so deleting browser deletes this, (close() would set main window as active)
	}
	else this->close();

}

