#include "WLItem.h"
#include <QStyle>


WLItem::WLItem(const QString& name, QTreeWidgetItem* parent, bool isFolder):
	QTreeWidgetItem(parent), WL_name(name), _isFolder(isFolder)
{
	if(parent!=NULL)
	{
		parent->addChild(this);
		if(isFolder){
			QFont _font = font(0);
			_font.setBold(true);
			setFont(0, _font);
			setIcon(0, qApp->style()->standardIcon(QStyle::SP_DirOpenIcon));
		}

		WL_path = ((WLItem*)parent)->getPath();
		WL_path.append("/");
		WL_path.append(name);

	}
	else
	{
		// Data or Time or Constants list folder
		setTextAlignment(0, Qt::AlignCenter);
		QFont _font = font(0);
		_font.setBold(true);
		setFont(0, _font);
		
		if(name == "Time Intervals") {
			QBrush brush(QColor(240, 240, 240), Qt::SolidPattern);
			setBackground(0, brush);
		}
		else {
			QBrush brush(QColor(230, 230, 230), Qt::SolidPattern);
			setBackground(0, brush);
		}
		
		WL_path=WL_name;
		
	}
	
	this->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled | Qt::ItemIsEditable);
	setText(0, name);
	
	_startTime = 1.e30;
	_objType = -1;
}

WLItem::~WLItem()
{
	// recursively delete children first
	while(childCount()>0){
		delete child(0);
	}
	
	QuDeleteObject(WL_path.toLatin1().data());

}

void WLItem::select(bool selected)
{	
	this->setSelected(selected);
	for(int i=0; i<childCount(); i++) ((WLItem*)child(i))->select(selected);
}


void WLItem::updateName()
{	
	QString newName = this->text(0);
	if(newName.isEmpty()) return;

	QString oldPath = WL_path;

	WL_name = newName;
	if(parent()){
		WL_path = ((WLItem*)parent())->getPath();
		WL_path += "/";
		WL_path += newName;
	}
	else WL_path = newName;
	QuRenameObject(oldPath.toLatin1().data(), WL_path.toLatin1().data());
    // propagate to children
	for(int i=0; i<childCount(); i++) ((WLItem*)child(i))->updatePath(WL_path);
    
}

void WLItem::updatePath(QString newPath)
{	
    QString oldPath = WL_path;
    oldPath += "/";
    oldPath += WL_name;
    
    WL_path = newPath;
    
    newPath += "/";
    newPath += WL_name;
    
    // tell DvList
    QuRenameObject(oldPath.toLatin1().data(), newPath.toLatin1().data() );
}

double WLItem::getTime()
{
	if(_isFolder){
		double tStart = 1.e30;
		int numChildren = childCount();
		if(numChildren == 0) return tStart;
		
		tStart = ((WLItem*)child(0))->getTime();

		for(int i=1; i<numChildren; i++){

			double tNext = ((WLItem*)child(i))->getTime();

			if(tNext < tStart) tStart = tNext;
		} 
		_startTime = tStart;
	}			
	else if(_startTime > 1.e29){
		DvObject_var dobj = getWLObject(WL_path.toLatin1().data());
		if(dobj.is_ok()){
			DvEvent ti = dobj->getTimeRange();
			if(!ti.empty()) _startTime = ti.start().getEpoch2000();
		}
	}
	
	return _startTime;
}

double WLItem::getType()
{
	if(_isFolder) _objType = 0;	
	else if(_objType < 0){
		DvObject_var dobj = getWLObject(WL_path.toLatin1().data());
		if(dobj.is_ok()){
			if(dobj->is_dbl()){
				if( dobj->seqSize() > 1 && dobj->arraySize() == 1) {
					_objType = 1;
				}
				if( dobj->seqSize() > 1 && dobj->arraySize() > 1) {
					if(dobj->isThreeVector()) _objType = 2;
					else _objType = 3;
				}
				if( dobj->seqSize() == 1 && dobj->arraySize() == 1) {
					_objType = 4;
				}
				if( dobj->seqSize() == 1 && dobj->arraySize() > 1) {
					if(dobj->isThreeVector()) _objType = 5;
					else _objType = 6;
				}
				return _objType;
			}
			if(dobj->is_int()) {
				if( dobj->seqSize() > 1 && dobj->arraySize() == 1) {
					_objType = 7;
				}
				if( dobj->seqSize() > 1 && dobj->arraySize() > 1) {
					_objType = 8;
				}
				if( dobj->seqSize() == 1 && dobj->arraySize() == 1) {
					_objType = 9;
				}
				if( dobj->seqSize() == 1 && dobj->arraySize() > 1) {
					_objType = 10;
				}
				return _objType;
			}
			if(dobj->is_str()) {
				if( dobj->seqSize() > 1 && dobj->arraySize() == 1) {
					_objType = 11;
				}
				if( dobj->seqSize() > 1 && dobj->arraySize() > 1) {
					_objType = 12;
				}
				if( dobj->seqSize() == 1 && dobj->arraySize() == 1) {
					_objType = 13;
				}
				if( dobj->seqSize() == 1 && dobj->arraySize() > 1) {
					_objType = 14;
				}
				return _objType;
			}
			if(dobj->is_time()) {
				if( dobj->seqSize() > 1 && dobj->arraySize() == 1) {
					_objType = 15;
				}
				if( dobj->seqSize() > 1 && dobj->arraySize() > 1) {
					_objType = 16;
				}
				if( dobj->seqSize() == 1 && dobj->arraySize() == 1) {
					_objType = 17;
				}
				if( dobj->seqSize() == 1 && dobj->arraySize() > 1) {
					_objType = 18;
				}
				return _objType;
			}
			if(dobj->is_event()) {
				if( dobj->seqSize() > 1 && dobj->arraySize() == 1) {
					_objType = 19;
				}
				if( dobj->seqSize() > 1 && dobj->arraySize() > 1) {
					_objType = 20;
				}
				if( dobj->seqSize() == 1 && dobj->arraySize() == 1) {
					_objType = 21;
				}
				if( dobj->seqSize() == 1 && dobj->arraySize() > 1) {
					_objType = 22;
				}
				return _objType;
			}
			else{
				_objType = 23;
				return _objType;
			}
		}
	}
	
	return _objType;
}


bool WLItem::setFolder(bool isFolder)
{
	_isFolder=isFolder;
	
	return true;
}


bool WLItem::isDescendentOf(WLItem* other)
{
	if(this==other) return true;
	
	if(parent()==NULL) return false;
	
	return ((WLItem*)parent())->isDescendentOf(other);
}

// ---------------- 

bool WLItem::ancestorInList(QList <QTreeWidgetItem*> &list)
{
	for(int i=0; i<list.size(); i++)
	{
		if(this != (WLItem*)list[i] && this->isDescendentOf((WLItem*)list[i]) ) return true;	
	}
	return false;
}

// ---------------- 

int WLItem::countName(const QString& name)
{
	int count = 0;
	WLItem *thisParent = (WLItem *)parent();
	
	if(thisParent == NULL ) return count;
	for(int i=0; i<thisParent->childCount(); i++)
	{
		if( ((WLItem*)thisParent->child(i))->text(0)==name) {
			count ++;
		}
	}
	return count;
}

// ---------------- 

bool WLItem::nameExists(const QString& name)
{
	if(!_isFolder || childCount() == 0) return false;
	for(int i=0; i<childCount(); i++)
	{
		if( ((WLItem*)child(i))->WL_name==name) {
			QuMsgBar("Name Exists, appending 'A': ", name.toLatin1().data(), false);

			return true;
		}
	}
	return false;
}

// ---------------- 

WLItem* WLItem::getChildByName(const QString& name)
{
	if(!_isFolder || childCount() == 0) return NULL;
	for(int i=0; i<childCount(); i++)
	{
		if( ((WLItem*)child(i))->WL_name==name) return ((WLItem*)child(i));
	}
	return NULL;
}

// ---------------- 

WLItem* WLItem::getChildByPath(const QString& path)
{
	if(path.contains("/"))
	{
		QStringList splitList=path.split("/", QString::SkipEmptyParts);
		
		WLItem* child=getChildByName(splitList.at(0));

		if(child)
		{
			splitList.removeFirst();
			return child->getChildByPath(splitList.join("/"));
		}
	}
	else
	{
		return getChildByName(path);
	}
	
	return NULL;
}

// ---------------- 

bool WLItem::isAncestorOf(WLItem* other)
{
	if(this==other) return true;
	
	for(int i=0; i<childCount(); i++)
	{
		if( ((WLItem*)child(i))->isAncestorOf(other)) return true;
	}
	return false;
}


void WLItem::setChildrenSelected(){

	
	int numChildren = childCount();
	for(int i=0; i<numChildren; i++) {
		((WLItem*)child(i))->setSelected(true);
		((WLItem*)child(i))->setChildrenSelected();
	}
}

// ---------------- 

int WLItem::childNumber()
{
	if(parent() != NULL) return parent()->indexOfChild(this);
	else  return treeWidget()->indexOfTopLevelItem(this);
}

// ---------------- 

void WLItem::insertAbove(WLItem* item)
{
	QTreeWidgetItem *_parent = this->parent();
	
	if(_parent){
		//this is not top level
		while (((WLItem*)_parent)->nameExists(item->text(0)) ) item->setText(0, item->text(0)+"A");

		_parent->insertChild(this->childNumber(), item);
	}
	else{
		// top level
		treeWidget()->insertTopLevelItem(this->childNumber(), item);
	}
	
	treeWidget()->setCurrentItem(item);
	item->updateName();
	
	return;
	
}
// ---------------- 

void WLItem::insertBelow(WLItem* item)
{
	QTreeWidgetItem *_parent = this->parent();
	
	if(_parent){
		//this is not top level
		while(((WLItem*)_parent)->nameExists(item->text(0)) ) item->setText(0, item->text(0)+"A");
		_parent->insertChild(this->childNumber()+1, item);
	}
	else{
		// top level
		treeWidget()->insertTopLevelItem(this->childNumber()+1, item);
	}
	
	treeWidget()->setCurrentItem(item);
	item->updateName();
	
	return;
	
}

// ---------------- 

void WLItem::expandChildren()
{
	treeWidget()->expandItem(this);
	for(int i=0; i<childCount(); i++){
		((WLItem*)this->child(i))->expandChildren();
	}
}
