// qpui_window.cc
//
// Plot Interface Implementation - Plot Window and ListView components
//
// Janet Barnes, July 2002
//
// Tony Allen, 2014

#include "qpui.h"
#include "qwui.h"
#include "qdutil.h"

extern qwuiWindow* qsasWindow;
extern int qsas_restore_release;

extern "C" int PLPexitHandler(const char *msg){
	cout << "Plplot crash " << msg << endl;
	return 1;
}

  static const char * spectrum_xpm[] = {
   "24 32 6 1",
   " 	c None",
   "D	c #0178FE",
   "B	c #17DEDE",
   ".	c #40FE35",
   "Y	c #E9F500",
   "R	c #FF5218",
   "                        ",
   "                        ",
   "  DDDDDDBBBDDDDDBBBDDD  ",
   "  DDBBDDBBBDDBBDDDDBBB  ",
   "  BBBBBDDDBBBBBDDDBBBB  ",
   "  ...BBBBB...BBBBBB.BB  ",
   "  BBBBB...BBBBB....BBB  ",
   "  ...BBB.....BB....BBB  ",
   "  ....YY.....YYY....BB  ",
   "  ....YY.....YYY......  ",
   "  YYY.....YYYY.....Y..  ",
   "  YYYY...YYYYY.....Y..  ",
   "  YYYY...YY..YY..YYYYY  ",
   "  YYYY..YYY..YR..YYYYY  ",
   "  YYYYYYYYYYRRRYYYYYYY  ",
   "  RRYYYYYYYYRRRYYYYYYY  ",
   "  RRRRYYYRRYYRRYYYYRRR  ",
   "  RRRRRYYYYRYYYRRRYYRR  ",
   "  RRRRRYYYYRYYYRRRYYRR  ",
   "  RRYYRRRRRRRRRYRRRRRR  ",
   "  RRYRRRYYRRRRRYYRRRRR  ",
   "  RRYYYYYYYYRRRYYYYYYY  ",
   "  ....YY.....YYY....BB  ",
   "  ...BBBBB...BBBBBB.BB  ",
   "  ...BBBBB...BBBBBB.BB  ",
   "  DDDBBBBBBDDDDDBBBDDD  ",
   "  DDDDDBBBBDDDDDBBBBDD  ",
   "  DDDDDDBBBDDDDDBBBDDD  ",
   "  DDDDBBBDDDDDDDBBBDDD  ",
   "  DDDDDDDDDDDDDDDDDDDD  ",
   "  DDDDDDDDDDDDDDDDDDDD  ",
   "  DDDDDDDDDDDDDDDDDDDD  "};
 
 static const char * event_xpm[] = {
  "24 30 4 1",
  " 	c None",
  "X	c #A0A0A0",
  ".	c #FFFFFF",
  "#	c #E0E0E0",
  "                        ",
  "                        ",
  "   #XXXXXXXXXXXXXXXX##  ",
  "   XXXXXXXXXXXXXXXXXX#  ",
  "   XX..............XX#  ",
  "   XX..............XX#  ",
  "   XX...XXXXXXX....XX#  ",
  "   XX...XXXXXXX....XX#  ",
  "   XX...XX.........XX#  ",
  "   XX...XX.........XX#  ",
  "   XX...XXXXX......XX#  ",
  "   XX...XXXXX......XX#  ",
  "   XX...XX.........XX#  ",
  "   XX...XX.........XX#  ",
  "   XX...XXXXXXXX...XX#  ",
  "   XX...XXXXXXXX...XX#  ",
  "   XX..............XX#  ",
  "   XX..............XX#  ",
  "   #XXXXXXXXXXXXXXXX##  ",
  "   #######XX##########  ",
  "          XX##          ",
  "          XX##          ",
  "          XX##          ",
  "          XX##          ",
  "          XX##          ",
  "          XX##          ",
  "          XX##          ",
  "          XX##          ",
  "          XX##          ",
  "            ##          "};
       
static const char * trace_xpm[] = {
       "22 18 4 1",
       " 	c None",
       "X	c #FF7070",
       ".	c #FFFFFF",
       "#	c #DF5050",
       "                      ",
       "  ................... ",
       "  ................... ",
       "  ................... ",
       "  ................... ",
       "  ................... ",
       "  .....XX............ ",
       "  ...X....X.......... ",
       "  ..X......X......... ",
       "  .X........X........ ",
       "  X..........X....... ",
       "  ............X...... ",
       "  .............X....X ",
       "  ...............XX.. ",
       "  ................... ",
       "  ................... ",
       "  ................... ",
       "  ................... ",
       "                      "};

//     QpuiWindow CLASS
//     ----------------

QpWindow::QpWindow( QWidget *parent ) : QMainWindow(0)
{
	// Constructor
	if(QuSlotDebug())
	{
		printf("QpWindow::QpWindow \n");
		fflush(stdout);
	}

    QIcon spectIcon = QIcon(QPixmap(spectrum_xpm));
    QIcon eventIcon = QIcon(QPixmap(event_xpm));
    QIcon traceIcon = QIcon(QPixmap(trace_xpm));
	
	// set up colour group palettes
	extern QPalette QPUI_normal_palette;
	QPUI_normal_palette = QPalette(qApp->palette());
	this->setPalette(QPUI_normal_palette);

    // Initialise colour tables
    stdPalette.clear();
    profilePalette.clear();
    importedPalette.clear();

    ColourTable rainbowTable("Rainbow"); rainbowTable.setPresetRainbow(); stdPalette.append(rainbowTable);
    ColourTable rainbowExtTable("Rainbow Extended"); rainbowExtTable.setPresetRainbowExt(); stdPalette.append(rainbowExtTable);
    ColourTable gwlTable("Gray White Low"); gwlTable.setPresetGrayWhiteLow(); stdPalette.append(gwlTable);
    ColourTable gwhTable("Gray White High"); gwhTable.setPresetGrayWhiteHigh(); stdPalette.append(gwhTable);
    ColourTable brdTable("Blue to Red (dark ends)"); brdTable.setPresetRedBlue(); stdPalette.append(brdTable);
    ColourTable brlTable("Blue to Red (light ends)"); brlTable.setPresetInvRedBlue(); stdPalette.append(brlTable);
    ColourTable earthTable("Earth"); earthTable.setPresetEarth(); stdPalette.push_back(earthTable);
    ColourTable spectTable("Spectrum"); spectTable.setPresetSpectrum(); stdPalette.push_back(spectTable);
    ColourTable spectTableDark("Spectrum Dark"); spectTableDark.setPresetSpectrumDark(); stdPalette.push_back(spectTableDark);

    foreach(ColourTable table, QSAS_profiles->current()->palette)
    {
        profilePalette.push_back(table);
    }

	setAcceptDrops(true);
	setFocusPolicy(Qt::WheelFocus);

	_top_widget = new QWidget(this);
	setCentralWidget( _top_widget );

	_top_gl = new QGridLayout( _top_widget);
	_top_gl->setSpacing(QSAS_profiles->current()->GUImargins);
	_top_gl->setMargin(QSAS_profiles->current()->GUImargins);
	
	_top_splitter=new QSplitter(_top_widget);
	_top_gl->addWidget( _top_splitter, 0,0 );

	QWidget * listsWidget=new QWidget(_top_splitter);
	QVBoxLayout* listsLayout=new QVBoxLayout;
	

	QScrollArea *scroll=new QScrollArea(_top_splitter);

	scroll->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
	scroll->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);


	QGroupBox* plot_groupbox = new QGroupBox(scroll->viewport());
	QVBoxLayout * plot_layout=new QVBoxLayout;

	plot_groupbox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));
	
	plotList=new PlotList(listsWidget);
	plotList->setColumnCount(2);
	plotList->tabWidget=new QTabWidget(plot_groupbox);
	plotList->setMinimumWidth(350);
	plotList->expandAll();
	plotList->setColumnWidth(0,300);
	plotList->setColumnWidth(1,24);
	QStringList colHead;
	QString head0 = "Data";
	QString head1 = "Link";
	colHead << head0;
	colHead << head1;
	plotList->setHeaderLabels(colHead);
	listsLayout->addWidget(plotList);
	
	listsWidget->setLayout(listsLayout);

		
	plot_layout->addWidget(plotList->tabWidget);
	plot_groupbox->setLayout(plot_layout);
	int margins = QSAS_profiles->current()->GUImargins;
	plotList->tabWidget->setContentsMargins(margins, margins, margins, margins);

	// set up menu bar
	_menu_bar = menuBar();
	_menu_bar->clear();
	QMenu* file_menu = 	_menu_bar->addMenu("&File");
	QMenu* insert_menu = _menu_bar->addMenu("Insert" );
	QMenu* edit_menu = 	_menu_bar->addMenu("Edit" );
	_menu_bar->addSeparator();
	QMenu* help_menu = 	_menu_bar->addMenu("&Help" );


	file_menu->clear();
	file_menu->addAction(QIcon(qsasWindow->qsas_home+"/icons/filesaveas.png"), "&Save", this, SLOT(save()));
	file_menu->addAction("Save &As...", this, SLOT(saveAsXML()));
	file_menu->addSeparator();
	file_menu->addAction("&Include...", this, SLOT(include()));
	file_menu->addAction(QIcon(qsasWindow->qsas_home+"/icons/reload.png"), "&Restore...", this, SLOT(restoreXML()));
	file_menu->addSeparator();
  	file_menu->addAction("Plot Selected Pages", plotList, SLOT(plotSelectedItems()), Qt::CTRL+Qt::Key_P);
	file_menu->addSeparator();
 	file_menu->addAction("Use Defaults in Selected Items", plotList, SLOT(setDefaultSelectedItems()));
	file_menu->addSeparator();
	file_menu->addAction("&Close", this, SLOT(close()));
	file_menu->addSeparator();
	toolBar = file_menu->addAction("Show Tool Bar", this, SLOT( toggleToolbar() ), Qt::CTRL+Qt::Key_T );

	insert_menu->clear();
	insert_menu->addAction(traceIcon, "Trace", plotList, SLOT(makeTraceItem()));
 	insert_menu->addAction(spectIcon, "SpectroGram", plotList, SLOT(makeSpectrumItem()));
	insert_menu->addSeparator();
 	insert_menu->addAction(QIcon(qsasWindow->qsas_home+"/icons/newtssubplot2.png"), "Panel", plotList, SLOT(makePanelItem()));
 	insert_menu->addAction("Time Axis Data Label", plotList, SLOT(makeTLabItem()));
	insert_menu->addSeparator();
 	insert_menu->addAction(QIcon(qsasWindow->qsas_home+"/icons/newtsplot2.png"), "TS Frame", plotList, SLOT(makeTSFrameItem()));
 	insert_menu->addAction(QIcon(qsasWindow->qsas_home+"/icons/newxyplot2.png"), "XY Frame", plotList, SLOT(makeXYFrameItem()));
	insert_menu->addSeparator();
 	insert_menu->addAction(QIcon(qsasWindow->qsas_home+"/icons/filenew.png"), "Page", plotList, SLOT(makePageItem()));
	insert_menu->addSeparator();
 	insert_menu->addAction("Constant X or T Trace", plotList, SLOT(makeConstXTraceItem()));
 	insert_menu->addAction("Constant Y Trace", plotList, SLOT(makeConstYTraceItem())); 
 	insert_menu->addAction("Regression Line", plotList, SLOT(makeRegLineTraceItem())); 
	insert_menu->addSeparator();
 	insert_menu->addAction(eventIcon, "Event Label", plotList, SLOT(makeEventItem()));
	insert_menu->addSeparator();
	QMenu* template_menu = insert_menu->addMenu("Template..."); 

	template_menu->clear();
 	template_menu->addAction("Vector RXYZ", plotList, SLOT(makeVector_RXYZ_Item())); 
 	template_menu->addAction("Vector XYZ", plotList, SLOT(makeVector_XYZ_Item()));
	template_menu->addSeparator();
 	template_menu->addAction("Vector R_XYZ", plotList, SLOT(makeVector_R_XYZ_Item()));
 	template_menu->addAction("Vector R_LP", plotList, SLOT(makeVector_R_LP_Item()));
 	template_menu->addAction("Vector R_TP", plotList, SLOT(makeVector_R_TP_Item()));
	template_menu->addSeparator();
 	template_menu->addAction("Vector R_X_Y_Z", plotList, SLOT(makeVector_R_X_Y_Z_Item()));
 	template_menu->addAction("Vector X_Y_Z", plotList, SLOT(makeVector_X_Y_Z_Item()));
	template_menu->addAction("Vector R_L_P", plotList, SLOT(makeVector_R_L_P_Item()));
 	template_menu->addAction("Vector R_T_P", plotList, SLOT(makeVector_R_T_P_Item()));
	template_menu->addSeparator();
 	template_menu->addAction("2 Panels", plotList, SLOT(make2Panel_Item()));
 	template_menu->addAction("3 Panels", plotList, SLOT(make3Panel_Item()));
 	template_menu->addAction("4 Panels", plotList, SLOT(make4Panel_Item()));
	template_menu->addSeparator();
 	template_menu->addAction("C1_2_3_4", plotList, SLOT(makeC1_2_3_4_Item()));
 	template_menu->addAction("C1234", plotList, SLOT(makeC1234_Item())); 


	edit_menu->clear();
	edit_menu->addAction("Rename", plotList, SLOT(renameSelectedItem()));
 	edit_menu->addAction(QIcon(qsasWindow->qsas_home+"/icons/editcut.png"), "Cut", plotList, SLOT(cut()), Qt::CTRL+Qt::Key_X);
 	edit_menu->addAction(QIcon(qsasWindow->qsas_home+"/icons/editcopy.png"), "&Copy", plotList, SLOT(copy()), Qt::CTRL+Qt::Key_C);
 	edit_menu->addAction(QIcon(qsasWindow->qsas_home+"/icons/editpaste.png"), "Paste", plotList, SLOT(paste()), Qt::CTRL+Qt::Key_V);
 	edit_menu->addAction("&Duplicate", plotList, SLOT(duplicate()), Qt::CTRL+Qt::Key_D);
	edit_menu->addSeparator();
 	edit_menu->addAction("Select &All", plotList, SLOT( selectAll() ),Qt::CTRL+Qt::Key_A );
 	edit_menu->addAction("Move Up", plotList, SLOT(moveUp()), Qt::CTRL+Qt::Key_Up);
 	edit_menu->addAction("Move Down", plotList, SLOT(moveDown()), Qt::CTRL+Qt::Key_Down);
	edit_menu->addSeparator();
 	
	QAction *delAct = edit_menu->addAction(style()->standardIcon(QStyle::SP_TrashIcon, 0, this), "Delete", plotList, SLOT(deleteSelectedItems()), Qt::Key_Delete);
	QList<QKeySequence> keyList;
    keyList.append(Qt::Key_Delete);
    keyList.append(Qt::Key_Backspace);
    delAct->setShortcuts(keyList);
   
	edit_menu->addSeparator();
	edit_menu->addAction("&Link Selected Traces", plotList, SLOT( linkSelected() ),Qt::CTRL+Qt::Key_L);
	edit_menu->addAction("&Unlink Selected Traces", plotList, SLOT( unlinkSelected() ),Qt::CTRL+Qt::Key_U);

	help_menu->clear();
	help_menu->addAction("QSAS &Manual", this, SLOT( launchManual() ), Qt::CTRL+Qt::Key_M );
	help_menu->addSeparator();
	help_menu->addAction("...Plotting", this, SLOT( launchManPlot() ));
	help_menu->addSeparator();
	help_menu->addAction("Module Version", this, SLOT( launchVersion() ));

	_status_bar = statusBar();
	_status_bar->showMessage(" ");

	createActions();
	createToolBars();

    connect(qsasWindow,  SIGNAL(updatePlotRequest()), this, SLOT(plotHidden()));  

	QHBoxLayout *buttons_layout = new QHBoxLayout();

	plotBtn = new QPushButton("Plot on Screen",_top_widget);
	plotBtn->setMinimumSize(plotBtn->sizeHint());
	plotBtn->setFocusPolicy(Qt::StrongFocus);
	connect(plotBtn, SIGNAL(clicked()), this, SLOT(doPlot()));
	buttons_layout->addWidget(plotBtn);
	plotBtn->setDefault(true);

    QLabel *space = new QLabel("   ");
    buttons_layout->addWidget(space);

    plotSelBtn = new QPushButton("Plot Selected pages",_top_widget);
    plotSelBtn->setMinimumSize(plotSelBtn->sizeHint());
    plotSelBtn->setFocusPolicy(Qt::StrongFocus);
    connect(plotSelBtn, SIGNAL(clicked()), plotList, SLOT(plotSelectedItems()));
    buttons_layout->addWidget(plotSelBtn);
    plotSelBtn->setDefault(false);
    #ifdef Q_OS_MACX 
        plotSelBtn->setToolTip("Plot only selected pages, CMD+P shortcut");
    #else 
        plotSelBtn->setToolTip("Plot only selected pages, CTRL+P shortcut");
    #endif
    
	QLabel *space2 = new QLabel("   ");
	buttons_layout->addWidget(space2);
	
	dismissBtn = new QPushButton( "Dismiss", _top_widget);
	dismissBtn->setMinimumSize(dismissBtn->sizeHint());
	dismissBtn->setFocusPolicy(Qt::StrongFocus);
	connect( dismissBtn, SIGNAL(clicked()), this, SLOT(close()) );
	//buttons_layout->insertSpacing(0, 50);
	buttons_layout->addWidget(dismissBtn);

	_top_gl->addLayout(buttons_layout,2,0);
	_top_gl->setRowStretch(0,1);
	_top_gl->setRowStretch(1,0);

 	PageItem *startPage = plotList->makePageItem();
 	startPage->showTabs();
	
//	plotList->tabWidget->adjustSize();
	
	int (*pPLPexitHandler)(const char *) = PLPexitHandler;
	plsexit(pPLPexitHandler );
	
	connect(plotList, SIGNAL(raiseWindow()), this, SLOT(bringToFront()));
	
	this->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding));


	_top_splitter->setStretchFactor(0, 1);
	_top_splitter->setStretchFactor(1, 10);
	scroll->setWidgetResizable(true);
	scroll->setWidget(plot_groupbox);
	
}

void QpWindow::toggleToolbar(){ 
    if(_file_tool_bar->isVisible()){
        _file_tool_bar->setVisible(false);
        toolBar->setText("Show Tool Bar");
    }
    else{
        _file_tool_bar->setVisible(true);
        toolBar->setText("Hide Tool Bar");
    }
    
}



void QpWindow::keyPressEvent(QKeyEvent * e )
{  
	if(QuSlotDebug())
	{
		printf("QpWindow::keyPressEvent \n");
		fflush(stdout);
	}

	// --- Trace color selection as key events after SEH ---
	if (e->key() == Qt::Key_1) plotList->fastSelect_SC1();
	if (e->key() == Qt::Key_2) plotList->fastSelect_SC2();
	if (e->key() == Qt::Key_3) plotList->fastSelect_SC3();
	if (e->key() == Qt::Key_4) plotList->fastSelect_SC4();
	if (e->key() == Qt::Key_X) plotList->fastSelectComponent_x();
	if (e->key() == Qt::Key_Y) plotList->fastSelectComponent_y();
	if (e->key() == Qt::Key_Z) plotList->fastSelectComponent_z();
	if (e->key() == Qt::Key_R) plotList->fastSelectComponent_r();
	if (e->key() == Qt::Key_Backslash) plotList->fastSelect_SolidLine();
	if (e->key() == Qt::Key_Minus) plotList->fastSelect_DashedLine();
	if (e->key() == Qt::Key_Period) plotList->fastSelect_DottedLine();
	// --- eo SEH -------------

	// use escape key to toggle default buttons
	if(e->key() == Qt::Key_Escape){
		if(   plotBtn->isDefault()) {
			plotBtn->setDefault(false);
			plotSelBtn->setDefault(true);
		}
		else if(plotSelBtn->isDefault())  {
			plotSelBtn->setDefault(false);
			dismissBtn->setDefault(true);
		}
		else if(dismissBtn->isDefault())  {
			dismissBtn->setDefault(false);
			plotBtn->setDefault(true);
		}
		return;
	}
	// use return or enter key to perform default
	if(e->key() == Qt::Key_Return ||  e->key() == Qt::Key_Enter) {
		if(plotBtn->isDefault()) doPlot();
		else if (plotSelBtn->isDefault()) plotList->plotSelectedItems();
		else if (dismissBtn->isDefault()) close();
		return;
	}
    
    // plot selected
	if (e->key() == Qt::Key_P) plotList->plotSelectedItems();

	// else pass on to list view 
	e->ignore(); // let selection change

}

void QpWindow::deleteAllItems()
{
	if(QuSlotDebug())
	{
		printf("QpWindow::deleteAllItems \n");
		fflush(stdout);
	}

	plotList->deleteAllItems();
}


// --------------------------------------

void QpWindow::resetDefaultBtn()
{  
	dismissBtn->setDefault(false);
	//printBtn->setDefault(false);
	plotBtn->setDefault(true);
}

// --------------------------------------

void QpWindow::doPlot()
{
    // slot invoked when Plot push button clicked
    if(QuSlotDebug())
    {
        printf("QpWindow::doPlot \n");
        fflush(stdout);
    }
    
    if(!plotList)
    {
        (void)QMessageBox::warning(this,"Print","No active listview");
        return;
    }
    
    if(plotList->topLevelItemCount() == 0)
    {
        (void)QMessageBox::warning(this,"Print","No pages to plot");
        return;
    }
    
    PageItem* page_item;
    
    plotList->_plot_page_list.clear();
    for(int i=0; i<plotList->topLevelItemCount(); ++i)
    {
        page_item=dynamic_cast<PageItem*>(plotList->topLevelItem(i));
        plotList->_plot_page_list.push_back(page_item);
    }
    
    plotList->plot();
    
}
// --------------------------------------

void QpWindow::plotHidden()
{
    // plot from batch, window not shown
        
    PageItem* page_item;
    
    plotList->_plot_page_list.clear();
    for(int i=0; i<plotList->topLevelItemCount(); ++i)
    {
        page_item=dynamic_cast<PageItem*>(plotList->topLevelItem(i));
        plotList->_plot_page_list.push_back(page_item);
    }
    
    plotList->plot(true);
    
}


// --------------------------------------

void QpWindow::launchManPlot()
{
	// slot invoked from Plotting Help menu sub item
	// launches browser showing Qsas Plot help docuumentation

	if(QuSlotDebug())
	{
		printf("QpWindow::launchManPlot \n");
		fflush(stdout);
	}

	qsasWindow->help_window->manEntry("QSAS_plotting.html");
}

void QpWindow::launchManual()
{
	// slot invoked from Qsas Manual Help menu sub item
	// launches browser showing Qsas help index

	if(QuSlotDebug())
	{
		printf("QpWindow::launchManual \n");
		fflush(stdout);
	}

	qsasWindow->help_window->manEntry("QSAS_help_index.html");
}

void QpWindow::launchVersion()
{
	// slot invoked from Module Version Help menu sub item
	// launches browser showing Qsas version numbers

	if(QuSlotDebug())
	{
		printf("QpWindow::launchVersion \n");
		fflush(stdout);
	}

	QString text;
	text  = QString("Version Numbers...\n");
	text += QString(QSAS_VERSION_ID);
	text += "\n Plotting  ";
	text += QString(QPUI_VERSION);
	QMessageBox::information( this, "QSAS information", text.toLatin1().data(), "OK" );
}

 void QpWindow::save()
 {

	if(QuSlotDebug())
 	{
		printf("QpWindow::save \n");
		fflush(stdout);
 	}

    if(_filename.isEmpty())
    {
       saveAsXML();
       return;
    }

	saveAsXML(_filename);
		
 }



void QpWindow::include()
{
  restoreXML(false);
} 


 void QpWindow::saveSession(QString folder)
 {
  if(folder.isEmpty()) return;

  // set session folder and save
     this->_filename = QString(folder.toLatin1().data());
     this->_filename += QString("/saveSet");
     this->_filename += this->SAVE_EXTENSION;

	if(this->SAVE_EXTENSION.contains(".qx_")) saveAsXML(this->_filename);
	else saveAsXML();


}


void QpWindow::restoreSession(QString folder){

	if ( QuSlotDebug() )
	{
		printf( "QpWindow::restoreSession \n");
		fflush(stdout);
	}
    
	if(folder.isEmpty()) return;
	// keep original name
	QString fileN  = QString(folder.toLatin1().data());
	fileN += QString("/saveSet.qx_plot");

	restoreXML(fileN, true);
 
}


void QpWindow::saveAsXML(QString path)
{
 	ofstream file;
	file.open(path.toLatin1().data(),ios::out);

	if(file.is_open())
	{
        _filename = path;
        
		QDomDocument doc("QSAS_plot_save");

		QDomElement root=doc.createElement("QSAS_Plot");
		doc.appendChild(root);
		string guiID;
		guiID=QSAS_VERSION_ID;

		QDomElement version=createTextXMLNode(doc, "QSAS_Version", guiID);
		root.appendChild(version);

		root.appendChild(plotList->toXMLElement(doc));
            
		QString xml = doc.toString();

		file << xml.toLatin1().data();
		file.close();
	}
}

void QpWindow::saveAsXML()
{
    QString path=QFileDialog::getSaveFileName(this, "Save as...", QString(), "QSAS plot save (*.qx_plot);; XML files (*.xml)");
    if(!path.isEmpty())
    {
		if(!path.contains(".qx_plot") && !path.contains(".xml")){
			path += ".qx_plot";
		}
       saveAsXML(path);
    } 
}

void QpWindow::restoreXML(QString path, bool replace)
{
	if( !QFile::exists(path) ) return;

    _filename = path;
    
	QDomDocument doc;
	QFile myFile(path);
	doc.setContent(&myFile);
	myFile.close();

	QDomElement docElem = doc.documentElement();
	if(!docElem.isNull())
	{
		if(replace) deleteAllItems();
		qApp->processEvents();
		
		string guiID="0.0.00";

		QDomElement version=docElem.firstChildElement("QSAS_Version");
		if(!version.isNull())
		{
			guiID=version.firstChild().nodeValue().toLatin1().data();
		}
		setRestoreVersion(guiID.c_str());


		plotList->blockSignals(true);
		QDomElement ListDoc=docElem.firstChildElement("Plot_Layout");

		if(!ListDoc.isNull())
		{
			plotList->fromXMLElement(ListDoc, replace);
			PageItem* pageOne=dynamic_cast<PageItem*>(plotList->topLevelItem(0));
			if(pageOne)
			{
				plotList->setCurrentItem(pageOne);
				plotList->updateOnSelection();
				plotList->expandAll();
			}
		}
		plotList->blockSignals(false);
		plotList->setUniformRowHeights(true);
	}

	return;
}


void QpWindow::restoreXML(bool replace)
{
    QString path=QFileDialog::getOpenFileName(this, "Open...", QString(), "QSAS join save (*.qx_plot);; XML files (*.xml);; All files (*)");
    if(!path.isEmpty())
    {
		restoreXML(path, replace);
	}
}


void QpWindow::refreshColourTableButtons()
{
    emit refreshColourTableButtonsSignal();
}

// ---------------------------------------------------
void QpWindow::createToolBars()
{
  _file_tool_bar = addToolBar("File");
  _file_tool_bar->setIconSize(QSize(24, 24));
  _file_tool_bar->addAction(restoreAct);
  _file_tool_bar->addAction(saveAsAct);	

  _file_tool_bar->insertSeparator(newPageAct);  
  _file_tool_bar->addAction(newTraceAct);
  _file_tool_bar->addAction(newSpecAct);
  _file_tool_bar->addAction(newSubAct);
  _file_tool_bar->addAction(newTSAct);
  _file_tool_bar->addAction(newXYAct);
  _file_tool_bar->addAction(newPageAct);
  _file_tool_bar->addAction(eventAct);

  _file_tool_bar->insertSeparator(deleteAct);  
  _file_tool_bar->addAction(cutAct);
  _file_tool_bar->addAction(copyAct);
  _file_tool_bar->addAction(pasteAct);
  _file_tool_bar->addAction(deleteAct);


  _file_tool_bar->insertSeparator(helpAct);  
  _file_tool_bar->addAction(helpAct);
  _file_tool_bar->setVisible(false);
  if(QSAS_profiles->current()->ShowToolbar) toggleToolbar();
  
}


// ------ SEH ----------------------------------------
/*!
 \brief creates actions
*/
// ---------------------------------------------------
void QpWindow::createActions()
{
    QIcon spectIcon = QIcon(QPixmap(spectrum_xpm));
    QIcon eventIcon = QIcon(QPixmap(event_xpm));
    QIcon traceIcon = QIcon(QPixmap(trace_xpm));
    
   char *qsas_home_env = getenv("QSAS_HOME"); 
   QString qsas_home=NULL;
   if( qsas_home_env != NULL) {
     qsas_home = QString(qsas_home_env);
     qsas_home = qsas_home.remove(QString("\""));
   } else qsas_home="";

   restoreAct = new QAction(QIcon(qsas_home+"/icons/reload.png"), tr("Restore"), this);
   connect(restoreAct, SIGNAL(triggered()), this, SLOT(restoreXML()));
   
   saveAsAct = new QAction(QIcon(qsas_home+"/icons/filesaveas.png"), tr("Save"), this);
   connect(saveAsAct, SIGNAL(triggered()), this, SLOT(save()));
   
   deleteAct = new QAction(style()->standardIcon(QStyle::SP_TrashIcon, 0, this), tr("Delete item"), this);
   connect(deleteAct, SIGNAL(triggered()), plotList,  SLOT(deleteSelectedItems()));
   
   cutAct = new QAction(QIcon(qsas_home+"/icons/editcut.png"), tr("Cut item"), this);
   connect(cutAct, SIGNAL(triggered()), plotList,  SLOT(cut()));
   cutAct->setEnabled(false);
   
   copyAct = new QAction(QIcon(qsas_home+"/icons/editcopy.png"), tr("Copy item"), this);
   connect(copyAct, SIGNAL(triggered()), plotList,  SLOT(copy()));
   
   pasteAct = new QAction(QIcon(qsas_home+"/icons/editpaste.png"), tr("Paste item"), this);
   connect(pasteAct, SIGNAL(triggered()), plotList,  SLOT(paste()));
   
   newPageAct = new QAction(QIcon(qsas_home+"/icons/filenew.png"), tr("New page"), this);
   connect(newPageAct, SIGNAL(triggered()), plotList, SLOT(makePageItem()));
   
   newTSAct = new QAction(QIcon(qsas_home+"/icons/newtsplot2.png"), tr("New TS frame"), this);
   connect(newTSAct, SIGNAL(triggered()), plotList, SLOT(makeTSFrameItem()));
   
   newXYAct = new QAction(QIcon(qsas_home+"/icons/newxyplot2.png"), tr("New XY frame"), this);
   connect(newXYAct, SIGNAL(triggered()), plotList, SLOT(makeXYFrameItem()));
   
    newSubAct = new QAction(QIcon(qsas_home+"/icons/newtssubplot2.png"), tr("New subpanel"), this);
    connect(newSubAct, SIGNAL(triggered()), plotList, SLOT(makePanelItem()));
   
   newTraceAct = new QAction(traceIcon, tr("New trace"), this);
   connect(newTraceAct, SIGNAL(triggered()), plotList, SLOT(makeTraceItem()));
   
   newSpecAct = new QAction(spectIcon, tr("New spectrogram"), this);
   connect(newSpecAct, SIGNAL(triggered()), plotList, SLOT(makeSpectrumItem()));
   
   eventAct = new QAction(eventIcon, tr("New Event marker"), this);
   connect(eventAct, SIGNAL(triggered()), plotList, SLOT(makeEventItem()));
   
    helpAct = new QAction(QIcon(qsas_home+"/icons/help.png"), tr("Help"), this);
   connect(helpAct, SIGNAL(triggered()), this, SLOT( launchManPlot()));

}

