#include "ColourTable.h"

ColourTablePalette stdPalette;
ColourTablePalette profilePalette;
ColourTablePalette importedPalette;
ColourTablePalette savedPalette;// Used to reference and save used and non-standard colour tables

ColourTable::ColourTable(const QString& name)
{
    this->name=name;
    i=NULL;
    h=NULL;
    s=NULL;
    v=NULL;

    setPresetRainbow();
}

ColourTable::ColourTable(const QString& name, const QVector<double>& points, const QVector<double>& H, const QVector<double>& S, const QVector<double>& V, bool isRGB)
{
    this->name=name;
    size=points.size();
    if(size==0 || size!=H.size() || size!=S.size() || size!=V.size())
    {
        i=NULL;
        h=NULL;
        s=NULL;
        v=NULL;
        setPresetRainbow();
        return;
    }
    
    i=new PLFLT[size];
    h=new PLFLT[size];
    s=new PLFLT[size];
    v=new PLFLT[size];

    for(int j=0; j<size; ++j)
    {
        i[j]=points[j];
        h[j]=H[j];
        s[j]=S[j];
        v[j]=V[j];
    }

    this->isRGB=isRGB;

    updateIcon();
}

ColourTable::ColourTable(const ColourTable& table)
{
    isRGB=table.isRGB;
    name=table.name;
    size=table.size;

    if(size!=0)
    {
        i=new PLFLT[size];
        h=new PLFLT[size];
        s=new PLFLT[size];
        v=new PLFLT[size];

        for(int j=0; j<size; ++j)
        {
            i[j]=table.i[j];
            h[j]=table.h[j];
            s[j]=table.s[j];
            v[j]=table.v[j];
        }
    }
    else
    {
        i=NULL;
        h=NULL;
        s=NULL;
        v=NULL;
        setPresetRainbow();
    }

    updateIcon();
}

ColourTable::~ColourTable()
{
    clear();
}

void ColourTable::operator=(const ColourTable& table)
{
    clear();
    name=table.name;
    size=table.size;
    isRGB=table.isRGB;

    i=new PLFLT[size];
    h=new PLFLT[size];
    s=new PLFLT[size];
    v=new PLFLT[size];
    
    for(int j=0; j<size; ++j)
    {
        i[j]=table.i[j];
        h[j]=table.h[j];
        s[j]=table.s[j];
        v[j]=table.v[j];
    }

    updateIcon();
}

void ColourTable::clear()
{
    size=0;
    if(i!=NULL) delete[] i;
    if(h!=NULL) delete[] h;
    if(s!=NULL) delete[] s;
    if(v!=NULL) delete[] v;
}

void ColourTable::setPresetRainbow()
{
    clear();
    isRGB=0;
    h=new PLFLT[7]; h[0]=300.; h[1]=250;h[2]=200.; h[3]=150.; h[4]=95; h[5]=40.; h[6]=-10.;
    s=new PLFLT[7]; s[0]=1.; s[1]=1.; s[2]=1.; s[3]=1.; s[4]=1.; s[5]=1.; s[6]=1.;
    v=new PLFLT[7]; v[0]=.5; v[1]=.5; v[2]=.5; v[3]=.5; v[4]=.5; v[5]=.5; v[6]=.5; 

    i=new PLFLT[7]; i[0]=0.; i[1]=0.17; i[2]=0.34; i[3]=.5; i[4]=0.66; i[5]=0.83; i[6]=1.; 
    size=7;

    updateIcon();
}
/*
void ColourTable::setPresetRainbow()
{
    clear();
    isRGB=0;
    h=new PLFLT[2]; h[0]=300.; h[1]=-10.;
    s=new PLFLT[2]; s[0]=1.; s[1]=1.;
    v=new PLFLT[2]; v[0]=.5; v[1]=.5;

    i=new PLFLT[2]; i[0]=0.; i[1]=1.;
    size=2;

    updateIcon();
}
*/
void ColourTable::setPresetRainbowExt()
{
    clear();
    isRGB=0;

    h=new PLFLT[7]; h[0]=240.; h[1]=240.; h[2]=178.; h[3]=115.; h[4]=52.; h[5]=-10.; h[6]=-10.;
    s=new PLFLT[7]; s[0]=1.; s[1]=1.; s[2]=1.; s[3]=1.;  s[4]=1.; s[5]=1.; s[6]=1.;
    v=new PLFLT[7]; v[0]=1.; v[1]=.5; v[2]=.5; v[3]=.5; v[4]=.5; v[5]=.5; v[6]=0.;
            
    i=new PLFLT[7]; i[0]=0.; i[1]=0.1; i[2]=0.3; i[3]=.5; i[4]=0.7; i[5]=0.9; i[6]=1.;
    size=7;

    updateIcon();
}

void ColourTable::setPresetSpectrum()
{
    clear();
    isRGB=0;

    h=new PLFLT[7]; h[0]=240.; h[1]=240.; h[2]=178.; h[3]=115.; h[4]=52.; h[5]=-10.; h[6]=-10.;
    s=new PLFLT[7]; s[0]=1.; s[1]=1.; s[2]=1.; s[3]=1.;  s[4]=1.; s[5]=1.; s[6]=1.;
    v=new PLFLT[7]; v[0]=.95; v[1]=.5; v[2]=.5; v[3]=.5; v[4]=.5; v[5]=.5; v[6]=0.4;
            
    i=new PLFLT[7]; i[0]=0.; i[1]=0.1; i[2]=0.3; i[3]=.5; i[4]=0.7; i[5]=0.9; i[6]=1.;
    size=7;

    updateIcon();
}

void ColourTable::setPresetSpectrumDark()
{
    clear();
    isRGB=0;

    h=new PLFLT[7]; h[0]=240.; h[1]=240.; h[2]=178.; h[3]=115.; h[4]=52.; h[5]=-10.; h[6]=-10.;
    s=new PLFLT[7]; s[0]=1.; s[1]=1.; s[2]=1.; s[3]=1.;  s[4]=1.; s[5]=1.; s[6]=1.;
    v=new PLFLT[7]; v[0]=.1; v[1]=.5; v[2]=.5; v[3]=.5; v[4]=.5; v[5]=.5; v[6]=0.4;
            
    i=new PLFLT[7]; i[0]=0.; i[1]=0.1; i[2]=0.3; i[3]=.5; i[4]=0.7; i[5]=0.9; i[6]=1.;
    size=7;

    updateIcon();
}

void ColourTable::setPresetGrayWhiteHigh()
{
    clear();
    isRGB=0;
    h=new PLFLT[5]; h[0]=0.; h[1]=0.; h[2]=0.; h[3]=0.; h[4]=0.; 
    s=new PLFLT[5]; s[0]=0.; s[1]=0.; s[2]=0.; s[3]=0.; s[4]=0.; 
    v=new PLFLT[5]; v[0]=0.; v[1]=0.25; v[2]=0.5; v[3]=0.75; v[4]=1.; 
            
    i=new PLFLT[5]; i[0]=0.; i[1]=0.25; i[2]=0.5; i[3]=0.75; i[4]=1.;
    size=5;

    updateIcon();
}

void ColourTable::setPresetGrayWhiteLow()
{
    clear();
    isRGB=0;
    h=new PLFLT[5]; h[0]=0.; h[1]=0.; h[2]=0.; h[3]=0.; h[4]=0.;
    s=new PLFLT[5]; s[0]=0.; s[1]=0.; s[2]=0.; s[3]=0.; s[4]=0.; 
    v=new PLFLT[5]; v[0]=1.; v[1]=0.75; v[2]=0.5; v[3]=0.25; v[4]=0.;
            
    i=new PLFLT[5]; i[0]=0.; i[1]=0.25; i[2]=0.5; i[3]=0.75; i[4]=1.;
    size=5;

    updateIcon();
}

void ColourTable::setPresetInvRedBlue()
{
    clear();
    isRGB=0;
    h=new PLFLT[6]; h[0]=230.; h[1]=230.; h[2]=230.; h[3]=-10.; h[4]=-10.; h[5]=-10.;
    s=new PLFLT[6]; s[0]=1.; s[1]=1.; s[2]=1.; s[3]=1.; s[4]=1.; s[5]=1.;
    v=new PLFLT[6]; v[0]=.9; v[1]=.5; v[2]=.1; v[3]=.1; v[4]=.5; v[5]=.9;
            
    i=new PLFLT[6]; i[0]=0.; i[1]=0.25; i[2]=0.499; i[3]=.5; i[4]=0.75; i[5]=1.;
    size=6;

    updateIcon();
}

void ColourTable::setPresetRedBlue()
{
    clear();
    isRGB=0;
    h=new PLFLT[6]; h[0]=230.; h[1]=230.; h[2]=230; h[3]=-10.; h[4]=-10.; h[5]=-10.;
    s=new PLFLT[6]; s[0]=1.; s[1]=1.; s[2]=1.; s[3]=1.; s[4]=1.; s[5]=1.;
    v=new PLFLT[6]; v[0]=.1; v[1]=.5; v[2]=.9; v[3]=.9; v[4]=.5; v[5]=.1;
            
    i=new PLFLT[6]; i[0]=0.; i[1]=0.25; i[2]=0.499; i[3]=0.5; i[4]=0.75; i[5]=1.;
    size=6;

    updateIcon();
}

void ColourTable::setPresetEarth()
{
    clear();
    isRGB=0;
    h=new PLFLT[5]; h[0]=50.; h[1]=50.; h[2]=50.; h[3]=50.; h[4]=50.;
    s=new PLFLT[5]; s[0]=0.2; s[1]=.4; s[2]=0.6; s[3]=.8; s[4]=1.; 
    v=new PLFLT[5]; v[0]=0.9; v[1]=0.675; v[2]=0.45; v[3]=0.225; v[4]=0.;
            
    i=new PLFLT[5]; i[0]=0.; i[1]=0.25; i[2]=0.5; i[3]=0.75; i[4]=1.;
    size=5;

    updateIcon();
}

void ColourTable::fromString(const QString& _str)
{
    QStringList sl, sl2;
    name="Unnamed";
    sl=_str.split("___");
    QString str;
    if(sl.size()==2)
    {
        name=sl[0];
        name.replace("__", " ");
        str=sl[1].trimmed();
    }
    else
    {
        str=sl[0].trimmed();
    }


    if(str.isEmpty())
    {
        clear();
        updateIcon();
        return;
    }

    sl=str.split(QRegExp("\\(|\\)"), QString::SkipEmptyParts);
    if(sl.size()<3)
    {
        clear();
        updateIcon();
        return;
    }
    if(sl[0].contains("RGB", Qt::CaseInsensitive))
    {
        isRGB=1;
    }
    else if(sl[0].contains("HSV", Qt::CaseInsensitive))
    {
        isRGB=0;
    }

    else
    {
        clear();
        updateIcon();
        return;
    }
    size=sl.size()-1;
    if(size==0)
    {
//         h=NULL;
//         s=NULL;
//         v=NULL;
//         i=NULL;
//         updateIcon();
        setPresetRainbow();
        return;
    }
    
    h=new PLFLT[size];
    s=new PLFLT[size];
    v=new PLFLT[size];
    i=new PLFLT[size];
    for(int index=0; index<size; ++index)
    {
        sl2=(sl[index+1]).split(",",  QString::SkipEmptyParts, Qt::CaseInsensitive);

        if(sl2.size()!=4)
        {
            h[index]=0.; s[index]=0.; v[index]=0.; i[index]=0.;
        }
        else
        {
            i[index]=sl2[0].toDouble();
            h[index]=sl2[1].toDouble();
            s[index]=sl2[2].toDouble();
            v[index]=sl2[3].toDouble();
        }
    }
    updateIcon();
}

QString ColourTable::toString() const
{
    QString encodedName=name;
    encodedName.replace(" ", "__");
    QString res(QString("%1___%2").arg(encodedName).arg(isRGB==1 ? "RGB" : "HSV"));
    for(int j=0; j<size; ++j)
    {
        res.append(QString("(%1,%2,%3,%4)").arg(i[j]).arg(h[j]).arg(s[j]).arg(v[j]));
    }
    return res;
}

void ColourTable::apply(double alpha) const
{
	if(alpha > 0.99){
   		if(isRGB)
    	{
       		plscmap1l(1, size, i, h, s, v, NULL);
    	}
    	else
    	{
        	plscmap1l(0, size, i, h, v, s, NULL);
    	}
	}
	else {
		PLFLT a[size];
		for (int k=0; k< size; k++) {
			a[k] = alpha + k * (1.-alpha) / (size - 1.);
			if(a[k] < 0.) a[k] = 0.0;
		}
   		if(isRGB)
    	{
       		plscmap1la(1, size, i, h, s, v, a, NULL);
    	}
    	else
    	{
        	plscmap1la(0, size, i, h, v, s, a, NULL);
    	}
		
	}

}

bool ColourTable::operator==(const ColourTable& table) const
{
    if(isRGB!=table.isRGB)
        return false;

    if(size!=table.size)
        return false;

    if(name!=table.name)
        return false;

    for(int j=0; j<size; ++j)
    {
        if(i[j]!=table.i[j]) return false;
        if(h[j]!=table.h[j]) return false;
        if(s[j]!=table.s[j]) return false;
        if(v[j]!=table.v[j]) return false;
    }

    return true;
}

bool ColourTable::operator!=(const ColourTable& table) const
{
    return !((*this)==table);
}

extern void c_plhlsrgb(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b); // in plctrl.c
extern void c_plrgbhls(PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l, PLFLT *p_s);

void ColourTable::drawColourBar(QPainter* painter, int width, int height)
{
    QBrush background(Qt::white);
    painter->setBrush(background);

    QPen pen(QColor(160, 160, 160));
    pen.setWidth(1);
    painter->setPen(pen);
    painter->drawRect(0, 0, width-1, height-1);

    for(int j=0; j<height; ++j)
    {
        double val=((double)j)*1./(height-1.);

        // find the index of the nearest neighbour

        int indexBelow=-1;
        int indexAbove=-1;
        double distBelow=2.;
        double distAbove=2.;

        for(int k=0; k<size; ++k)
        {
            if(i[k]>val)
            {
                if(i[k]-val<distAbove)
                {
                    distAbove=i[k]-val;
                    indexAbove=k;
                }
            }
            else
            {
                if(val-i[k]<distBelow)
                {
                    distBelow=val-i[k];
                    indexBelow=k;
                }
            }
        }

        // get the colour code for "index"

        if(isRGB)
        {
            PLFLT R, G, B;
            PLFLT H1, H2, S1, S2, V1, V2, H, S, V;
            if(indexBelow==-1)
            {
                R=h[indexBelow];
                G=s[indexBelow];
                B=v[indexBelow];
//                 c_plrgbhls(R, G, B, &H, &V, &S);
//                 R=h[indexBelow]*255;
//                 G=s[indexBelow]*255;
//                 B=v[indexBelow]*255;
            }
            else if(indexAbove==-1)
            {
                R=h[indexAbove]/**255*/;
                G=s[indexAbove]/**255*/;
                B=v[indexAbove]/**255*/;
//                 c_plrgbhls(R, G, B, &H, &V, &S);
            }
            else
            {
                double fact=(val-i[indexBelow])/(i[indexAbove]-i[indexBelow]);
                R=h[indexBelow];
                G=s[indexBelow];
                B=v[indexBelow];
                c_plrgbhls(R, G, B, &H1, &V1, &S1);
                R=h[indexAbove];
                G=s[indexAbove];
                B=v[indexAbove];
                c_plrgbhls(R, G, B, &H2, &V2, &S2);
                
                H=(H1+(H2-H1)*fact);
                S=(S1+(S2-S1)*fact);
                V=(V1+(V2-V1)*fact);

                c_plhlsrgb(H, V, S, &R, &G, &B);
            }
            
            QColor color=QColor::fromRgb((int)(R*255.), (int)(G*255.), (int)(B*255.));
            pen.setColor(color);
        }
        else
        {
            PLFLT H, S, V;
            if(indexAbove==-1 && indexBelow>=0)
            {
                H=h[indexBelow];
                S=s[indexBelow]/**255*/;
                V=v[indexBelow]/**255*/;
            }
            else if(indexBelow==-1 && indexAbove>=0)
            {
                H=h[indexAbove];
                S=s[indexAbove]/**255*/;
                V=v[indexAbove]/**255*/;
            }
            else
            {
                double fact=(val-i[indexBelow])/(i[indexAbove]-i[indexBelow]);
                H=(h[indexBelow]+(h[indexAbove]-h[indexBelow])*fact);
                S=(s[indexBelow]+(s[indexAbove]-s[indexBelow])*fact)/**255*/;
                V=(v[indexBelow]+(v[indexAbove]-v[indexBelow])*fact)/**255*/;
            }
            while(H>360) h-=360;
            while(H<0) H+=360;
            PLFLT R, G, B;
            c_plhlsrgb(H, V, S, &R, &G, &B);
            QColor color((int)(R*255.), (int)(G*255.), (int)(B*255.));
            pen.setColor(color);
        }

        painter->setPen(pen);
        painter->drawLine(1, height-2-j, width-2, height-2-j);
    }
}

void ColourTable::updateIcon()
{
    if(size==0 || h==NULL || s==NULL || v==NULL || i==NULL)
        return;

    QImage img(12, 16, QImage::Format_RGB32);

    QPainter* painter=new QPainter(&img);

    drawColourBar(painter, img.width(), img.height());
    
    icon=QIcon(QPixmap::fromImage(img));

    delete painter;
}

//Writes log ticks into vector after accepting the minimum and maximum logged value of the scale.
void logticks(PLFLT min, PLFLT max, std::vector<PLFLT> &ticks)
{
	int pow_of_min = (int)floor(min);
	PLFLT value = min;

	for(int i=0; i<30; i++) //30 just for protecting ourselves from freezing.
	{
		for(int j=1; j<10;j++)
		{
			value = pow_of_min + i + SafeLog10(j);
			ticks.push_back(value);
			if(value>max)
				break;
		}
		if(value>max)
			break;
	}
}

PLFLT arrmax(PLFLT a[], int size) {
	
	PLFLT maxval=1e-30;
	if(!a) return maxval;

	for (int i = 0; i < size; i++) {
		if (a[i] > maxval && isfinite(a[i]) )
			maxval = a[i];
	}
	return maxval;
}

PLFLT arrmin(PLFLT a[], int size) {
	
	PLFLT minval=1e30;
	if(!a) return minval;

	for (int i = 0; i < size; i++) {
		if (a[i] < minval && isfinite(a[i]) ){
			minval = a[i];
		}
	}
	return minval;
}



/***********************************************************************/

QColor QuGetColour(QColor inC, bool *newChoice, QWidget * parent){

  *newChoice = false;
  static bool first = true;
  
  if(first)
  {
	int Ncustom = QColorDialog::customCount();
	if (Ncustom >= 16) 
	{
		QColorDialog::setCustomColor(0, QColor(255, 255, 255).rgb());
		QColorDialog::setCustomColor(1, QColor(0, 0, 0).rgb());
		QColorDialog::setCustomColor(2, QColor(255, 0, 0).rgb());
		QColorDialog::setCustomColor(3, QColor(0, 255, 0).rgb());
		QColorDialog::setCustomColor(4, QColor(0, 0, 255).rgb());
		QColorDialog::setCustomColor(5, QColor(0, 255, 255).rgb());
		QColorDialog::setCustomColor(6, QColor(255, 0, 255).rgb());
		QColorDialog::setCustomColor(7, QColor(255, 255, 0).rgb());
		QColorDialog::setCustomColor(8, QColor(255, 128, 0).rgb());
		QColorDialog::setCustomColor(9, QColor(128, 255, 0).rgb());
		QColorDialog::setCustomColor(10, QColor(0, 255, 128).rgb());
		QColorDialog::setCustomColor(11, QColor(0, 128, 255).rgb());
		QColorDialog::setCustomColor(12, QColor(128, 0, 255).rgb());
		QColorDialog::setCustomColor(13, QColor(255, 0, 128).rgb());
		QColorDialog::setCustomColor(14, QColor(85, 85, 85).rgb());
		QColorDialog::setCustomColor(15, QColor(170, 170, 170).rgb());
	}
	
	first = false;
  }

  QColor c=QColorDialog::getColor(inC, parent);
  if(c.isValid() && c != inC){
    // 
    if(newChoice) *newChoice = true;  // ensure valid bool pointer newChoice was passed in before setting it
    return c;
  }
  else{
    if(newChoice) *newChoice = false;
    return inC;
  }


} // end QuGetColour

