#include "plDevs.h"

#ifdef PLD_qt

#include "plplotP.h"
#include "drivers.h"

#include "QtPLWidget.h"


#define HANDLES_TRUETYPE 1

void QtPLWidget::setNewSaveSize(double i_dSaveWidth, double i_dSaveHeight)
{
	double aspectRatio=i_dSaveWidth/i_dSaveHeight;
	if(aspectRatio>1.) //width/height > 1.
	{
		m_dVirtualWidth=PIXELS_X;
		m_dVirtualHeight=PIXELS_Y;//(PLINT)(PIXELS_Y/aspectRatio);
	}
	else
	{
		m_dVirtualHeight=PIXELS_Y;
		m_dVirtualWidth=PIXELS_X;//(PLINT)(PIXELS_X*aspectRatio);
	}

	m_dAspectRatio=aspectRatio;
	m_dSaveWidth=i_dSaveWidth;
	m_dSaveHeight=i_dSaveHeight;

    if((PLINT)m_dVirtualWidth>0 && (PLINT)m_dVirtualHeight>0)
	    plP_setphy (0, (PLINT)m_dVirtualWidth, 0, (PLINT)m_dVirtualHeight);
    else
        plP_setphy (0, 1, 0, 1); // error
}

void plsetqtdev(void *plotdev)
{
    PLINT w, h;
    plsc->dev = plotdev;
    if((PLINT)((QtPLWidget*)plotdev)->getVirtualWidth()>0 && (PLINT)((QtPLWidget*)plotdev)->getVirtualHeight()>0)
        plP_setphy (0, (PLINT)((QtPLWidget*)plotdev)->getVirtualWidth(), 0, (PLINT)((QtPLWidget*)plotdev)->getVirtualHeight());
    else
        plP_setphy (0, 1, 0, 1); // error
}

const char* plD_DEVICE_INFO_qt="qt:qt Widget device:-1:qt:99:qt";

void plD_dispatch_init_qt(PLDispatchTable *pdt);

void plD_init_qt(PLStream *);
void plD_line_qt(PLStream *, short, short, short, short);
void plD_polyline_qt(PLStream *, short*, short*, PLINT);
void plD_eop_qt(PLStream *);
void plD_bop_qt(PLStream *){}
void plD_tidy_qt(PLStream *){}
void plD_state_qt(PLStream *, PLINT);
void plD_esc_qt(PLStream *, PLINT, void*);

void plD_dispatch_init_qt(PLDispatchTable *pdt)
{
#ifndef ENABLE_DYNDRIVERS
    pdt->pl_MenuStr  = "Qt Widget";
    pdt->pl_DevName  = "qt";
#endif
    pdt->pl_type     = plDevType_Null;
    pdt->pl_seq      = 99;
    pdt->pl_init     = (plD_init_fp)     plD_init_qt;
    pdt->pl_line     = (plD_line_fp)     plD_line_qt;
    pdt->pl_polyline = (plD_polyline_fp) plD_polyline_qt;
    pdt->pl_eop      = (plD_eop_fp)      plD_eop_qt;
    pdt->pl_bop      = (plD_bop_fp)      plD_bop_qt;
    pdt->pl_tidy     = (plD_tidy_fp)     plD_tidy_qt;
    pdt->pl_state    = (plD_state_fp)    plD_state_qt;
    pdt->pl_esc      = (plD_esc_fp)      plD_esc_qt;
}

void plD_init_qt(PLStream * pls)
{
    if ((pls->phyxma == 0) || (pls->dev == NULL)) {
        plexit("Must call plsetqtdev first to set user plotting widget!");
    }
    plP_setpxl((PLFLT) 4, (PLFLT) 4); /* rough pixels/mm on *my* screen */
    pls->color=1; // Color device
    pls->plbuf_write=0;
    pls->dev_fill0=1; // handles solid fills
    pls->dev_fill1=0;
    pls->dev_dash=0;
    pls->dev_flush=1;
    pls->dev_clear=1;
    pls->dev_text=HANDLES_TRUETYPE;
    pls->dev_unicode=HANDLES_TRUETYPE;
    
}

void plD_line_qt(PLStream * pls, short x1a, short y1a, short x2a, short y2a)
{
    QtPLWidget * widget=(QtPLWidget *) pls->dev;
    widget->setColor(pls->curcolor.r, pls->curcolor.g, pls->curcolor.b, pls->curcolor.a);
    widget->drawLine(x1a, y1a, x2a, y2a); 
}

void plD_polyline_qt(PLStream *pls, short *xa, short *ya, PLINT npts)
{
    QtPLWidget * widget=(QtPLWidget *) pls->dev;
    widget->setColor(pls->curcolor.r, pls->curcolor.g, pls->curcolor.b, pls->curcolor.a);
    widget->drawPolyline(xa, ya, npts);
}

void plD_eop_qt(PLStream *pls)
{
  /* Set the 'dev' member (which holds the user supplied memory image)
    to NULL here so it won't be freed when PLplot is closed.
    (the user is responsible for freeing it when ready).
  */
    ((QtPLWidget *)pls->dev)->clearWidget();
//     pls->dev = NULL;
}

void plD_esc_qt(PLStream * pls, PLINT op, void* ptr)
{
//     short *xa, *ya;
//     PLINT i, j;
    QtPLWidget * widget=(QtPLWidget *) pls->dev;
//     PLBufferingCB * buf_ptr;
		    
    switch(op)
    {
// 	case PLESC_DASH:
// 	    widget->setDashed(pls->nms, pls->mark, pls->space);
// 	    widget->setColor(pls->curcolor.r, pls->curcolor.g, pls->curcolor.b, pls->curcolor.a);
// 	    widget->drawPolyline(pls->dev_x, pls->dev_y, pls->dev_npts);
// 	    widget->setSolid();
// 	    break;
		    
        case PLESC_FILL:
	    widget->setColor(pls->curcolor.r, pls->curcolor.g, pls->curcolor.b, pls->curcolor.a);
	    widget->drawPolygon(pls->dev_x, pls->dev_y, pls->dev_npts);
            break;

	case PLESC_HAS_TEXT:
		    widget->setColor(pls->curcolor.r, pls->curcolor.g, pls->curcolor.b, pls->curcolor.a);
			widget->drawText(pls, (EscText*)ptr);
		    break;
	    
        default: /*std::cout << "ESC " << op << std::endl;*/break;
	
    }
}

void plD_state_qt(PLStream * pls, PLINT op)
{
    QtPLWidget * widget=(QtPLWidget *) pls->dev;
    
    switch(op)
    {
        case PLSTATE_WIDTH:
            widget->setWidth(pls->width);
            break;
       
        case PLSTATE_COLOR1:
		widget->setColor(pls->curcolor.r, pls->curcolor.g, pls->curcolor.b, pls->curcolor.a);
            break;

            
            default: /*std::cout << "STATE " << op << std::endl;*/break;
    }
}

#endif
